import zero.core.context.GlobalContext;
import com.ibm.websphere.rainmaker.logging.Output;
import com.ibm.ws.xs.stats.client.StatsDescriptorUtil;
import xsa.wxs.context.GlobalContextURIs;
import com.ibm.ws.xs.stats.StatsUtil;
import com.ibm.ws.xs.stats.client.StatsHelper;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.ws.xs.app.DomainConnectionUpdater;

class StatInfo {
    // levels of stats.
    static JVM = 0x1
    static GRID = 0x2
    static MAP = 0x4
    static ZONE = 0x8

    static MS = "ms"
    static TS = "trans/sec"
    static CN = "cardinal"
    static PC = "percentage"
    static B  = "bytes"
    static MB = "MB"

    static supportedStats = ["X": 0]
    static units = [
        (StatsDescriptorUtil.GRID_AVG_TRAN_TIME): MS,
        (StatsDescriptorUtil.GRID_AVG_TP): TS,
        (StatsDescriptorUtil.GRID_MAX_TIME): MS,
        (StatsDescriptorUtil.GRID_MIN_TIME): MS,
        (StatsDescriptorUtil.GRID_TOTAL_TIME): MS,
        (StatsDescriptorUtil.MAP_NUM_ENTRIES): CN,
        (StatsDescriptorUtil.MAP_NUM_ENTRIES_MAX):CN,
        (StatsDescriptorUtil.MAP_NUM_ENTRIES_MIN):CN,
        (StatsDescriptorUtil.MAP_HIT_RATE):PC,
        (StatsDescriptorUtil.MAP_USEDBYTES):B,
        (StatsDescriptorUtil.MAP_USEDBYTES_MIN):B,
        (StatsDescriptorUtil.MAP_USEDBYTES_MAX):B,
        (StatsDescriptorUtil.MAP_TOTALHITS):CN,
        (StatsDescriptorUtil.MAP_TOTALGETS):CN,
        (StatsDescriptorUtil.JVM_FREE_MEM):MB,
        (StatsDescriptorUtil.JVM_TOTAL_MEM):MB,
        (StatsDescriptorUtil.JVM_AVAIL_PROCS):CN,
        (StatsDescriptorUtil.JVM_MAX_MEM): MB,
        (StatsDescriptorUtil.JVM_USED_MEM): MB
        
    ]
    static initialized = false;

    static init() {
        if (!initialized) {
            StatsDescriptorUtil.GRID_STATS_LIST.each {
                supportedStats[it] = JVM|GRID
            }
            
            StatsDescriptorUtil.MAP_STATS_LIST.each {
                supportedStats[it] = JVM|MAP
            }
            
            StatsDescriptorUtil.JVM_STATS_LIST.each {
                supportedStats[it] = JVM|MAP;
            }

            // Stat used in calculations, not exposed to UI.
            supportedStats.remove(StatsDescriptorUtil.GRID_COUNT)
            supportedStats.remove(StatsDescriptorUtil.GRID_ZONES)
            supportedStats.remove(StatsDescriptorUtil.MAP_NUM_ENTRIES_MAX)
            supportedStats.remove(StatsDescriptorUtil.MAP_NUM_ENTRIES_MIN)

            // Remove capacity limit related stats added for XC10 2.0
            supportedStats.remove(StatsDescriptorUtil.MAP_CAPACITY_LIMIT);
            supportedStats.remove(StatsDescriptorUtil.MAP_PERCENT_USED_CAPACITY_LIMIT);
            supportedStats.remove(StatsDescriptorUtil.MAP_UNUSED_CAPACITY_LIMIT);
            supportedStats.remove(StatsDescriptorUtil.MAP_PRIMARY_USEDBYTES);
            supportedStats.remove(StatsDescriptorUtil.MAP_REPLICA_USEDBYTES);
            
            initialized = true;
        }
    }

    static statsApplicableFor(mask) {
        def rv = []

        supportedStats.each {
            if ((mask & it.value) != 0) {
                rv.add(it.key)
            }
        }
        return(rv)
    }

    // avgTP:all:1hr:map:TestGrid!Customer
    // <metricType>:<numDataPoints>:<timeRange>:<seriesType>:<seriesID>:
    
    static statsLevelFor(mask, seriesType, seriesID) {
        def children = []

        statsApplicableFor(mask).each {
            def x = ["chartable": true, 
                              "name": MessageUtils.message(GlobalContext.zget("/user/lang"), it, []), 
                              "metricType": it,
                              "seriesType": seriesType,
                              "seriesID": seriesID,
                              "scopeId": "all",
                              "unit": units[it]
                          ]
            children.add( x )
        }
        return children
    }
     
    static statsCategoryFor(name, mask, seriesType, seriesID) {
        def rv = ["name": name, "chartable": false, "children": []]
        
        rv["children"] += statsLevelFor(mask, seriesType, seriesID)
        return rv
    }

    static isInternalMap(map) {
        return map.contains("_" + StatsUtil.STATS_SERVER_STATS_MAP_PREFIX + "_") || map.contains(StatsUtil.STATS_SHARD_PREFIX);
    }

    static isInternalGrid(grid) {
        return grid.equals(StatsUtil.STATS_CLIENT_CACHE_GRID_NAME) || grid.equals(StatsUtil.STATS_SERVER_GRID_NAME)
    }

    static customStatsCategories() {
        def rv = [
            "name": MessageUtils.message(GlobalContext.zget("/user/lang"), "XSGRIDS", []), 
            "chartable": false, "children": [] 
        ]
                           
        def grids = GlobalContext.zget(GlobalContextURIs.Appliance.grids);
        grids.each { grid ->
            if (!isInternalGrid(grid)) {
                def gridlevel = statsCategoryFor(grid, GRID, "datagrid", grid)
                rv["children"].add(gridlevel)
                
                def maproot = [ 
                    "name": MessageUtils.message(GlobalContext.zget("/user/lang"), "XSMAPS", []),
                    "chartable": false, "children": [] 
                ]
                
                gridlevel["children"].add(maproot)
                
                GlobalContext.zget(GlobalContextURIs.Appliance.grids + "/" + grid)["mapNames"].each { map ->
                    if (!isInternalMap(map)) {
                        maproot["children"].add(statsCategoryFor(map, MAP, "map", grid + "!" + map))
                    }
                }
            }
        }


        return rv
    }

    static jvmStats() {
        def rv = [
            "name": MessageUtils.message(GlobalContext.zget("/user/lang"), "XSSERVERS", []), 
            "chartable": false, "children": [] 
        ]

        StatsHelper.listAllHostNames().each { host -> 
            def jvms = StatsHelper.listAllJVMNamess(host)
            jvms.each { jvm ->
                rv["children"].add(statsCategoryFor(jvm, JVM, "jvm", host + "!" + jvm))
            }
        }

        return rv
    }

    static rootStatsTree() {

        def rv = statsCategoryFor(GlobalContext.zget(DomainConnectionUpdater.CURRENT_DOMAIN), GRID, "datagrid", "all")
        rv["children"].add(customStatsCategories())
        rv["children"].add(jvmStats())

        
        return [ "items": [rv], "label": "name"]
    }
                           
}

 
def onList() {

    StatInfo.init()
    if (true) {
        request.view = 'JSON'
    	request.json.output = StatInfo.rootStatsTree()
    	render() 
    }else {
    	request.status=500
    }
}

