import com.ibm.ws.xs.app.StartHandler;
import com.ibm.ejs.ras.ManagerAdmin;
import zero.core.context.GlobalContext;
import com.ibm.ejs.ras.MalformedTraceStringException;
import java.util.logging.Logger;
import java.util.logging.Level;
import zero.json.Json;


// In: { rasTrace: "whatever@bozo.clown" }
// Out: {
//   "id": 1,
//   "rasTrace": "whatever@bozo.clown"
// }
def onUpdate()  {
    def req = Json.decode(request.input[])

    // Trim spaces out because ManagerAdmin does not tolerate them.
    String trimRASTrace = (req.rasTrace).replaceAll(" ", "");
    ManagerAdmin.checkTraceString(trimRASTrace)

    // old trace levels
    def currentTraceString = GlobalContext.<Map<String, String>>zget(StartHandler.RAS_TRACE);

    logger.FINE{'currentTraceString: ' + currentTraceString}
    def currentTraceLevels = currentTraceString.split(':').inject([:]) { map, token ->
        token.split('=').with { map[it[0]] = it[1] }
        map
    }

    logger.FINER{'currentTraceLevels: ' + currentTraceLevels}

    def newTraceString = trimRASTrace;

    // convert incoming trace string into a Map (assuming that it is of form
    // logger=logLevel:logger2=logLevel2(:loggern=logLevelN(=enabled|disabled)?)*
    
    def newTraceLevels = newTraceString.split(':').inject([:]) { map, token ->
        token.split('=').with {
          map[it[0]] = it[1];
          if(it.length == 3 && it[2].equalsIgnoreCase("disabled")) {
            map[it[0]] = "off";
          }
        }
        map
    }

    logger.FINE{'incoming trace string: ' + newTraceString }
    logger.FINER{'incoming trace levels: ' + newTraceLevels }

    if (currentTraceLevels != null) {
        for (String loggingLevel : currentTraceLevels.keySet()) {
            // First remove all of the current loggers not in new trace string.
            
            // We do this in case the user did not simply just add a logger to the
            // end of the current log string but replaced the whole existing string with a
            // partially or totally new one. This way, we don't accidentally carry
            // around trace that is not used anymore.
            // example to think through:
            //   oldTraceString = "*=info:ObjectGridMonitor=finer:ObjectGridReplication=warning"
            //   newTraceString = "Syslog=fine:ObjectGridServer=warning:com.ibm.ws.xs.app.CatalogListener=all"
            if (!newTraceLevels.containsKey(loggingLevel)) {
                    String mappedLoggerName = (loggingLevel.equals("*")) ? "" : loggingLevel;
                    Logger.getLogger(mappedLoggerName).setLevel(null);
            }
        }
    }

    setLogLevel(newTraceLevels);
    logger.FINER{'final trace levels: ' + newTraceLevels}

    GlobalContext.zput(StartHandler.RAS_TRACE, trimRASTrace)
    ManagerAdmin.setTraceState(trimRASTrace)
    logger.FINE{'Done with XS console trace configuration. Saved to ManagerAdmin and GC at key: ' + StartHandler.RAS_TRACE}

    request.status = 201
}

def setLogLevel(traceLevels) {
    logger.FINE{'entered setLogLevel'}
    for (String loggingLevel : traceLevels.keySet()) {
        String mappedLoggerName = (loggingLevel.equals("*")) ? "" : loggingLevel;
        Logger.getLogger(mappedLoggerName).setLevel(mapToLevel(loggingLevel, traceLevels.get(loggingLevel).toUpperCase()));
        logger.FINER{'logger(key): ' + loggingLevel + ", level(input_value): " +
            traceLevels.get(loggingLevel) + ", level(output_value): " + mapToLevel(loggingLevel, traceLevels.get(loggingLevel))}
    }
    logger.FINE{'exited setLogLevel'}
}

def mapToLevel(spec, level) {
    // converts RAS level (String) to JDK level (String)
    def levelToReturn = null;
    logger.FINE {' mapToLevel - level.toUpperCase(): ' + level.toUpperCase()};
    if (level.equalsIgnoreCase("event")) {
        if(spec.equalsIgnoreCase("*")) {
            levelToReturn = "INFO";
            logger.FINER {' mapToLevel - converting invalid trace level: event, to logger level INFO for default logger: ' + spec}
        } else {
            levelToReturn = "FINE";
            logger.FINER {' mapToLevel - converting EVENT to FINE'}
        }
    } else if (level.equalsIgnoreCase("entryExit")) {
        if(spec.equalsIgnoreCase("*")) {
            levelToReturn = "INFO";
            logger.FINER {' mapToLevel - converting invalid trace level: entryExit, to logger level INFO for default logger: ' + spec}
        } else {
            levelToReturn = "FINER";
            logger.FINER {' mapToLevel - converting ENTRYEXIT to FINER'}
        }
    } else if (level.equalsIgnoreCase("debug")) {
        if(spec.equalsIgnoreCase("*")) {
            levelToReturn = "INFO";
            logger.FINER {' mapToLevel - converting invalid trace level: debug, to logger level INFO for default logger: ' + spec}
        } else {
            levelToReturn = "FINEST";
            logger.FINER {' mapToLevel - converting DEBUG to FINEST'}
        }
    } else {
        if(spec.equalsIgnoreCase("*")) {
            if(!(level.equalsIgnoreCase("SEVERE") || level.equalsIgnoreCase("WARNING")
                || level.equalsIgnoreCase("INFO") || level.equalsIgnoreCase("OFF"))) {
                levelToReturn = "INFO";
                logger.FINER {' mapToLevel - converting invalid trace level: ' + level + ', to logger level INFO for default logger: ' + spec}
            } else {
                levelToReturn = level;
                logger.FINER {' mapToLevel - keeping level at: ' + level + ', for default logger: ' + spec}
            }
        } else {
            if (level.equalsIgnoreCase("off")) {
                levelToReturn = "OFF";
            } else {
                levelToReturn = level;
            }
            logger.FINER {' mapToLevel - keeping level at: ' + level + ' for specification: ' + spec}
        }
    }
    return parseLevel(levelToReturn);
}

def parseLevel(level) {
    logger.FINE{'Entered mapToLevel with level: ' + level}
    try {
            logger.FINE{'Returning valid level.'}
            return Level.parse(level);
    } catch (IllegalArgumentException e) {
            logger.FINE{'Returning null.'}
            return null;
    }
}
                      