/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;

import com.ibm.websphere.acl.PermissionsACL;
import groovy.rainmaker.constants.Status;
import com.ibm.websphere.rainmaker.logging.MessageUtils;

def _captureAction(resource, acl, state, disabled = false, nameKey = 'RM10225')
{
    if(!disabled)
    	disabled = ![Status.DRAFT, Status.CAPTURED].contains(resource.currentstatus);

    if (disabled)
        nameKey = acl < PermissionsACL.UPDATE ? 'RM10264' : 'RM10268';
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/capture-icon.gif',
        state: state,
        disabled: disabled
    ];
}

def _cloneAction(resource, acl, state, disabled = false, nameKey = 'RM10093')
{
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/clone-icon.gif',
        state: state, 
        disabled: disabled
    ];
}

def _extendAction(resource, acl, state, disabled = false, nameKey = 'RM10267')
{
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/extend-icon.gif',
        state: state, 
        disabled: disabled
    ];
}

def _deleteAction(resource, acl, disabled = false, nameKey = 'RM10029') {
    if (!disabled)
        disabled = resource.currentstatus == Status.DELETING;
    
    def typeNLS = null;
    
    if (disabled && nameKey == 'RM10029')
        nameKey = acl < PermissionsACL.DELETE ? 'RM10265' : 'RM10646';
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/delete-icon-large.gif',
        state: 'deleteResource',
        disabled: disabled || acl < PermissionsACL.DELETE
    ]
}

def _discoverAction(resource, acl, nameKey = 'RM10599') {
    
    def disabled = resource.currentstatus != Status.MAINTENANCE;

    if (disabled)
        nameKey = acl < PermissionsACL.UPDATE ? 'RM10264' : 'RM10605';

    if (!disabled && resource.certified != "T") {
        disabled = true;
        nameKey = 'RM10598';
    }

    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/discover-icon.gif',
        state: 'discoverResource',
        disabled: disabled || acl < PermissionsACL.UPDATE
    ]
}

def _editAction(resource, acl, disabled = false, nameKey = 'RM10033')
{
    if (disabled)
        nameKey = acl < PermissionsACL.UPDATE ? 'RM10264' : 'RM10668';
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/edit-icon.gif',
        state: 'editOffering',
        disabled: disabled || acl < PermissionsACL.UPDATE
    ];
}

def _lockAction(resource, acl, state, disabled = false, nameKey = 'RM10666') {
    if (disabled)
        nameKey = acl < PermissionsACL.UPDATE ? 'RM10264' : 'RM10667';
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/lock-icon.gif',
        state: state,
        disabled: disabled ||  acl < PermissionsACL.UPDATE
    ];
}

def _maintenanceAction(resource, acl, disabled = false, nameKey = 'RM10086') {
    if (!disabled)
        disabled = [Status.MAINTENANCE].contains(resource.currentstatus);
    
    if (disabled)
        nameKey = acl < PermissionsACL.UPDATE ? 'RM10264' : 'RM10074';
        
    if (!disabled && resource.currentstatus == Status.STARTED) {
        // Check for running instances which disables maintenance action
        def filters = [ "HypervisorID":resource?.id ];
        def vms = getResources('virtualmachines', filters, false);
        if (vms != null
            && !vms.isEmpty()) {
            vms.each {
                if ([Status.STARTED, Status.STARTING].contains(it.currentstatus)) {
                    disabled = true;
                    nameKey = 'RM10687';
                }
            }
        }
    }

    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/maint-mode-icon.gif',
        state: [ desiredstatus: Status.MAINTENANCE ],
        disabled: disabled || acl < PermissionsACL.UPDATE
    ]
}

def _startAction(resource, acl, state, disabled = false, nameKey = 'RM10050') {
    if (!disabled)
        disabled = resource.currentstatus == Status.STARTED;
    
    if (acl < PermissionsACL.UPDATE)
    {
        disabled = true;
        nameKey = 'RM10264';
    }
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/start-icon.gif',
        state: state,
        disabled: disabled || acl < PermissionsACL.UPDATE
    ]
}

def _storeAction(resource, acl, nameKey = 'RM10571') {
    def disabled = resource.currentstatus != Status.STOPPED;
    
    if (disabled)
        nameKey = acl < PermissionsACL.UPDATE ? 'RM10264' : (resource.currentstatus == Status.ARCHIVED ? 'RM10570' : 'RM10640');
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/store-icon.gif',
        state: [ desiredstatus: Status.ARCHIVED ],
        disabled: disabled || acl < PermissionsACL.UPDATE
    ]
}

def _stopAction(resource, acl, state, nameKey = 'RM10319') {
    def disabled = (resource.currentstatus != Status.STARTED);
    
    if (disabled)
        nameKey = acl < PermissionsACL.UPDATE ? 'RM10264' : (resource.currentstatus == Status.STOPPED ? 'RM10248' : 'RM10636');
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/stop-icon.gif',
        state: state,
        disabled: disabled || acl < PermissionsACL.UPDATE
    ]
}

def _commitServiceAction(resource, acl, state, disabled = false, nameKey = 'RM10437') {
    if (!disabled)
        disabled = ![Status.SERVICED].contains(resource.currentstatus);
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/done-editing-icon.gif',
        state: state,
        disabled: disabled || acl < PermissionsACL.UPDATE
    ]
}

def _rollbackServiceAction(resource, acl, state, disabled = false, nameKey = 'RM10438') {
    if (!disabled)
        disabled = ![Status.SERVICED, Status.SERVICE_FAILED].contains(resource.currentstatus);
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/restore-panel-icon.gif',
        state: state,
        disabled: disabled || acl < PermissionsACL.UPDATE
    ]
}

def _serviceAction(resource, acl, state, disabled = false, nameKey = 'RM10436') {
    if (!disabled)
        disabled = ![Status.STARTED, Status.COMPLETED].contains(resource.currentstatus);
    
    
    return [
        name: MessageUtils.message(user.lang[], nameKey, []),
        icon: '/images/service-icon.gif',
        state: state,
        disabled: disabled || acl < PermissionsACL.UPDATE
    ]
}

def _cloudsActions(cloud, acl) {

    def hypervisors = getResources("hypervisors", [cloudid:cloud.id], false);
    def disabled = false;

    // Prevent deleting a cloud with started hypervisor(s)
    if (hypervisors) {
        hypervisors.each {
            if ([Status.STARTED, Status.STARTING].contains(it.currentstatus)) {
                disabled = true;
            }
        }
    }
    return [ _deleteAction(cloud, acl, disabled) ]
}

def _groupsActions(group, acl) {
    def disabled = group.id == 1;
    def deleteKey = disabled ? 'RM10350' : 'RM10029';
        
    return [
        _deleteAction(user, acl, disabled, deleteKey)
    ];
}

def _hypervisorsActions(hypervisor, acl) {
    if (!request.params.admin[])
        return [];
        
    def disabled = false;
    def nameKey = 'RM10050';
    
    switch (hypervisor.currentstatus) 
    {
       case Status.FAILED:
           disabled = true;
           nameKey = 'RM10500';
           break;
       
       case Status.INITIALIZING:
           disabled = true;
           nameKey = 'RM10501';
           break;
           
       case Status.RESETTING:
           disabled = true;
           nameKey = 'RM10576';
           break;
        
       case Status.STARTED:
           disabled = true;
           nameKey = 'RM10246';
           break;
    }
    
    // These are validating in same order as back-end for consistency
    // 1. certified
    // 2. subnet
    // 3. cloud

    if (!disabled && hypervisor?.certified != "T") {
        disabled = true;
        nameKey = 'RM10499';
    }
    
    if (!disabled && hypervisor?.currentmessage == "RM03109") {
        disabled = true;
        nameKey = 'RM10644';
    }

    if (!disabled && hypervisor?.cloudid < 1) {
        disabled = true;
        nameKey = 'RM10498';
    }
    
    return [
        _startAction(hypervisor, acl, [ desiredstatus: Status.STARTED ], disabled, nameKey),
        _maintenanceAction(hypervisor, acl, false),
        _discoverAction(hypervisor, acl),
        _deleteAction(hypervisor, acl, [Status.STARTED, Status.STARTING].contains(hypervisor.currentstatus))
    ]
}

def _instancesActions(instance, acl) {    
    def startDisabled = false;
    def startNameKey = 'RM10050';
    
    switch (instance.currentstatus)
    {
        case Status.INITIALIZING:
            startDisabled = true;
            startNameKey = 'RM10501';
            break;

        case Status.STARTING:
            startDisabled = true;
            startNameKey = 'RM10637';
            break;

        case Status.PROCESSING:
            startDisabled = true;
            startNameKey = 'RM10637';
            break;

        case Status.TRANSFERING:
            startDisabled = true;
            startNameKey = 'RM10637';
            break;

        case Status.QUEUED:
            startDisabled = true;
            startNameKey = 'RM10637';
            break;
            
        case Status.CAPTURING:
            startDisabled = true;
            startNameKey = 'RM10638';
            break;
            
        case Status.SERVICING:
            startDisabled = true;
            startNameKey = 'RM10639';
            break;
        
        case Status.STARTED:
            startDisabled = true;
            startNameKey = 'RM10246';
            break;
    }
    
    return [
        _startAction(instance, acl, [ desiredstatus: Status.STARTED ], startDisabled, startNameKey),
        _stopAction(instance, acl, [ desiredstatus: Status.STOPPED ]),
        _storeAction(instance, acl), 
        _serviceAction(instance, acl, 'applyFixes'),
        _deleteAction(instance, acl)
    ]
}

def _patternsActions(pattern, acl) {
    if (request.params.editing[])
    {
        def undo = Boolean.valueOf(request.params.undo[]);
        def undoAll = Boolean.valueOf(request.params.undoAll[]);
        
        return [
            [
                name: MessageUtils.message(user.lang[], 'RM10090', []),
                icon: '/images/undo-icon.gif',
                state: 'undo',
                disabled: !undo
            ],
            [
                name: MessageUtils.message(user.lang[], 'RM10091', []),
                icon: '/images/undo-all-icon.gif',
                state: 'undoAll',
                disabled: !undoAll
            ],
            [
                name: MessageUtils.message(user.lang[], 'RM10092', []),
                icon: null,
                state: 'doneEditing'
            ]
        ]
    }
    
    else
    {
        def cloneDisabled = !user.session[].permissions.contains(Permissions.PATTERN_CREATOR);
        def cloneText = cloneDisabled ? 'RM10674' : 'RM10093';
        
        return [
            [
                name: MessageUtils.message(user.lang[], 'RM10655', []),
                icon: '/images/add-to-cloud-icon.gif',
                state: 'createInstance'
            ],
            _editAction(pattern, acl, pattern.currentstatus != Status.DRAFT),
            _cloneAction(pattern, acl, 'clonePattern', cloneDisabled, cloneText),
            _lockAction(pattern, acl, 'lockPattern', pattern.currentstatus != Status.DRAFT),
            _deleteAction(pattern, acl, pattern.currentstatus != Status.DRAFT && pattern.counter > 0)
        ]
    }
}

def _scriptsActions(script, acl) {
	def canBeDeleted = canScriptBeDeleted(script);
    return [ 
      _lockAction(script, acl, 'lockScripts', script.currentstatus != Status.DRAFT),
      _deleteAction(script, acl, !canBeDeleted)
    ]
}

def _storageActions(storage, acl) {
    def relas = getResources("physicalmachinesstorage", [storageid:storage.id], false);
    def pms = new ArrayList();
    relas.each {
        pms.add(getResource("physicalmachines", [id:it.physicalmachineid]));
    }
    
    def storageActive = false;

    pms.each {
        if (it?.currentstatus == Status.STARTED || it?.desiredstatus == Status.STARTED) {
            storageActive = true;
        }
    }
    [ _deleteAction(storage, acl, storageActive) ]
}

def _subnetsActions(subnet, acl) {
    def relas = getResources("networks", [subnetid:subnet.id], false);
    def pms = new ArrayList();
    relas.each {
        pms.add(getResource("physicalmachines", [id:it.physicalmachineid]));
    }
    
    def subnetActive = false;

    pms.each {
        if (it?.currentstatus == Status.STARTED || it?.desiredstatus == Status.STARTED) {
            subnetActive = true;
        }
    }
    [ _deleteAction(subnet, acl, subnetActive) ]
}

def _templatesActions(template, acl) {
	def canBeDeleted = canVirtualImageBeDeleted(template);
    
    def notReadyForClone = template.currentstatus != Status.PUBLISHED;
    def noPermissionToClone = !user.session[].permissions.contains(Permissions.CATALOG_CREATOR);
    
    def cloneDisabled = notReadyForClone || noPermissionToClone;
    def cloneText = cloneDisabled ? (notReadyForClone ? 'RM10641' : 'RM10675') : 'RM10267';
        
    return [
        _extendAction(template, acl, 'cloneTemplate', cloneDisabled, cloneText),
        _captureAction(template, acl, 'captureTemplate'),
        _lockAction(template, acl, 'lockTemplate', template.currentstatus != Status.DRAFT), 
        _deleteAction(template, acl, !canBeDeleted)
    ];    
}

def _fixesActions(fixes, acl) {
    return [
        _deleteAction(fixes, acl)
    ];    
}

def _domainsActions(targetUser, acl) {
    def deleteKey = 'RM10029';
            
    return [ _deleteAction(user, acl, false, 'RM10029') ]
}

def _serversActions(targetUser, acl) {
    def deleteKey = 'RM10029';
            
    return [ _deleteAction(user, acl, false, 'RM10029') ]
}


def _usersActions(targetUser, acl) {
    def disabled = targetUser.id == user.session[].id || targetUser.id == 1;
    def deleteKey = 'RM10029';
    
    if (disabled)
        deleteKey = targetUser.id == 1 ? 'RM10250' : 'RM10251';
        
    [ _deleteAction(user, acl, disabled, deleteKey) ]
}

def _part2sActions(part, acl) {
    [ _deleteAction(part, acl) ]
}


def session = user.session[];

def resource = request.json.output[];
def type = request.params.render.resourceType[];
def id = request.params.render.resourceID[];

def acl = getAccessControl(type, id, session.id);

//
// if there's no ACL entry but com.ibm.websphere.rest.RESTUtils.authorized() == true, it's a resource 
// type w/o an 'owner', which is evaluated by the user's roles
//
if (!acl && com.ibm.websphere.rest.RESTUtils.authorized())
    acl = PermissionsACL.DELETE;

request.headers.out.'Content-Type' = 'application/json';
def actions = "_${type}Actions"(resource, acl);

def json = [
    name: resource.name, 
    actions: actions
];

if (resource.label)
    json.label = resource.label;

if (type == 'hypervisors')
    json.address = resource.address;

Json.encode(json, request.writer[]);
