/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;

def onTimer() {
	
	logger.FINE { "entering onTimer task for DiskCleanup" };
	
	def applications = config.rainmaker.applications[]
	def filePatterns = config.rainmaker.diskcleanup.patterns[]
	def expireDays = config.rainmaker.diskcleanup.expireDays[].intValue()
	def dumpLocation = config.rainmaker.diskcleanup.dumpLocation[]
	
	def today = new Date()
	def todayMinusExpireDays = today - expireDays
	
	
	
	applications.each { application ->
		def appHome = application["appHome"]
		def diskRoot = application["diskRoot"];
		
		def appDirectory = new File(appHome);
		if (appDirectory.exists()){
			filePatterns.each { pattern ->
				appDirectory.eachFileMatch(~"${pattern}"){ file ->
					logger.FINE (clazz: "app.scripts." + this.getClass().getName() +".groovy" , method: "onTimer", msg: { "found match " + file + "--> renaming to diskRoot location" } );
					file.renameTo (new File (dumpLocation + "/" + file.name));	
				}
			}
		}
		
		def diskRootDirectory = new File(diskRoot);
		if (diskRootDirectory.exists()){
			filePatterns.each { pattern ->
				diskRootDirectory.eachFileMatch(~"${pattern}"){file ->
					def fileDate = new Date(file.lastModified() );
					if( (fileDate.compareTo(todayMinusExpireDays))  < 0 ){
						logger.FINE (clazz: "app.scripts." + this.getClass().getName()+".groovy" , method: "onTimer", msg: {"Found matching file " + file + "(" + fileDate+ ") that is older than expiration date --> deleting" } );
						def deletedFile = file.delete()
						logger.FINE (clazz: "app.scripts." +this.getClass().getName()+".groovy" , method: "onTimer", msg: { "file deleted -->" + deletedFile });
					}
				}
			}
		}
	}
	logger.FINE { "exiting onTimer task for DiskCleanup" };
}