/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;
import com.ibm.bedrock.BedrockUtils;
import com.ibm.bedrock.firmwaremanager.FirmwareManagerUtils;
import com.ibm.bedrock.processmanager.ProcessManagerUtils;
import com.ibm.bedrock.appliance.ApplianceUtils;

def getSerialNumber() {
	if(BedrockUtils.isBedrock()) {
		return ApplianceUtils.getHardwareInfo().getSerialNumber();
	} else {
		return "Virtual appliance does not have serial number.";
	}
		
}

def getModelType() {
	if(BedrockUtils.isBedrock()) {
		String modelType = ApplianceUtils.getMTM();
		if (modelType.size() >= 7) {
			return modelType.substring(0, 4)+"-"+modelType.substring(4);
		} else {
			return moduleType;
		}
	} else {
		return "Virtual appliance does not have machine model and type.";
	}
		
}

def rollbackFirmware()
{
	if(BedrockUtils.isBedrock()){
		return FirmwareManagerUtils.rollbackFirmware(true);
	}else{
		rollbackFirmwareDeprecated();
	}
}

def rollbackFirmwareDeprecated()
{
    try
    {
    	return executeApplianceRequest([
            name: 'soma:do-action', 
            content: [:], 
            children: [
                [
                    name: 'BootSwitch', 
                    content: [:]
                ]
            ]
        ]);
    }
    
    catch (Throwable t)
    {
        logger.FINE { CleanStackTrace.cleanStackTrace(t) };
        return null;
    }
}

def upgradeFirmware(fileName)
{
	if(BedrockUtils.isBedrock()){
		def instance = com.ibm.websphere.rainmaker.appliance.bedrock.FirmwareFactory.getInstance();
		instance.upgradeFirmware(fileName);
		return true;
		/*
		logger.INFO { "about to start firmware upgrade process" }
		def result= FirmwareManagerUtils.upgradeFirmware(fileName, true, false);// false to skip reboot; do after
		logger.INFO { "completed firmware upgrade process result : ${result}" }
		
		def mesg =  MessageUtils.message(user.lang[], 'RM16022', []);
	 	if( !result) {
	 		//TODO:  get status for failure paths..... currently just refer to log files for further assistance
	 		mesg =  MessageUtils.message(user.lang[], 'RM16021', []);
			logger.SEVERE { MessageUtils.message(false, 'RM16021') };		
		} else {
			logger.INFO { MessageUtils.message(false, 'RM16022', []) };
		}
		zput("/app/lastFirmwareUpgradeResult", mesg);
		
		logger.INFO { "request rebooting appliance" }
		Thread.start {
			sleep(5000); 
			ProcessManagerUtils.requestReboot();
		}
		logger.INFO { "returning result" }
		return result;
		*/

	}else{
		return upgradeFirmwareDeprecated(fileName);
	}
}
def upgradeFirmwareDeprecated(fileName)
{
    try
    {
    	def response =  executeApplianceRequest([
            name: 'soma:do-action', 
            content: [:], 
            children: [
                [
                    name: 'ApplyPatch', 
                    content: [:], 
                    children: [
                        [
                            name: 'File', 
                            content: "${fileName}"
                        ]
                    ]
                ]
            ]
        ]);
	def mesg = "OK";
 	if( !response.result.equals("OK")) { 
		mesg  = response.result.get("error-log").get("log-event").content[0];
		logger.SEVERE { MessageUtils.message(false, 'RM16021', [mesg]) };		
	} else {
		logger.INFO { MessageUtils.message(false, 'RM16022', []) };
	}
	zput("/app/lastFirmwareUpgradeResult", mesg);
	return response;
    }
    
    // MAJOR LEAGUE HACK, FIX ASAP AFTER 1.0.0.1
    // Because of using the -f flag for a firmware upgrade, we get a SOMA response like the following:
    // {
    // "html": {
    //  "body": {
    //     "P": true,
    //     "content": "You may want to clear the cookies in your browser."
    //  },
    //  "title": "Error"
    //	}
    // }
    // In the case of a firmware upgrade, let's ignore the exception that gets thrown because of this. In the future,
    // we need to fix this so that the -f succeeds cleanly and then we can remove this hack.
    catch (IOException ioe) {
    	if (ioe.message.equals("No response received from appliance admin interface.")) {
    		app.lastFirmwareUpgradeResult = "OK";
    		logger.FINE { "Caught ioe for 'No response received from appliance admin interface.' Setting status to OK." };
    	}
    	else {
    		app.lastFirmwareUpgradeResult = ioe.message;
    		logger.FINE { "Caught ioe for " + ioe.message + ". Setting status to message." };
    	}
    	
    	logger.FINE { CleanStackTrace.cleanStackTrace(ioe) };
    	return null;
    }
    catch (Throwable t)
    {
        logger.FINE { "Caught throwable while upgrading firmware. Stack trace=" + cleanStackTrace(t) };
        if (t.message.contains("No response received from appliance admin interface.")) {
    		app.lastFirmwareUpgradeResult = "OK";
    		logger.FINE { "Caught throwable for 'No response received from appliance admin interface.' Setting status to OK." };
    	}
        else {
        	app.lastFirmwareUpgradeResult = t.message;
        }
        return null;
    }
}
