/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;

import groovy.xml.MarkupBuilder;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;

import zero.core.connection.Connection;
import zero.json.Json;
import zero.json.xml.XMLToJSONTransformer;
import com.ibm.bedrock.BedrockUtils;

def executeApplianceRequest(soapBody) {

	if(onAppliance()){
		logger.FINEST { "onAppliance is set. Using local http connection for SOMA call" };
		def connection = new Connection(config.rainmaker.dpURLUnsecured[], Connection.Operation.POST);
	    def parameters = [
	                      readTimeout: 300,
	                      writeTimeout: 300
	                  ];
	    
	    connection.setProtocolConfiguration('http', parameters);
		return _executeSOMA(soapBody, connection);
	}else{	// execute request based upon the xcadmin user
		logger.FINEST { "onAppliance is disabled. Using secured https connection for SOMA call with xcadmin account" };
		return executeApplianceRequestWithUser(soapBody, 'xcadmin', config.rainmaker.admin.defaultPassword[])
	}
}

def executeApplianceRequestWithUser(soapBody, userId, password) {
    def connection = new Connection(config.rainmaker.dpURL[], Connection.Operation.POST);
    
    def parameters = [
        httpsConfig: 'dpHttpsConfig'
    ]
    connection.setProtocolConfiguration('https', parameters);
    
    def authInfo = "${userId}:${password}";
    def authEncoded = authInfo.getBytes('UTF-8').encodeBase64();
    def authHeader = ["Basic ${authEncoded}"];
    connection.setRequestHeader('Authorization', authHeader);
    return _executeSOMA(soapBody, connection);
}

def _executeSOMA(soapBody, connection){
	if(true){
		logger.FINE { "Attempt to execute SOMA call ignored.  Has been replaced by Bedrock.  Ignore SOAP message call from being made "+ soapBody };
		return null;
	}
	
    def writer = new StringWriter();
    def xml = new MarkupBuilder(writer);
    
    xml.'soap:Envelope'('xmlns:soap': 'http://schemas.xmlsoap.org/soap/envelope/') {
        'soap:Body'() {
            'soma:request'('xmlns:soma': 'http://www.datapower.com/schemas/management', domain: 'default') {
                "${soapBody.name}"(soapBody.content) {
                    soapBody.children?.each { child -> 
                        "${child.name}"(child.content) {
                            child.children?.each { subchild -> 
                                "${subchild.name}"(subchild.content) {
                                    subchild.children?.each {
                                        "${it.name}"(it.content)
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    def soapRequest = writer.toString();
    logger.FINEST { "SOAP request to admin interface:\n\n${soapRequest}\n" };
    connection.setRequestBody(soapRequest);
    
    def response = connection.getResponse();
    
    if (!response.responseStatus.startsWith('2')){
    	System.out.println("gc connnection response " + zdump("/connection/response"));
    	System.out.println("error response " + response.getResponseBodyAsString());
        throw new IOException("Received error from appliance admin interface: ${response.getResponseStatus()}");
    }
        
    def soapResponse = response.getResponseBodyAsString()?.replaceAll(' read-only=\"true\"', '');
    def jsonObject = null;
    
    try
    {
        def soapBytes = soapResponse.getBytes();
        def byteStream = new ByteArrayInputStream(soapBytes);
        def jsonString = XMLToJSONTransformer.transform(byteStream);
        jsonObject = Json.decode(jsonString);
    }
    
    catch (Throwable t)
    {
        logger.FINE { CleanStackTrace.cleanStackTrace(t) };
        return null;
    }
    
    logger.FINEST { "SOAP response from admin interface (as JSON):\n\n${Json.encode(jsonObject, true)}\n" };
    
    if (jsonObject?.Envelope?.Body?.response?.result == 'Authentication failure' )
        throw new IOException('Authentication error for appliance admin interface.');
    
    def result = jsonObject?.Envelope?.Body?.response;
    
    if (!result)
        throw new IOException('No response received from appliance admin interface.');
        
    return result;
}

def saveApplianceConfiguration()
{
	executeApplianceRequest([
        name: 'soma:do-action', 
        content: [:],
        children: [
            [
                name: 'SaveConfig',
                content: [:]
            ]
        ]
    ]);
}
