/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;


import com.ibm.websphere.rainmaker.appliance.NLSUtils;
import com.ibm.websphere.rainmaker.logging.MessageUtils;



def initializeLocale(){
	if(tmp.rainmaker.setServerLocale[]) return;
	
	logger.FINE { 'Retrieving the defined locale if available; default otherwise' }	
	if (NLSUtils.hasDefault()) {
		Locale definedLocale = NLSUtils.readLocale();
		_updateServerLocale(definedLocale);
	}
	else logger.FINE { 'Appliance locale set to ' + Locale.getDefault().getDisplayName() };
	
	tmp.rainmaker.setServerLocale = true
}

def onTimer(){
	logger.FINE { 'Timer task to determine if locale has been updated' }
	
	if (NLSUtils.hasDefault()) {
		Locale definedLocale = NLSUtils.readLocale();
        def storageLocale = storage.rainmaker.serverLocale[];
        if(!definedLocale.toString().equals(storageLocale)){
        	_updateServerLocale(definedLocale);
        }
	}
}

def _updateServerLocale(definedLocale){
		Locale.setDefault(definedLocale);	
		logger.FINE { 'Appliance has default locale set to ' + definedLocale.getDisplayName() };
	    if( config.dependencies[].contains("ibm:xsa.app")){
	        def storageLocale = storage.rainmaker.serverLocale[];
	        if(!definedLocale.toString().equals(storageLocale)){
	                logger.FINE { 'Server locale has been changed; updating user and group to reflect this'};
	                def lang = MessageUtils.getLanguageFromLocale(definedLocale);
	                updateUser([id: 1, name: MessageUtils.message(lang, 'RM02010', [])]);
	                updateGroup([id: 1, name: MessageUtils.message(lang, 'RM02014', [])]);
	                storage.rainmaker.serverLocale = definedLocale.toString();
	        }
	    }
}