/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.appliance;

import com.ibm.websphere.rainmaker.appliance.TimeZoneUtils;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.bedrock.BedrockUtils;
import com.ibm.bedrock.appliance.TimeZoneUtils;

def getTimeZones() 
{

	if(BedrockUtils.isBedrock()){
		return getTimeZonesBedrock();
	}else{
		getTimeZonesDeprecated()
	}
}

def getTimeZonesBedrock(){
	return [
	        	"HST (${MessageUtils.message(user.lang[], 'RM16000', []) })": 'posix/US/Hawaii',
	            "AKST (${MessageUtils.message(user.lang[], 'RM16001', []) })": 'posix/US/Alaska', 
	            "PST (${MessageUtils.message(user.lang[], 'RM16002', []) })": 'posix/US/Pacific', 
	            "MST (${MessageUtils.message(user.lang[], 'RM16003', []) })": 'posix/US/Mountain', 
	            "CST (${MessageUtils.message(user.lang[], 'RM16004', []) })": 'posix/CST6CDT', 
	            "EST (${MessageUtils.message(user.lang[], 'RM16005', []) })": 'posix/EST5EDT', 
	            "AST (${MessageUtils.message(user.lang[], 'RM16006', []) })": 'posix/Canada/Atlantic', 
	            "UTC (${MessageUtils.message(user.lang[], 'RM16007', []) })": 'posix/UTC', 
	            "GMT (${MessageUtils.message(user.lang[], 'RM16008', []) })": 'posix/GMT', 
	            "CET (${MessageUtils.message(user.lang[], 'RM16009', []) })": 'posix/Europe/Berlin', 
	            "EET (${MessageUtils.message(user.lang[], 'RM16010', []) })": 'posix/Europe/Istanbul', 
	            "MKS (${MessageUtils.message(user.lang[], 'RM16011', []) })": 'posix/Europe/Moscow',
	            "AST (${MessageUtils.message(user.lang[], 'RM16012', []) })": 'posix/Asia/Baghdad',
	            "KRT (${MessageUtils.message(user.lang[], 'RM16013', []) })": 'Etc/GMT-5', //'KRT-5',	//???? 
	            "IST (${MessageUtils.message(user.lang[], 'RM16014', []) })": 'posix/Asia/Kolkata', 
	            "NOV (${MessageUtils.message(user.lang[], 'RM16015', []) })": 'Asia/Almaty', 
	            "CST (${MessageUtils.message(user.lang[], 'RM16016', []) })": 'posix/Asia/Taipei', 
	            "WST/WDT (${MessageUtils.message(user.lang[], 'RM16017', []) })": 'posix/Australia/Perth', 
	            "JST (${MessageUtils.message(user.lang[], 'RM16018', []) })": 'posix/Asia/Tokyo', 
	            "CST/CDT (${MessageUtils.message(user.lang[], 'RM16019', []) })": 'posix/Australia/Adelaide', 
	            "EST/EDT (${MessageUtils.message(user.lang[], 'RM16020', []) })": 'posix/Australia/Sydney'
	        
	        ]
}

def getTimeZonesDeprecated() 
{
	 return [
         "HST (${MessageUtils.message(user.lang[], 'RM16000', []) })": 'HST10',
         "AKST (${MessageUtils.message(user.lang[], 'RM16001', []) })": 'AKST9AKDT', 
         "PST (${MessageUtils.message(user.lang[], 'RM16002', []) })": 'PST8PDT', 
         "MST (${MessageUtils.message(user.lang[], 'RM16003', []) })": 'MST7MDT', 
         "CST (${MessageUtils.message(user.lang[], 'RM16004', []) })": 'CST6CDT', 
         "EST (${MessageUtils.message(user.lang[], 'RM16005', []) })": 'EST5EDT', 
         "AST (${MessageUtils.message(user.lang[], 'RM16006', []) })": 'AST4ADT', 
         "UTC (${MessageUtils.message(user.lang[], 'RM16007', []) })": 'UTC', 
         "GMT (${MessageUtils.message(user.lang[], 'RM16008', []) })": 'GMT0BST', 
         "CET (${MessageUtils.message(user.lang[], 'RM16009', []) })": 'CET-1CEST', 
         "EET (${MessageUtils.message(user.lang[], 'RM16010', []) })": 'EET-2EEST', 
         "MKS (${MessageUtils.message(user.lang[], 'RM16011', []) })": 'MKS-3MSD', 
         "AST (${MessageUtils.message(user.lang[], 'RM16012', []) })": 'AST-3', 
         "KRT (${MessageUtils.message(user.lang[], 'RM16013', []) })": 'KRT-5', 
         "IST (${MessageUtils.message(user.lang[], 'RM16014', []) })": 'IST-5:30', 
         "NOV (${MessageUtils.message(user.lang[], 'RM16015', []) })": 'NOVST-6NOVDT', 
         "CST (${MessageUtils.message(user.lang[], 'RM16016', []) })": 'CST-8', 
         "WST/WDT (${MessageUtils.message(user.lang[], 'RM16017', []) })": 'WST-8WDT', 
         "JST (${MessageUtils.message(user.lang[], 'RM16018', []) })": 'JST-9', 
         "CST/CDT (${MessageUtils.message(user.lang[], 'RM16019', []) })": 'CST-9:30CDT', 
         "EST/EDT (${MessageUtils.message(user.lang[], 'RM16020', []) })": 'EST-10EDT'
     ];
}

def getTimeZone()
{
    if(BedrockUtils.isBedrock()){
            def timeZone = com.ibm.bedrock.appliance.TimeZoneUtils.getTimeZone();
            logger.INFO { "isBedrock runtime --> bedrock default timeZone ${timeZone}" };
            def timeZoneMap = [:];
            timeZoneMap.localtimezone = timeZone;
            return timeZoneMap;
    }else{
            return getTimeZoneDeprecated();
    }
}

    
def getTimeZoneDeprecated(){	
	
	def response = executeApplianceRequest([
        name: 'soma:get-config',
        content: [
            class: 'TimeSettings'
        ]
    ]);
    
    def tz = response?.config?.TimeSettings;
    
    if (!tz)
        return tz;
    
    def keys = tz.keySet().collect { it };    
    keys.each {
        def value = tz.remove(it);
        tz[it.toLowerCase()] = value;
    }
    
    return tz;
}

def updateTimeZone(tz) 
{
	if(BedrockUtils.isBedrock()){
		logger.INFO { "isBedrock runtime --> setting timezone ${tz}" };
		com.ibm.bedrock.appliance.TimeZoneUtils.setTimeZone(tz.localtimezone);
		
	}else{
		updateTimeZoneDeprecated(tz);
	}
	com.ibm.websphere.rainmaker.appliance.TimeZoneUtils.setTimeZone (tz.localtimezone);
	
}

def updateTimeZoneDeprecated(tz){
    executeApplianceRequest([
        name: 'soma:modify-config',
        content: [:],
        children: [
            [
                name: 'TimeSettings',
                content: [:], 
                children: [[
                    name: 'LocalTimeZone', 
                    content: tz.localtimezone
                ]]
            ]
        ]
    ]);
    
    saveApplianceConfiguration();
    
} 
