/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.core;

import zero.core.context.GlobalContextURIs.Event;
import zero.core.events.EventEngine;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;

def cloneContext()
{
    def context = [:];
    
    zlist('/event').each {
    	def parts = it.tokenize('/');
    	if(parts.size() > 1) {
    		def name = parts[1];            
            if (_isUserEventData(name))
                context[name] = zget(it);    		
    	}        
    }
    
    return context;
}

def _cloneMap(map)
{
    return map.findAll { _isUserEventData(it.key) };
}

def deepCopy(obj)
{
    try
    {
        obj = obj.clone();
    }
    
    catch (CloneNotSupportedException e)
    {
        logger.FINER { "Tried to clone object of type ${obj.getClass()} and failed - continuing on." };
    }
    
    if (obj instanceof Collection)
        for (i in 0..<obj.size())
            obj[i] = deepCopy(obj[i]);
    
    else if (obj instanceof Map)
        obj.each {
            it.value = deepCopy(it.value);
        }
        
    return obj;
}

def fire(name, context, async)
{
    def alwaysJoinThreads = zget("/config/rainmaker/alwaysJoinThreads", false);
    
    if (alwaysJoinThreads)
        async = false;
    
    def record = [
        id: new Object(), 
        name: name, 
        context: _cloneMap(context), 
        async: async, 
        rainmakerParentEvent: event.rainmakerCurrentEvent[] 
    ];
    
    _recordEventStart(record);
    
    context.rainmakerCurrentEvent = record;
    
    def previous = cloneContext();

    if (async) {
        def t = new Thread({ _fire(name, context, record); });
        t.name = name;
        EventEngine.fire('event-created', [ context: context, previous: previous, thread: t ]);
        t.start();
    }

    else {
        EventEngine.fire('event-created', [ context: context, previous: previous ]);
        _fire(name, context, record);
    }
}

def _fire(name, context, record)
{
    try {
        EventEngine.fire(name, context);
        _recordEventEnd(record);
    }
    catch (Throwable t) {
        logger.SEVERE { MessageUtils.message('RM15008', [name, t.message]) }
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
    }
}

def _isUserEventData(name)
{
    def isZeroData = Event.fields.find { it.equals(name) } != null || name.equals('_name');
    def isRainmakerData = name.startsWith('rainmaker');
    return !isZeroData && !isRainmakerData;
}

def _recordEventStart(record)
{
    logger.FINER { "Event started: ${record.name}" };
    
    record.context.each { x ->
        logger.FINEST(msg: {"-----> ${x.key}: ${x.value}"})}
    
    getEventStorage().put(record.id, record);
}

def _recordEventEnd(record)
{
    logger.FINER { "Event finished: ${record.name}" };
    getEventStorage().remove(record.id);
}
