/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
package groovy.rainmaker.core

// returns s capitalized
def capitalize(s) {
    return s[0].toUpperCase(Locale.US) + s[1..-1];
}


// returns s capitalized like pattern
def matchCase(pattern, s) {
    switch (pattern) {
        // alllowercase
        case { ! it.find { c -> Character.isUpperCase(c as Character) } }:
            return s.toLowerCase()

        // ALLUPPERCASE
        case { ! it.find { c -> Character.isLowerCase(c as Character) } }:
            return s.toUpperCase()

        // Capitalized
        case { Character.isUpperCase(it[0] as Character) }:
            return capitalize(s)

        // rAnDoMjUnK
        default:
            return s
    }
}


// returns the singular noun corresponding to plural, preserving capitalization
def singularize(plural) {
    def lcplural = plural.toLowerCase()
    def singular = StringUtilsStatics.SINGULAR_PLURAL.find { s,p -> p == lcplural }?.key

    return singular ? matchCase(plural, singular) :
        lcplural.endsWith('s') ?
            plural[0..-2] :
            plural
}


// returns the plural noun corresponding to singular, preserving capitalization
def pluralize(singular) {
    matchCase(singular,  StringUtilsStatics.SINGULAR_PLURAL[singular.toLowerCase()] ?: singular + 's')
}

//
// normalizeForGC() replaces invalid GC key characters with an underscore.
//
def normalizeForGC(key)
{
    return key.replaceAll('[/#]', '_');
}
