/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
package groovy.rainmaker.auditing;

import groovy.rainmaker.logging.*;
import groovy.rainmaker.constants.*;
import com.ibm.websphere.rainmaker.auditing.*;

import java.util.*;

/*
 * This class contains the milliseconds for some time period. 
 */
class TimePeriodConstants {
	public static final long YEAR   = (long)365 * 86400 * 1000;
	public static final long MONTH  = (long) 31 * 86400 * 1000;
	public static final long WEEK   = (long)  7 * 86400 * 1000;
	public static final long DAY    = (long)  1 * 86400 * 1000;
	public static final long HOUR   = (long)  1 *  3600 * 1000;
}


def generateUtilizationData(filtered, filter, summary, align)
{
	def stage1 = _generateStageOneData(filtered);

	def ret = null;
	if(true == summary)
	{
		ret = [];
		def item = [:];

		if(!stage1.avearray.contains(null) && !stage1.maxarray.contains(null) )
		{
			def sum = (0).doubleValue();
			def n = 0;
			def max = (0).doubleValue();
			stage1.each {
				sum += it.avearray.value.sum();
				n += it.avearray.size();
				if(max < it.maxarray.value.max()) {
					max = it.maxarray.value.max();
				}
			}
			if(n != 0) {
				item.average = sum / n;
			}
			else
			{
				item.average = (0).doubleValue();
			}
			
			item.maximum = max;
		}
		else if(!stage1.utilarray.contains(null))
		{
			def sum = (0).doubleValue();
			def n = 0;
			def max = (0).doubleValue();
			stage1.each {
				sum += it.utilarray.value.sum();
				n += it.utilarray.size();
				if(max < it.utilarray.value.max()) {
					max = it.utilarray.value.max();
				}
			}
			item.average = sum / n
			item.maximum = max;
		}
		
		item.put("unit", "percent");

		ret << item;
	}
	else
	{
		// If the time period is not specified, it will return an hourly report.
		long time = TimePeriodConstants.DAY;
		if(filter.end != null && filter.start != null) {
			time = Long.parseLong(filter.end) - Long.parseLong(filter.start);
		}

		ret = _generateTimePeriodData(stage1, time);

		if(ret != null && ret.size() > 0)
		{
			ret.each {
				if(it.avearray != null && it.avearray.size() > 0)
				{
					it.average = it.avearray.value.sum() / it.avearray.size();  
				}
				if(it.maxarray != null)
				{
					it.maximum = it.maxarray.value.max();
				}
				
				it.put("unit", "percent");
			}
		}
		if(true == align)
		{
			_paddingDataSheet(ret, filter);
		}
		ret = _sort(ret, "id", false);
		
	}
	
	java.text.DecimalFormat df = new java.text.DecimalFormat("0.##");
	if(ret != null && ret.size() > 0)
	{
		ret.each {
			if(it.average != null)
			{
				it.average = df.format(it.average);
			}
			
			if(it.maximum != null)
			{
				it.maximum = df.format(it.maximum);
			}
			
		}
	}

	
	
	return ret;
}

def _paddingDataSheet(data, filter)
{
	def x_axis = _generateXAxisLabel(filter);
	
	data.each
	{ row ->
		def avearray = [];
		def maxarray = [];
		if(row.avearray != null)
		{
			avearray.addAll(row.avearray);
			row.avearray.clear();
		}
		if(row.maxarray != null)
		{
			maxarray.addAll(row.maxarray);
			row.maxarray.clear();
		}
		
		x_axis.each
		{ label ->
			if(avearray.timestamp.contains(label) == true)
			{
				def item = [:];
				def index = avearray.timestamp.indexOf(label);
				item.put("timestamp", label);
				item.put("value", avearray[index].value);
				row.avearray << item;
			}
			else
			{
				def item = [:];
				item.put("timestamp", label);
				item.put("value", "");
				row.avearray << item;
			}
			
			if(maxarray.timestamp.contains(label) == true)
			{
				def item = [:];
				def index = maxarray.timestamp.indexOf(label);
				item.put("timestamp", label);
				item.put("value", maxarray[index].value);
				row.maxarray << item;
			}
			else
			{
				def item = [:];
				item.put("timestamp", label);
				item.put("value", "");
				row.maxarray << item;
			}
		}
		
	}

}

def _generateXAxisLabel(filter)
{
	long start = Long.parseLong(filter.start);
	long end = Long.parseLong(filter.end);
	long time = end - start;
	def x_axis = [];
	
	if(time > TimePeriodConstants.YEAR)
	{
		def point = start;
		while(point <= end)
		{
			def month = _getMonth(point.toString());
			def year = _getYear(point.toString());
			def key = "" + year + "-" + month;
			if(!x_axis.contains(key)) 
			{
				x_axis << key;
			}
			point += TimePeriodConstants.MONTH / 2;
		}
		def month = _getMonth(end.toString());
		def year = _getYear(end.toString());
		def key = "" + year + "-" + month;
		if(x_axis.contains(key) == false)x_axis << key;
		
	}
	else if(time <= TimePeriodConstants.YEAR && time > TimePeriodConstants.MONTH)
	{
		def point = start;
		while(point <= end)
		{
			def month = _getMonth(point.toString());
			def year = _getYear(point.toString());
			def key = "" + year + "-" + month;
			if(!x_axis.contains(key)) 
			{
				x_axis << key;
			}
			point += TimePeriodConstants.MONTH / 2;
		}
		def month = _getMonth(end.toString());
		def year = _getYear(end.toString());
		def key = "" + year + "-" + month;
		if(x_axis.contains(key) == false)x_axis << key;
	}
	else if(time <= TimePeriodConstants.MONTH && time > TimePeriodConstants.WEEK)
	{
		def point = start;
		while(point <= end)
		{
			def month = _getMonthName(point.toString());
			def week = _getWeekOfMonth(point.toString());
			def key = "Week " + week + ", " + month;
			if(!x_axis.contains(key)) 
			{
				x_axis << key;
			}
			point += TimePeriodConstants.DAY;
		}
		def month = _getMonthName(end.toString());
		def week = _getWeekOfMonth(end.toString());
		def key = "Week " + week + ", " + month;
		if(x_axis.contains(key) == false)x_axis << key;
	}
	else if(time <= TimePeriodConstants.WEEK && time > TimePeriodConstants.DAY)
	{
		def point = start;
		while(point <= end)
		{
			def date = _getDate(point.toString());
			def key = "" + date;
			if(!x_axis.contains(key)) 
			{
				x_axis << key;
			}
			point += TimePeriodConstants.DAY / 2;
		}
		def date = _getDate(end.toString());
		def key = "" + date;
		if(x_axis.contains(key) == false)x_axis << key;
	}
	else if(time <= TimePeriodConstants.DAY && time > TimePeriodConstants.HOUR)
	{
		def point = start;
		while(point <= end)
		{
			def hour = _getHourOfDay(point.toString());
			def key = "" + hour;
			if(!x_axis.contains(key)) 
			{
				x_axis << key;
			}
			point += TimePeriodConstants.HOUR / 2;
		}
		def hour = _getHourOfDay(end.toString());
		def key = "" + hour;
		if(x_axis.contains(key) == false)x_axis << key;
	}
	else if(time <= TimePeriodConstants.HOUR && time > 0)
	{
	}
	
	return x_axis;
}

def _sort(data, field, desc)
{
	def keys = [];
	def ret = [];
	if(data.id != null && data.id.size() > 0 && !data.id.contains(null)) {
		data.id.each {
			keys << Integer.parseInt(it);
		}
		Collections.sort(keys);

		keys.each {
			def index = data.id.indexOf(String.valueOf(it));
			ret << data[index];
		}
		
		if(desc == true) {
			ret.reverse();
		}
	} else {
		ret = data;
	}

	return ret;
}

def _generateStageOneData(filtered)
{
	def ret = [];
	
	// All: id, name, avearray, maxarray, average, maximum 
	for(AuditUnit record : filtered)
	{
		// PM CPU
		// Other: NONE
		if(record.getObject() == Constants.PMCPUUTILIZATION)
		{
			String id = record.getResult().get("id");
			if(ret.id.contains(id))
			{
				int index = ret.id.indexOf(id);
				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				ret[index].avearray << a;
				ret[index].maxarray << m;
			}
			else
			{
				def item = [:];
				def avearray = [];
				def maxarray = [];
			
				item.put("id", id);
				
				String name = record.getResult().get("name");
				item.put("name", name);
				
				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				avearray << a;
				maxarray << m;
				item.put("avearray", avearray);
				item.put("maxarray", maxarray);
				
				ret << item;
			}
		}
		//PM Memory
		//Other: NONE
		else if(record.getObject() == Constants.PMMEMORYUTILIZATION)
		{
			String id = record.getResult().get("id");
			if(ret.id.contains(id))
			{
				int index = ret.id.indexOf(id);
				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				ret[index].avearray << a;
				ret[index].maxarray << m;
			}
			else
			{
				def item = [:];
				def avearray = [];
				def maxarray = [];
			
				item.put("id", id);
				
				String name = record.getResult().get("name");
				item.put("name", name);
				
				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				
				avearray << a;
				maxarray << m;
				
				item.put("avearray", avearray);
				item.put("maxarray", maxarray);

				ret << item;
			}
		}
		//VM CPU
		//Other: instancename, instanceid, 
		//       ipaddress, ownerid, ownername, group
		else if(record.getObject() == Constants.VMCPUUTILIZATION)
		{
			String id = record.getResult().get("id");
			if(ret.id.contains(id))
			{
				int index = ret.id.indexOf(id);
				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				ret[index].avearray << a;
				ret[index].maxarray << m;
			}
			else
			{
				def item = [:];
				def avearray = [];
				def maxarray = [];
			
				item.put("id", id);
				item.put("name", record.getResult().get("name"));
				
				item.put("instancename", record.getResult().get("instancename"));
				item.put("instanceid", record.getResult().get("instanceid"));
				item.put("ipaddress", record.getResult().get("ipaddress"));
				item.put("ownerid", record.getResult().get("ownerid"));
				item.put("ownername", record.getResult().get("ownername"));
				item.put("groupname", record.getResult().get("groupname"));

				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				
				avearray << a;
				maxarray << m;
				
				item.put("avearray", avearray);
				item.put("maxarray", maxarray);

				ret << item;
			}
		}
		//VM Memory
		//Other: instancename, instanceid, 
		//       ipaddress, ownerid, ownername, group
		else if(record.getObject() == Constants.VMMEMORYUTILIZATION)
		{
			String id = record.getResult().get("id");
			if(ret.id.contains(id))
			{
				int index = ret.id.indexOf(id);
				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				ret[index].avearray << a;
				ret[index].maxarray << m;
			}
			else
			{
				def item = [:];
				def avearray = [];
				def maxarray = [];
			
				item.put("id", id);
				item.put("name", record.getResult().get("name"));
				
				item.put("instancename", record.getResult().get("instancename"));
				item.put("instanceid", record.getResult().get("instanceid"));
				item.put("ipaddress", record.getResult().get("ipaddress"));
				item.put("ownerid", record.getResult().get("ownerid"));
				item.put("ownername", record.getResult().get("ownername"));
				item.put("groupname", record.getResult().get("groupname"));

				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				
				avearray << a;
				maxarray << m;
				
				item.put("avearray", avearray);
				item.put("maxarray", maxarray);

				ret << item;
			}
		}
		//Storage
		//Other: NONE
		else if(record.getObject() == Constants.STORAGEUTILIZATION)
		{
			String id = record.getResult().get("id");
			if(ret.id.contains(id))
			{
				int index = ret.id.indexOf(id);
				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				ret[index].avearray << a;
				ret[index].maxarray << m;
			}
			else
			{
				def item = [:];
				def avearray = [];
				def maxarray = [];
			
				item.put("id", id);
				
				String name = record.getResult().get("name");
				item.put("name", name);
				
				HashMap a = new HashMap();
				HashMap m = new HashMap();
				a.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				//Defect 3627: To support legacy format of audit log
				a.put("value", Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0"));
				m.put("timestamp", Long.toString(record.getTimeStamp().getTime()));
				m.put("value", Double.parseDouble(record.getResult().get("max")));
				
				avearray << a;
				maxarray << m;
				
				item.put("avearray", avearray);
				item.put("maxarray", maxarray);

				ret << item;
			}
		}
		// IP Utilization
		else if(record.getObject() == Constants.IPUTILIZATION)
		{
			if(ret.size() == 0) {
				def item = [:];
				def utilarray = [];
				def utildata = [
				                "timestamp": String.valueOf(record.getTimeStamp().getTime()),
				                "value": Double.parseDouble(record.getResult().get("utilization"))
				                ];
				utilarray << utildata;
				item.put("utilarray", utilarray);
				ret << item;
			}
			else
			{
				def utildata = [
				                "timestamp": String.valueOf(record.getTimeStamp().getTime()),
				                "value": Double.parseDouble(record.getResult().get("utilization"))
				                ];
				ret[0].utilarray << utildata;  
			}
		}
		else if(record.getAction() == Action.LOGIN && record.getObject() == "RM02018")
		{
			if(record.getResult().get("result") == AuditResult.SUCCESS) {
				
				String id = record.getResult().get("id");
				if(ret.id.contains(id))
				{
					int index = ret.id.indexOf(id);
					ret[index].accesscount++;
				}
				else
				{
					def item = [:];
					item.put("id", id);
					String name = record.getResult().get("username");
					item.put("username", name);	
					item.put("accesscount", 1);
					ret << item;
				}
			}
		}

	}
	return ret;
}



def _generateTimePeriodData(ret, time)
{
	if(time > TimePeriodConstants.YEAR)
	{
		/*
		 * Time duration lasts more than one year, 
		 * it will have a monthly scale.
		 */
		ret.each {
			it.put("period", "months");
			if(it.avearray != null && it.maxarray != null) 
			{
				it.avearray = _summarizeAsTimePeriod(it.avearray, TimePeriodConstants.MONTH, false);
				it.maxarray = _summarizeAsTimePeriod(it.maxarray, TimePeriodConstants.MONTH, true);
			}
			else if(it.utilarray != null)
			{
				it.avearray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.MONTH, false);
				it.maxarray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.MONTH, true);
				it.remove("utilarray");
			}
			
		}
	}
	else if(time <= TimePeriodConstants.YEAR && time > TimePeriodConstants.MONTH)
	{
		/*
		 * Time duration lasts more than one month but less than a year, 
		 * it will have a monthly scale.
		 */
		ret.each {
			it.put("period", "months");
			if(it.avearray != null && it.maxarray != null) 
			{
				it.avearray = _summarizeAsTimePeriod(it.avearray, TimePeriodConstants.MONTH, false);
				it.maxarray = _summarizeAsTimePeriod(it.maxarray, TimePeriodConstants.MONTH, true);
			}
			else if(it.utilarray != null)
			{
				it.avearray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.MONTH, false);
				it.maxarray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.MONTH, true);
				it.remove("utilarray");
			}
		}
	}
	else if(time <= TimePeriodConstants.MONTH && time > TimePeriodConstants.WEEK)
	{
		/*
		 * Time duration lasts more than one week but less than a month, 
		 * it will have a weekly scale.
		 */
		ret.each {
			it.put("period", "weeks");
			if(it.avearray != null && it.maxarray != null) 
			{
				it.avearray = _summarizeAsTimePeriod(it.avearray, TimePeriodConstants.WEEK, false);
				it.maxarray = _summarizeAsTimePeriod(it.maxarray, TimePeriodConstants.WEEK, true);
			}
			else if(it.utilarray != null)
			{
				it.avearray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.WEEK, false);
				it.maxarray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.WEEK, true);
				it.remove("utilarray");
			}
		}
	}
	else if(time <= TimePeriodConstants.WEEK && time > TimePeriodConstants.DAY)
	{
		/*
		 * Time duration lasts more than one day but less than a week, 
		 * it will have a daily scale.
		 */
		ret.each {
			it.put("period", "days");
			if(it.avearray != null && it.maxarray != null) 
			{
				it.avearray = _summarizeAsTimePeriod(it.avearray, TimePeriodConstants.DAY, false);
				it.maxarray = _summarizeAsTimePeriod(it.maxarray, TimePeriodConstants.DAY, true);
			}
			else if(it.utilarray != null)
			{
				it.avearray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.DAY, false);
				it.maxarray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.DAY, true);
				it.remove("utilarray");
			}
		}
	}
	else if(time <= TimePeriodConstants.DAY && time > TimePeriodConstants.HOUR)
	{
		/*
		 * Time duration lasts more than one hour but less than a day, 
		 * it will have a hourly scale.
		 */
		ret.each {
			it.put("period", "hours");

			if(it.avearray != null && it.maxarray != null) 
			{
				it.avearray = _summarizeAsTimePeriod(it.avearray, TimePeriodConstants.HOUR, false);
				it.maxarray = _summarizeAsTimePeriod(it.maxarray, TimePeriodConstants.HOUR, true);
			}
			else if(it.utilarray != null)
			{
				it.avearray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.HOUR, false);
				it.maxarray = _summarizeAsTimePeriod(it.utilarray, TimePeriodConstants.HOUR, true);
				it.remove("utilarray");
			}
		}
	}
	else if(time <= TimePeriodConstants.HOUR && time > 0)
	{
		/* NOT ALLOWED 
		 * 
		 * Return an empty value
		 */
		 return [];
		
	}
	
	return ret;
}

def _summarizeAsTimePeriod(array, period, max)
{
	def ret = [];
	if(period == TimePeriodConstants.MONTH) 
	{
		def tempArray = [];
		if(max)
		{
			array.each {
				def month = _getMonth(it.timestamp);
				def year = _getYear(it.timestamp);
				def key = "" + year + "-" + month;
				if(tempArray.id.contains(key))
				{
					int index = ret.id.indexOf(key);
					if(tempArray[index].value < it.value)
					{
						tempArray[index].value = it.value;
					}
					
				}
				else
				{
					def item = [
					            "id": key,
								"value": it.value
					            ]
					tempArray << item;
				}
			}
			
			tempArray.each {
				def item = [:];
				item.timestamp = it.id;
				item.value = it.value;
				ret << item;
			}
			
		}
		else
		{
			array.each {
				def month = _getMonth(it.timestamp);
				def year = _getYear(it.timestamp);
				def key = "" + year + "-" + month;
				if(tempArray.id.contains(key))
				{
					int index = ret.id.indexOf(key);
					tempArray[index].value += it.value;
					tempArray[index].counts++;
				}
				else
				{
					def item = [
					            "id": key,
								"value": it.value,
								"counts": 1
					            ]
					tempArray << item;
				}
			}
			
			tempArray.each {
				def item = [:];
				item.timestamp = it.id;
				item.value = ((Number)it.value).doubleValue()/it.counts; 
				ret << item;
			}
		}
	}
	else if(period == TimePeriodConstants.WEEK)
	{
		def tempArray = [];
		if(max)
		{
			array.each {
				def month = _getMonthName(it.timestamp);
				def week = _getWeekOfMonth(it.timestamp);
				def key = "Week " + week + ", " + month;
				if(tempArray.id.contains(key))
				{
					int index = ret.id.indexOf(key);
					if(tempArray[index].value < it.value)
					{
						tempArray[index].value = it.value;
					}
					
				}
				else
				{
					def item = [
					            "id": key,
								"value": it.value
					            ]
					tempArray << item;
				}
			}
			
			tempArray.each {
				def item = [:];
				item.timestamp = it.id;
				item.value = it.value;
				ret << item;
			}
			
		}
		else
		{
			array.each {
				def month = _getMonthName(it.timestamp);
				def week = _getWeekOfMonth(it.timestamp);
				def key = "Week " + week + ", " + month;
				if(tempArray.id.contains(key))
				{
					int index = ret.id.indexOf(key);
					tempArray[index].value += it.value;
					tempArray[index].counts++;
				}
				else
				{
					def item = [
					            "id": key,
								"value": it.value,
								"counts": 1
					            ]
					tempArray << item;
				}
			}
			
			tempArray.each {
				def item = [:];
				item.timestamp = it.id;
				item.value = ((Number)it.value).doubleValue()/it.counts; 
				ret << item;
			}
		}
	}
	else if(period == TimePeriodConstants.DAY)
	{
		def tempArray = [];
		if(max)
		{
			array.each {
				def date = _getDate(it.timestamp);
				def key = "" + date;
				if(tempArray.id.contains(key))
				{
					int index = ret.id.indexOf(key);
					if(tempArray[index].value < it.value)
					{
						tempArray[index].value = it.value;
					}
					
				}
				else
				{
					def item = [
					            "id": key,
								"value": it.value
					            ]
					tempArray << item;
				}
			}
			
			tempArray.each {
				def item = [:];
				item.timestamp = it.id;
				item.value = it.value;
				ret << item;
			}
			
		}
		else
		{
			array.each {
				def date = _getDate(it.timestamp);
				def key = "" + date;
				if(tempArray.id.contains(key))
				{
					int index = ret.id.indexOf(key);
					tempArray[index].value += it.value;
					tempArray[index].counts++;
				}
				else
				{
					def item = [
					            "id": key,
								"value": it.value,
								"counts": 1
					            ]
					tempArray << item;
				}
			}
			
			tempArray.each {
				def item = [:];
				item.timestamp = it.id;
				item.value = ((Number)it.value).doubleValue()/it.counts; 
				ret << item;
			}
		}
	}
	else if(period == TimePeriodConstants.HOUR)
	{
		def tempArray = [];
		if(max)
		{
			array.each {
				def hour = _getHourOfDay(it.timestamp);
				def key = "" + hour;
				if(tempArray.id.contains(key))
				{
					int index = ret.id.indexOf(key);
					if(tempArray[index].value < it.value)
					{
						tempArray[index].value = it.value;
					}
					
				}
				else
				{
					def item = [
					            "id": key,
								"value": it.value
					            ]
					tempArray << item;
				}
			}
			
			tempArray.each {
				def item = [:];
				item.timestamp = it.id;
				item.value = it.value;
				ret << item;
			}
			
		}
		else
		{
			array.each {
				def hour = _getHourOfDay(it.timestamp);
				def key = "" + hour;
				if(tempArray.id.contains(key))
				{
					int index = ret.id.indexOf(key);
					tempArray[index].value += it.value;
					tempArray[index].counts++;
				}
				else
				{
					def item = [
					            "id": key,
								"value": it.value,
								"counts": 1
					            ]
					tempArray << item;
				}
			}
			
			tempArray.each {
				def item = [:];
				item.timestamp = it.id;
				item.value = ((Number)it.value).doubleValue()/it.counts; 
				ret << item;
			}
		}
		
	}

	return ret;
}

def _getYear(time)
{
	Calendar c = Calendar.getInstance();
	c.setTimeInMillis(Long.parseLong(time));
	return (String.valueOf(c.get(Calendar.YEAR))).substring(2);
}

def _getMonthName(time)
{
	Calendar c = Calendar.getInstance();
	c.setTimeInMillis(Long.parseLong(time));
	return c.getDisplayName(Calendar.MONTH, Calendar.SHORT, Locale.US);
}
def _getMonth(time)
{
	Calendar c = Calendar.getInstance();
	c.setTimeInMillis(Long.parseLong(time));
	return c.get(Calendar.MONTH) + 1;
}

def _getWeekOfMonth(time)
{
	Calendar c = Calendar.getInstance();
	c.setTimeInMillis(Long.parseLong(time));
	return c.get(Calendar.WEEK_OF_MONTH);
}

def _getDate(time)
{
	Calendar c = Calendar.getInstance();
	c.setTimeInMillis(Long.parseLong(time));
	return c.get(Calendar.DATE);
}

def _getHourOfDay(time)
{
	Calendar c = Calendar.getInstance();
	c.setTimeInMillis(Long.parseLong(time));
	return c.get(Calendar.HOUR_OF_DAY);
}



def generateUsageData(filtered)
{
/*
 * returned value:
 * 
   [
	 "userid" :
	 "username" :
	 "groupname" : 
	 "instancehours" :
	 "vmhours" :
	 "avecpu" :
	 "avemem" :
	 "aveips" :
   ];
*/	
	def usageReport = [];

	def INTERVAL = zget('/config/rainmaker/reporting/interval', 900000);
	double INVERVALHOUR = 
		((Number)INTERVAL).doubleValue() / TimePeriodConstants.HOUR;

	for(AuditUnit record : filtered)
	{
		// VM CPU
		if(record.getObject() == Constants.VMCPUUTILIZATION)
		{
			String userid = record.getResult().get("ownerid");
			if(usageReport.userid.contains(userid))
			{
				int index = usageReport.userid.indexOf(userid);
				
				//Defect 3627: To support legacy format of audit log
				Double avecpu = Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0");
				
				if (usageReport[index].avecpuArray == null)
				{
					def avecpuArray = [];
					avecpuArray << avecpu;
					usageReport[index].put("avecpuArray", avecpuArray);
				}
				else
				{
					usageReport[index].avecpuArray << avecpu;
				}
				
				/*
				 * Update vmhours field.
				 */
				usageReport[index].vmhours += INVERVALHOUR;
				
				/*
				 * Update instancehours field
				 */
				def instanceid = record.getResult().get("instanceid");
				if(usageReport[index].instancearray.id
					.contains(instanceid))
				{
					// Update existing instancehours data.
					int i = usageReport[index].instancearray.id.indexOf(instanceid);
					if(record.getTimeStamp().getTime() - 
						usageReport[index].instancearray[i].lastactivity > 
						INTERVAL)
					{
						usageReport[index].instancearray[i].lastactivity =
							record.getTimeStamp().getTime();
						usageReport[index].instancearray[i].hours += INVERVALHOUR;
					}
				}
				else
				{
					// A new instance comes.
					def instancedata = [:];
					instancedata.put("id", instanceid);
					instancedata.put("lastactivity", record.getTimeStamp().getTime());
					instancedata.put("hours", INVERVALHOUR);
					usageReport[index].instancearray << instancedata;
				}
				
				/*
				 * Update ipdata field
				 */
				def timestamp = record.getTimeStamp().getTime();
				def exist = false;
				usageReport[index].iparray.each {
					if(timestamp >= it.lastactivity &&
						timestamp < it.lastactivity + INTERVAL)
					{
						exist = true;
						it.number++;
					}
				}
				if(false == exist)
				{
					def ipdata = [:];
					ipdata.put("lastactivity", record.getTimeStamp().getTime());
					ipdata.put("number", 1);
					usageReport[index].iparray << ipdata;
				}

			}
			else
			{
				def usageItem = [:];
				def avecpuArray = [];
			
				usageItem.put("userid", userid);
				
				String username = record.getResult().get("ownername"); 
				usageItem.put("username", username);
				
				String groupname = record.getResult().get("groupname");
				usageItem.put("groupname", groupname);
				
				//Defect 3627: To support legacy format of audit log
				Double avecpu = Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0");

				avecpuArray << avecpu;
				usageItem.put("avecpuArray", avecpuArray);

				/*
				 * Added vmhours field.
				 */
				usageItem.put("vmhours", INVERVALHOUR);
				
				/* Add instancehours stat. field
				 */
				def instancedata = [:];
				instancedata.put("id", record.getResult().get("instanceid"));
				instancedata.put("lastactivity", record.getTimeStamp().getTime());
				instancedata.put("hours", INVERVALHOUR);
				def instancearray = [];
				instancearray << instancedata;
				usageItem.put("instancearray", instancearray);
				
				/*
				 * Add ip stat. field
				 * This field holds the raw data that is used 
				 * summarize the ip usage.
				 */
				def ipdata = [:];
				ipdata.put("lastactivity", record.getTimeStamp().getTime());
				ipdata.put("number", 1);
				def iparray = [];
				iparray << ipdata;
				usageItem.put("iparray", iparray);
				
				usageReport << usageItem;
			}
		}
		// VM Mem
		else if(record.getObject() == Constants.VMMEMORYUTILIZATION)
		{
			String userid = record.getResult().get("ownerid");
			if(usageReport.userid.contains(userid))
			{
				int index = usageReport.userid.indexOf(userid);
				
				//Defect 3627: To support legacy format of audit log
				Double avemem = Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0");
				
				if (usageReport[index].avememArray == null)
				{
					def avememArray = [];
					avememArray << avemem;
					usageReport[index].put("avememArray", avememArray);
				}
				else
				{
					usageReport[index].avememArray << avemem;
				}

			}
			else
			{
				def usageItem = [:];
				def avememArray = [];
			
				usageItem.put("userid", userid);
				
				String username = record.getResult().get("ownername"); 
				usageItem.put("username", username);
				
				String groupname = record.getResult().get("groupname");
				usageItem.put("groupname", groupname);
				
				//Defect 3627: To support legacy format of audit log
				Double avemem = Double.parseDouble(record.getResult().get("avg")?:record.getResult().get("ave")?:"0");

				avememArray << avemem;
				usageItem.put("avememArray", avememArray);

				/*
				 * Added vmhours field.
				 */
				usageItem.put("vmhours", INVERVALHOUR);
				
				/* Add instancehours stat. field
				 */
				def instancedata = [:];
				instancedata.put("id", record.getResult().get("instanceid"));
				instancedata.put("lastactivity", record.getTimeStamp().getTime());
				instancedata.put("hours", INVERVALHOUR);
				def instancearray = [];
				instancearray << instancedata;
				usageItem.put("instancearray", instancearray);
				
				/*
				 * Add ip stat. field
				 * This field holds the raw data that is used 
				 * summarize the ip usage.
				 */
				def ipdata = [:];
				ipdata.put("lastactivity", record.getTimeStamp().getTime());
				ipdata.put("number", 1);
				def iparray = [];
				iparray << ipdata;
				usageItem.put("iparray", iparray);

				usageReport << usageItem;
			}
		}
	
	}
	
	usageReport.each {
		if(it.iparray != null && it.iparray.size() != 0) {
			it.aveips = it.iparray.number.sum() / it.iparray.size();
			it.remove("iparray");
		} else {
			it.aveips = 0;
		}
		
		if(it.instancearray != null && it.instancearray.size() != 0) {
			it.instancehours = it.instancearray.hours.sum();
			it.remove("instancearray");
		} else {
			it.instancehours = 0;
		}
		
		if(it.avecpuArray != null && it.avecpuArray.size() != 0) {
			it.avecpu = it.avecpuArray.sum() / it.avecpuArray.size();
			it.remove("avecpuArray");
		} else {
			it.avecpu = 0;
		}
		
		if(it.avememArray != null && it.avememArray.size() != 0) {
			it.avemem = it.avememArray.sum() / it.avememArray.size();
			it.remove("avememArray");
		} else {
			it.avemem = 0;
		}
		
	}
	
	if(usageReport.size() > 0) {
		def total = [:];
		total.put("userid", 0);
		total.put("username", "total");
		if(usageReport.instanceshours != null && usageReport.instancehours.size() != 0) {
			total.put("instancehours", usageReport.instancehours.sum());
		} else {
			total.put("instancehours", 0);
		}
		
		if(usageReport.avecpu != null && usageReport.avecpu.size() != 0) {
			total.put("avecpu", usageReport.avecpu.sum()/usageReport.avecpu.size());
		} else {
			total.put("avecpu", 0);
		}
		
		if(usageReport.avemem != null && usageReport.avemem.size() != 0) {
			total.put("avemem", usageReport.avemem.sum()/usageReport.avemem.size());
		} else {
			total.put("avemem", 0);
		}
		
		if(usageReport.avemem != null && usageReport.avemem.size() != 0) {
			total.put("vmhours", usageReport.vmhours.sum());
		} else {
			total.put("vmhours", 0);
		}
		
		if(usageReport.avemem != null && usageReport.avemem.size() != 0) {
			total.put("aveips", usageReport.aveips.sum());
		} else {
			total.put("aveips", 0);
		}

		usageReport << total;
	}
	
	
	// Deal with decimal format.
	java.text.DecimalFormat df = new java.text.DecimalFormat("0.00");
	usageReport.each {
		it.instancehours = df.format(it.instancehours);
		it.avecpu = df.format(it.avecpu);
		it.avemem = df.format(it.avemem);
		it.vmhours = df.format(it.vmhours);
		it.aveips = df.format(it.aveips);
	}
	
	
	return usageReport;
}

