/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;

import com.ibm.websphere.users.Permissions;

//
// This REST API supports the following DP APIs:
//
// ethernetInterface (get, update)
// ethernetInterfaces (get)
// dnsServers (get, update)
// ntpServers (get, update)
// timeZone (get, update)
// timeZones (get)
// power (update)
// locatorDevice (update)
//

def onRetrieve()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
        
    def results = null;
    
    try
    {
        def type = capitalize(request.params.applianceId[]);
        String param = (event.pathInfo[]).substring(1);
        if (param == null) {
	        logger.FINEST { "Executing method get${type}()." };
	        results = "get${type}"();
        }
        else {
        	logger.FINEST { "Executing method get${type}(${param})." };
	        results = "get${type}"(param);
        }
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08049', t);
    }
    
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView();
    request.json.output = results;
    render();
}

def onUpdate()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
        
    def type = null;
    def json = null;
    
    try
    {
        type = capitalize(request.params.applianceId[]);
        json = Json.decode(request.input[]);
        
        if (event.pathInfo[] != null && event.pathInfo[].length() > 1)
        	json.id = event.pathInfo[].substring(1);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    try
    {
        logger.FINEST { "Executing method update${type}()." };
        "update${type}"(json);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08049', t);
    }
}
