/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rainmaker.appliance.firmware.FirmwareUpgradeHandler;
import com.ibm.websphere.rainmaker.logging.MessageUtils;

def onList() 
{
    request.writer[].write(app.lastFirmwareUpgradeResult[] ?: '');	
}

def onCreate()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    if (request.files[].isEmpty())
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08014');
    
    def fileMetadata = request.files[].firmware[0];
    def uploadedFile = new File(fileMetadata.path);  
    
    /*
     * AKS: Uncomment the following lines when using the "new" firmware packaging and comment the lines below
     * that perform the file copies to move the firmware into /drouter/temporary.
     */
     
     /*
     	FirmwareUpgradeHandler fuh = new FirmwareUpgradeHandler();
    	fuh.upgrade (uploadedFile);
    */
    
    def fileName = fileMetadata.filename;
    
    //
    // HACK: IE7 sends a full local path, so we have to remove everything 
    //       before the last backslash
    //
    def lastSlash = fileName.lastIndexOf('\\');
    fileName = fileName.substring(lastSlash < 0 ? 0 : lastSlash + 1);
    
    def destFile = new File('/drouter/temporary', fileName);
    
    logger.FINE { "Copying firmware file to ${destFile.path}..." };
    
    copyFile (uploadedFile, destFile.parentFile, destFile.getName());    
    def success = destFile.exists();
    
    if (!success)
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08011');
    
    logger.FINE { "Creating upgrade request with file ${destFile.path}..." };
    /*    
    Thread.start {
        upgradeFirmware(destFile.name);
    }
    */
    logger.INFO { MessageUtils.message(false, 'RM08064', []) };
    
    app.lastFirmwareUpgradeResult = MessageUtils.message(user.lang[], 'RM08013', []);
    request.writer[].write("<div id='results'><span>${app.lastFirmwareUpgradeResult[]}</span></div>");
    
    upgradeFirmware(destFile.name);
}

def onDeleteCollection()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    Thread.start {
        rollbackFirmware();
    }
    
    logger.INFO { MessageUtils.message(false, 'RM08065', []) };
    
    request.writer[].write("<div id='results'><span>${MessageUtils.message(user.lang[], 'RM08013', []) }</span></div>");
}

def onPostMember()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    def destFile = new File('/drouter/temporary', request.params.firmwareId[]);
    
    logger.FINE { "Saving uploaded firmware file to ${destFile.path}..." };

    try {
        destFile.withOutputStream { it << request.input[] }    
    }
    catch (Exception e) {    
        return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08011');
    }
    
    logger.FINE { "Creating upgrade request with file ${destFile.path}..." };
        
    Thread.start {
        upgradeFirmware(destFile.name);
    }
    
    logger.INFO { MessageUtils.message(false, 'RM08064', []) };
    
    request.status = 202;
}
