/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;

import com.ibm.websphere.users.Permissions;

def onPutCollection()
{
    def session = user.session[];
    def userID = com.ibm.websphere.rest.RESTUtils.getRequestedUserID();
    
    if (userID != session.id && !session.permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    def change = null;
    def targetUser = null;
    
    try
    {
        change = Json.decode(request.input[]);
        targetUser = getUser(userID);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    change.value = change.value.trim();
    change.verification = change.verification.trim();
    
    if (!change.value.equals(change.verification))
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08006');
    
    if (change.value.length() == 0)
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08017');
    
    logger.INFO { "User ${session.username} is changing the password for user ${targetUser.username}..." };
    
    def throwable;
    def result = false;
    try
    {
        result = changePassword(userID, change.value);
    }
    catch (Throwable t)
    {
        throwable = t;
    }

    if(throwable || !result){
        logger.SEVERE { "The password for user ${targetUser.username} could not be changed." };
        if(throwable){
                return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08007', throwable);
        }else{
                return com.ibm.websphere.rest.RenderUtils.renderError(500, 'RM08007');
        }
    }else{
        logger.INFO { "The password for user ${targetUser.username} was changed successfully." }
    }
    
    request.status = 204;
}
