/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;
import xsa.wxs.context.GlobalContextURIs.Appliance;
import com.ibm.websphere.users.Permissions;
import com.ibm.ws.xc10.syslog.rest.Syslog;

def _normalize(name)
{
    return name.replaceAll("['\"]", "");
}

//don't think this is called (we never update a collection)
def onPutCollection()
{
	logger.INFO{ 'entered onPutCollection()'}
	
	if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
	def syslogConfigMap = Json.decode(request.input[]);
	logger.INFO{ 'updating syslogConfigMap: ' + syslogConfigMap }
	// update the sMash config zone for syslog configuration (invokes Listener which updates the Syslog logging Handler)
	// for all processes on XC10.
    config.logging.remote.syslog =  syslogConfigMap;
    logger.INFO{ 'updated GC /config/logging/remote/syslog with: ' + syslogConfigMap }
    logger.INFO{ 'before syslog REST internal call' }
    Syslog.updateSyslogSettings(syslogConfigMap);
	logger.INFO{ 'after syslog REST internal call' }
	
    // persist to configuration file so information is available on restart.
    writeToDisk();
    
    request.status = 200;
    renderView();

}


def onList()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    logger.INFO{'PPM entered syslog onList method'}
    
    renderView();
    logger.INFO{'PPM leaving syslog onList method'}
}

def onCreate(){
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
	renderView();	
}

def onUpdate(){

    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
	
	logger.INFO{ 'entered onUpdate()'}	
	//def loggerName = request.params.syslogId[];
	
	def syslogConfigMap = Json.decode(request.input[]);
		
	logger.INFO{ 'updated syslogConfigMap: ' + syslogConfigMap }
		
	// update the sMash config zone for syslog configuration (invokes Listener which updates the Syslog logging Handler)
	// for all processes on XC10.
    config.logging.remote.syslog[] =  syslogConfigMap;
    
    // persist to configuration file so information is available on restart.
    writeToDisk();
    logger.INFO{ 'exiting onUpdateCollection()'}
}

def onDelete(){
	
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

	renderView();
}

def parseLevel(spec, level){
	def parsedLevel;
	try{
		parsedLevel = java.util.logging.Level.parse(mapToLevel(spec, level));
		return parsedLevel;
	}catch(IllegalArgumentException e){
		return null;
	}
}

def mapToLevel(spec, level) {
	// converts RAS level (String) to JDK level (String)
	def levelToReturn = null;
	logger.FINE {' mapToLevel - level.toUpperCase(): ' + level.toUpperCase()};
	if (level.equalsIgnoreCase("event")) {
		if(spec.equalsIgnoreCase("*")) {
			levelToReturn = "INFO";
			logger.FINER {' mapToLevel - converting invalid trace level: event, to logger level INFO for default logger: ' + spec}
		} else {
			levelToReturn = "FINE";
			logger.FINER {' mapToLevel - converting EVENT to FINE'}
		}		
	} else if (level.equalsIgnoreCase("entryExit")) {
		if(spec.equalsIgnoreCase("*")) {
			levelToReturn = "INFO";
			logger.FINER {' mapToLevel - converting invalid trace level: entryExit, to logger level INFO for default logger: ' + spec}
		} else {
			levelToReturn = "FINER";
			logger.FINER {' mapToLevel - converting ENTRYEXIT to FINER'}
		}
	} else if (level.equalsIgnoreCase("debug")) {
		if(spec.equalsIgnoreCase("*")) {
			levelToReturn = "INFO";
			logger.FINER {' mapToLevel - converting invalid trace level: debug, to logger level INFO for default logger: ' + spec}
		} else {
			levelToReturn = "FINEST";
			logger.FINER {' mapToLevel - converting DEBUG to FINEST'}
		}
	} else {
		if(spec.equalsIgnoreCase("*")) {
			if(!(level.equalsIgnoreCase("SEVERE") || level.equalsIgnoreCase("WARNING") 
				|| level.equalsIgnoreCase("INFO") || level.equalsIgnoreCase("OFF"))) {
				levelToReturn = "INFO";
				logger.FINER {' mapToLevel - converting invalid trace level: ' + level + ', to logger level INFO for default logger: ' + spec}
			} else {
				levelToReturn = level;
				logger.FINER {' mapToLevel - keeping level at: ' + level + ', for default logger: ' + spec}
			}
		} else {
			levelToReturn = level;	
			logger.FINER {' mapToLevel - keeping level at: ' + level + ' for specification: ' + spec}
		}
	}
	return levelToReturn;
}

def writeToDisk(){
	// tmp.rainmaker.backupRunning[] == true : do not write to disk
	// but update the runtime to help in pd process
	if( tmp.rainmaker.backupRunning[]){
		logger.FINE { "Server is in backup mode and disks might be in process of backing up. Skip persisting changes to disk" };
		return;
	}
	
	// adding new config file for syslog properties
	// #config.rainmaker.disk.shared[] is /drouter/ramdisk2/mnt/raid-volume/raid0/shared
	//def outputFile = config.rainmaker.disk.root[] + "/config/logging-custom.config";
	
	def outputFile = config.rainmaker.disk.shared[] + "/config/syslog.config";
	
	String tsk=config.rainmaker.defaultPersistentZone[] + "/logging/remote/syslog";
	zpost(Appliance.lock, tsk);
	
    try {
		def writer = new File(outputFile).newWriter();
		try{
			if(config.logging.remote.syslog[] instanceof Map) {
				Map configSyslogLevels = new HashMap(config.logging.remote.syslog[]);
				Map configSyslogLevelsNew = new HashMap();
				
				if (configSyslogLevels != null && configSyslogLevels.size() > 0) {
					def syslogConfigEntries = configSyslogLevels.entrySet();
					syslogConfigEntries.each { syslogConfigEntry ->
						logger.FINE{'before zeroizing - key=' + syslogConfigEntry.key + ", value=" + syslogConfigEntry.value};
						def zeroizedKey = zeroizeTraceLevel(syslogConfigEntry.key);
						def zeroizedValue = zeroizeTraceLevel(syslogConfigEntry.value);
						configSyslogLevelsNew[zeroizedKey] = zeroizedValue;
						logger.FINE{'after zeroizing - key=' + zeroizedKey + ", value=" + zeroizedValue};
					}
				}
				writer.writeLine("/config/logging/remote/syslog=")
				//writer.writeLine(Json.encode(config.logging.levels[]))
				writer.writeLine(Json.encode(configSyslogLevelsNew))
				writer.close()
			}
		} catch (Exception e) {
			logger.WARNING {"Exception occured while trying to persist syslog config to the file system. Exception: " + e.getMessage()}
			e.printStackTrace();
			throw new RuntimeException ("failed to persist syslog settings");
		} finally {
			try{
				// make setting take effect in memory
				zero.core.config.ConfigLoader.parseFile(new File(outputFile))
				// persist file
				zput(config.rainmaker.defaultPersistentZone[] + "/logging/remote/syslog", config.logging.remote.syslog[] )		
			} catch (Throwable t) {
				logger.WARNING {"Exception occured while trying to parse file containing GC key: " + t.getMessage()}
		    	t.printStackTrace();
				// catch exception and clear out the file to prevent possible
				// failures loading the file after restart
				new File(outputFile).delete();
			}
		}
	} finally {
	  zpost(Appliance.unlock, tsk);
	}	
}

def zeroizeTraceLevel(originalTraceString) {
	// escape characters that zero doesn't like, like $ (which becomes $$)
	def replaceStr=/\$\$/;	
	def newTraceString = originalTraceString.replaceAll("\\\$", replaceStr)
	return newTraceString;
}

def renderView(){
    request.json.output = config.logging.remote.syslog[]
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView()
    request.view.resourceURL= '/resources/syslog';
    request.view.logLevelType= 'uiSyslog';
    request.view.logLevelHeader= 'BR10041';
    request.view.syslogConfig = config.logging.remote.syslog[]
    render()

}
