/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

import zero.json.Json;
import xsa.wxs.context.GlobalContextURIs.Appliance;

import com.ibm.websphere.users.Permissions;

def _normalize(name)
{
    return name.replaceAll("['\"]", "");
}

//don't think this is called (we never update a collection)
def onPutCollection()
{
	logger.INFO{ 'entered onPutCollection()'}
	
	if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    def newTraceString = null;
    
    try
    {
    	newTraceString = Json.decode(request.input[]);
    }
    
    catch (Throwable t)
    {
        return com.ibm.websphere.rest.RenderUtils.renderError(400, 'RM08002', t);
    }
    
    
    logger.FINE { "Trace spec is " + newTraceString }

    
    def levels = config.logging.levels[]
    for (loggingLevel in levels){
    	// first remove all of the current loggers not in new trace string
    	if(!newTraceString.containsKey(loggingLevel.key)){
    		def mappedLoggerName = (loggingLevel.key== '*')?'':loggingLevel.key;
    		java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(null);
    	}
    }
    
    def names = newTraceString.keySet();
    
    names.each {
        def level = newTraceString.remove(it);
         def loggerName = _normalize(it)
	   	 if(! ( loggerName == null || "".equals(loggerName.trim())) ){
			 newTraceString[] = level;
		 }
    }
    
    config.logging.levels = newTraceString;
    
    levels = config.logging.levels[]
    for (loggingLevel in levels){
        def mappedLoggerName = (loggingLevel.key== '*')?'':loggingLevel.key;
        java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(parseLevel(loggingLevel.key, loggingLevel.value) ) ;
    }
    
    writeToDisk();
    
    
    request.status = 200;
    renderView();

}


def onList()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    renderView();
}

def onCreate(){

    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
	
	 def json = Json.decode(request.input[]);
	 
	 //replace everything below here with what is in the comments
	 def loggerNames = json["logger"];
	 if(loggerNames == null || "".equals(loggerNames.trim())){
		 return;
	 }
	 
	 logger.FINE{ 'Trace string (pre-processed): ' + loggerNames}
	  
	  def loggerNamesList = loggerNames.tokenize(":");
	  logger.INFO{ 'loggerNamesList.size(): ' + loggerNamesList.size() }
	  for (loggerName in loggerNamesList) {

		  logger.FINE{ 'pre-normalized loggerName: ' + loggerName }
		  loggerName = _normalize(loggerName);
		  logger.FINE{ 'post-normalized loggerName: ' + loggerName }
		  
		  logger.INFO{ 'loggerName to process: ' + loggerName }
		  
		  // lets find the "=" sign now to get the logging_spec=logging_level pairs
		  // we also get rid of all spaces at begining and end, and before and after 
		  // the equal sign
		  def replaceStr = /\\$/;
		  def loggerArray = loggerName.trim().replaceAll("\\s","").replaceAll(/\\$/, replaceStr).split('=');
		  logger.FINE{ 'loggerArray: ' + loggerArray + ', length: ' + loggerArray.length }	
		  
		  switch ( loggerArray.length ) {
			  case 0:
				 //Client-side validation should let this condition occur
				 logger.WARNING{ 'There was an error setting the trace specification and level, please check your trace string, and try again.'}
			  break;
			  case 1:
			     // user forgot to type the level and state, or the equals and the level/state altogether, default the level to INFO
				 // ENABLED is default and semantics are to just enable this trace component as INFO
				 // ex: "Grid=" or "Grid"
			     // maps root logger for sMash zone store to root logger for JDK
			     def mappedLoggerName = (loggerArray[0] == '*') ? '' : loggerArray[0];
			     logger.INFO{ ' loggerArray[0] converted from: ' + loggerArray[0] + ', to mappedLoggerName: ' + mappedLoggerName }
			     // JDK style logging uses blank instead of * for the "root" logger
			     java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(java.util.logging.Level.INFO);
			     // Will set level to default level because none was provided
			     // (config is a non-persistent sMash zone)
			  
			     // sMash should not store '' for root logger instead of '*'
			     config.logging.levels[loggerArray[0]] =  "INFO";			     
			  
				 logger.INFO{' ' + loggerArray[0] + ', defaults to state ENABLED and defaults to level: INFO '}
			  break;
			  case 2:
			    //  missing only trace state, which defaults to ENABLED, which sematically means we display this traceSpec in the UI and enabled it
			  	def loggerLevel = loggerArray[1].toUpperCase();
				def convertedLevel = null;
				
			    logger.FINE{ 'Normalized JDK logger level: ' + loggerLevel}
			    java.util.logging.Level parsedLevel = parseLevel(loggerArray[0], loggerLevel);
			    logger.FINE{ 'parsedLevel is now: ' + (parsedLevel != null ? parsedLevel : 'null')}
			  
			    // maps root logger for sMash zone store to root logger for JDK
			    def mappedLoggerName = (loggerArray[0] == '*') ? '' : loggerArray[0];
				
				if (mappedLoggerName.equals('')) {
			    	logger.FINE{ ' loggerArray[0] converted from: ' + loggerArray[0] + ', to mappedLoggerName: ' + mappedLoggerName}
				}

			    // null means there was an IllegalArgumentException because we passed an illegal level.
			    if (parsedLevel == null) {
				    logger.FINE{ ' Applying the defaul level of INFO because trace level ' + loggerArray[1] + 'was not valid'}
				    java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(java.util.logging.Level.INFO);
				    
					//this is the default level, config is a non-persistent sMash zone
				    config.logging.levels[loggerArray[0]] =  "INFO";
					convertedLevel = "INFO";
			    } else {
					// using the parsedLevel to set JDK controlled trace.
				    java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(parsedLevel);
				    
					// config is a non-persistent sMash zone
					
					// mapToLevel converts a RAS traceType to JDK Logging Type (as a String)
					// whereas parseLevel() returns it as a Logger object
					
					// ex: entryExit is converted to FINER
					
					// calling mapToLevel directly to add the RAS trace set via GlobalContext
				    convertedLevel = mapToLevel(loggerArray[0], loggerLevel);  
				    config.logging.levels[loggerArray[0]] =  convertedLevel;
				    logger.INFO{ ' Set trace specification ' + loggerArray[0] + ', and tracing level: ' + convertedLevel}
			    }
				logger.INFO{' '+ loggerArray[0] + ' is set to level ' + convertedLevel + ' and defaults to state ENABLED'}
			  break;
			  case 3:
			     def loggerLevel = loggerArray[1].toUpperCase();				 
				 def convertedLevel = null;
				 
				 logger.FINE{ 'Normalized JDK logger level: ' + loggerLevel }
				 java.util.logging.Level parsedLevel = parseLevel(loggerArray[0], loggerLevel);
				 logger.FINE{ 'parsedLevel is now: ' + (parsedLevel != null ? parsedLevel : 'null')}
				 
				 if(parsedLevel == null) {
					 logger.FINE{ ' applying default level of INFO'};
					 parsedLevel = java.util.logging.Level.INFO;
					 convertedLevel = "INFO";
				 } else {
				 	convertedLevel = mapToLevel(loggerArray[0], loggerLevel);
				 }
			   
				 // maps root logger for sMash zone store to root logger for JDK
				 def mappedLoggerName = (loggerArray[0] == '*') ? '' : loggerArray[0];
				 
				 if (mappedLoggerName.equals('')) {
					 logger.FINE{ 'loggerArray[0] converted from: ' + loggerArray[0] + ', to mappedLoggerName: ' + mappedLoggerName }
				 }				 	 
				 
				 // the full case, setting the trace state to ENABLED semantically does nothing to the trace string, but setting it to DISABLED
				 // sets it to OFF in the UI, and would be converted to "all=disabled" when setting the RAS trace for ObjectGrid
				 // if the state is anything other than enabled or disabled is not valid, and will default to ENABLED
				 if (loggerArray[2].equalsIgnoreCase("enabled")) {
					 java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(parsedLevel);
				     logger.FINE {' entering mapToLevel with level: ' + loggerArray[1]}					 
				     config.logging.levels[loggerArray[0]] =  convertedLevel;
				     logger.FINE {' exititng mapToLevel with level: ' + convertedLevel}
				     logger.INFO{loggerArray[0] + ' is set to level ' + convertedLevel + ' and state ' + loggerArray[2]}
				 } else if (loggerArray[2].equalsIgnoreCase("disabled")) {					 
					 // set to OFF, since this will be our effective level on the UI, which translates to all=disabled in RAS trace syntax.
				 	 java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(java.util.logging.Level.OFF);
					 config.logging.levels[loggerArray[0]] =  "OFF";
					 logger.INFO{' ' + loggerArray[0] + ' is set to level OFF and state ' + loggerArray[2]}
				 } else {
 				     java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(parsedLevel);
					 config.logging.levels[loggerArray[0]] = convertedLevel;  				 
					 logger.INFO{' ' + loggerArray[0] + ' is set to level ' + convertedLevel + ' and unrecognized state which will default to enabled'}
				 }
			  break;
			  default:
				  logger.WARNING{ 'There was an error setting the trace specification and level, please check your trace string syntax, and try again.'}
			  break;
		  }
	  }
	  logger.FINE{'calling writeToDisk, config.logging.levels Map=' + config.logging.levels[].toString()};
      writeToDisk();
}

def onUpdate(){

    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
	
	def loggerName = request.params.traceSpecId[];
	
	def json = Json.decode(request.input[]) ;
	def loggerLevel = json["level"];

	def mappedLoggerName = (loggerName== '*')?'':loggerName;
	
	logger.FINE{ 'Converting loggerName: ' + loggerName + ", to JDK mappedLoggerName: " + mappedLoggerName}
	
	logger.INFO{ 'Updating : ' + mappedLoggerName + ", with new level: " + loggerLevel}
	
	// update JDK tracing (administrative console)
	java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(parseLevel(loggerName, loggerLevel));
	
	// update the sMash config zone (data cache tracing)
    config.logging.levels[loggerName] =  loggerLevel;
    writeToDisk();
}

def onDelete(){
	
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

	def loggerName = request.params.traceSpecId[];
	
	// maps root logger for sMash zone store to root logger for JDK
	def mappedLoggerName = (loggerName == '*') ? '' : loggerName;
	
	logger.FINE{ 'loggerName converted from: ' + loggerName + ', to mappedLoggerName: ' + mappedLoggerName}
	
	// delete logger from sMash config store (ObjectGrid logging)
	def deleted = zdelete('/config/logging/levels#'+ loggerName);
	
	// delete logger from JDK logging (from admin console logging)
	java.util.logging.Logger.getLogger(mappedLoggerName).setLevel(null);
	
	writeToDisk();
}

def parseLevel(spec, level){
	def parsedLevel;
	try{
		parsedLevel = java.util.logging.Level.parse(mapToLevel(spec, level));
		return parsedLevel;
	}catch(IllegalArgumentException e){
		return null;
	}
}

def mapToLevel(spec, level) {
	// converts RAS level (String) to JDK level (String)
	def levelToReturn = null;
	logger.FINE {' mapToLevel - level.toUpperCase(): ' + level.toUpperCase()};
	if (level.equalsIgnoreCase("event")) {
		if(spec.equalsIgnoreCase("*")) {
			levelToReturn = "INFO";
			logger.FINER {' mapToLevel - converting invalid trace level: event, to logger level INFO for default logger: ' + spec}
		} else {
			levelToReturn = "FINE";
			logger.FINER {' mapToLevel - converting EVENT to FINE'}
		}		
	} else if (level.equalsIgnoreCase("entryExit")) {
		if(spec.equalsIgnoreCase("*")) {
			levelToReturn = "INFO";
			logger.FINER {' mapToLevel - converting invalid trace level: entryExit, to logger level INFO for default logger: ' + spec}
		} else {
			levelToReturn = "FINER";
			logger.FINER {' mapToLevel - converting ENTRYEXIT to FINER'}
		}
	} else if (level.equalsIgnoreCase("debug")) {
		if(spec.equalsIgnoreCase("*")) {
			levelToReturn = "INFO";
			logger.FINER {' mapToLevel - converting invalid trace level: debug, to logger level INFO for default logger: ' + spec}
		} else {
			levelToReturn = "FINEST";
			logger.FINER {' mapToLevel - converting DEBUG to FINEST'}
		}
	} else {
		if(spec.equalsIgnoreCase("*")) {
			if(!(level.equalsIgnoreCase("SEVERE") || level.equalsIgnoreCase("WARNING") 
				|| level.equalsIgnoreCase("INFO") || level.equalsIgnoreCase("OFF"))) {
				levelToReturn = "INFO";
				logger.FINER {' mapToLevel - converting invalid trace level: ' + level + ', to logger level INFO for default logger: ' + spec}
			} else {
				levelToReturn = level;
				logger.FINER {' mapToLevel - keeping level at: ' + level + ', for default logger: ' + spec}
			}
		} else {
			levelToReturn = level;	
			logger.FINER {' mapToLevel - keeping level at: ' + level + ' for specification: ' + spec}
		}
	}
	return levelToReturn;
}

def writeToDisk(){
	// tmp.rainmaker.backupRunning[] == true : do not write to disk
	// but update the runtime to help in pd process
	if( tmp.rainmaker.backupRunning[]){
		logger.FINE { "Server is in backup mode and disks might be in process of backing up. Skip persisting changes to disk" };
		return;
	}
	// config.rainmaker.disk.root[] is /drouter/ramdisk2/mnt/raid-volume/raid0/xsa.app
	def outputFile = config.rainmaker.disk.root[] + "/config/logging-custom.config";
	
	String tsk=config.rainmaker.defaultPersistentZone[] + "/logging/levels";
	    zpost(Appliance.lock, tsk);
    try{
	def writer = new File(outputFile).newWriter();
	try{
		if(config.logging.levels[] instanceof Map){
			Map configLoggingLevels = new HashMap(config.logging.levels[]);
			Map configLoggingLevelsNew = new HashMap();
			if (configLoggingLevels != null && configLoggingLevels.size() > 0) {
			def loggingLevelsEntries = configLoggingLevels.entrySet();
				loggingLevelsEntries.each { loggingLevelEntry ->
					logger.FINE{'before zeroizing - key=' + loggingLevelEntry.key + ", value=" + loggingLevelEntry.value};
					def zeroizedKey = zeroizeTraceLevel(loggingLevelEntry.key);
					def zeroizedValue = zeroizeTraceLevel(loggingLevelEntry.value);
					configLoggingLevelsNew[zeroizedKey] = zeroizedValue;
					logger.FINE{'after zeroizing - key=' + zeroizedKey + ", value=" + zeroizedValue};
				}
			}
			writer.writeLine("/config/logging/levels=")
			//writer.writeLine(Json.encode(config.logging.levels[]))
			writer.writeLine(Json.encode(configLoggingLevelsNew))
			writer.close()
		}
	}catch (Exception e){
		logger.WARNING {"Exception occured while trying to persist log levels to file system. Exception: " + e.getMessage()}
		e.printStackTrace();
		throw new RuntimeException ("failed to persist log setting");
	}finally{
		try{
			// make setting take effect in memory
			zero.core.config.ConfigLoader.parseFile(new File(outputFile))
			// persist file
			zput(config.rainmaker.defaultPersistentZone[] + "/logging/levels", config.logging.levels[] )
			
		}catch (Throwable t){
			logger.WARNING {"Exception occured while trying to parse file containing GC key: " + t.getMessage()}
		    t.printStackTrace();
			// catch exception and clear out the file to prevent possible
			// failures loading the file after restart
			new File(outputFile).delete();
		}
	}
	}finally{
	  zpost(Appliance.unlock, tsk);
	}
	
}

def zeroizeTraceLevel(originalTraceString) {
	// escape characters that zero doesn't like, like $ (which becomes $$)
	def replaceStr=/\$\$/;	
	def newTraceString = originalTraceString.replaceAll("\\\$", replaceStr)
	return newTraceString;
}

def renderView(){
    request.json.output =  config.logging.levels[]
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView()
    request.view.resourceURL= '/resources/traceSpec';
    request.view.logLevelType= 'uiLogging';
    request.view.logLevelHeader= 'BR10041';
    request.view.loggingLevels = config.logging.levels[]

    render()

}
