import java.util.logging.Level;

import com.ibm.websphere.users.Permissions;
import zero.json.Json;
import xsa.wxs.context.GlobalContextURIs.Appliance;

def _normalize(name)
{
    return name.replaceAll("['\"]", "");
}

def onList()
{
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');
    
    logger.INFO{ 'entering onList'}
    renderView();
}

def onCreate(){
	 if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
	 return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');


	 def json = Json.decode(request.input[]);
	 def loggerNames = json["logger"];
	 
	 if(loggerNames == null || "".equals(loggerNames.trim())){
		 return;
	 }
	 
	 logger.FINE{ 'Trace string (pre-processed): ' + loggerNames}
	 
	 def loggerNamesList = loggerNames.tokenize(":");
	 
	 String tsk=config.rainmaker.defaultPersistentZone[] + "/logging/dataCacheLevels";
	 
	 logger.FINE{'config.rainmaker.defaultPersistentZone[]: ' + config.rainmaker.defaultPersistentZone[]}
	 logger.FINE{'tsk: ' + tsk}
	 	 
	 // Make the write to the GC thread-safe
	 zpost(Appliance.lock, tsk);
	 
	 //if nothing found in that part of the GC, default is to return an empty map, which we will fill out.
	 def currentTraceSpec = zget(tsk, [:]);
	 
	 try {
		 for (loggerName in loggerNamesList)
		 {
			 logger.FINE{ 'Trace string (pre-normalized): ' + loggerName}
			 loggerName = _normalize(loggerName);
			 logger.FINE{ 'Trace string (post-normalized): ' + loggerName}
			 
			 logger.INFO{ 'loggerName to process: ' + loggerName}
		 			 
			 // lets find the "=" sign now to get the logging_spec=logging_level pairs
			 // we also get rid of all spaces at begining and end, and before and after
			 // the equal sign
			 def loggerArray = loggerName.trim().replaceAll("\\s","").split('=');
			 
			 logger.FINE {'loggerArray.length: ' + loggerArray.length}
			 
			 switch ( loggerArray.length ) {
				 case 0:
				   //Client-side validation should let this condition occur
				   logger.WARNING{ 'There was an error setting the trace specification and level, please check your trace string, and try again.'}
				 break;
				 case 1:
				   // user forgot to type the level and state, or the equals and the level/state altogether, default the level to INFO
				   // ENABLED is default and semantics are to just enable this trace component as INFO
				   // ex: "Grid=" or "Grid"
				   logger.FINE{'Printing out currentTraceSpec map before supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
				  
				   // This adds to the map, believe it or not....in Groovy, if it can't find the key, it'll take it to mean you are adding this to the map
				   // If the key is there, it will replace the value for that key.
				   currentTraceSpec[loggerArray[0]] =  "INFO";
				   
				   //updating the map also also the UI to be updated as appropriate
				 
				   logger.FINE{'Printing out currentTraceSpec map after supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
				   logger.INFO{loggerArray[0] + ', defaults to state ENABLED and defaults to level: INFO '}
			     break;
				 case 2:
				 	//missing only trace state, which defaults to ENABLED, which sematically means we display this traceSpec in the UI and enabled it
				 	logger.FINE{'Printing out currentTraceSpec map before supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
					currentTraceSpec[loggerArray[0]] =  mapToLevel(loggerArray[0], loggerArray[1]);
					logger.FINE{'Printing out currentTraceSpec map after supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
					logger.INFO{loggerArray[0] + ' is set to level ' + mapToLevel(loggerArray[0], loggerArray[1]) +' and defaults to state ENABLED'}
				 break;			 
				 case 3:
				 	// the full case, setting the trace state to ENABLED semantically does nothing to the trace string, but setting it to DISABLED
				    // sets it to OFF in the UI, and would be converted to "all=disabled" when setting the RAS trace for ObjectGrid
				    // if the state is anything other than enabled or disabled is not valid
				    if (loggerArray[2].equalsIgnoreCase("enabled")) {
				 	  logger.FINE{'Printing out currentTraceSpec map before supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
					  logger.FINE {' entering mapToLevel with level: ' + loggerArray[1]}
					  currentTraceSpec[loggerArray[0]] =  mapToLevel(loggerArray[0], loggerArray[1]);
					  logger.FINE {' exititng mapToLevel with level: ' + mapToLevel(loggerArray[0], loggerArray[1])}
					  logger.FINE {'Printing out currentTraceSpec map after supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
					  logger.INFO{loggerArray[0] + ' is set to level ' + mapToLevel(loggerArray[0], loggerArray[1]) + ' and state ' + loggerArray[2]}
				    } else if (loggerArray[2].equalsIgnoreCase("disabled")) {
						logger.FINE{'Printing out currentTraceSpec map before supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
						// set to OFF, since this will be our effective level on the UI, which translates to all=disabled in RAS trace syntax.
						currentTraceSpec[loggerArray[0]] =  mapToLevel(loggerArray[0], "OFF");
						logger.FINE{'Printing out currentTraceSpec map after supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
						logger.INFO{loggerArray[0] + ' is set to level ' + mapToLevel(loggerArray[0], loggerArray[1]) + ' and state ' + loggerArray[2]}
					} else {
						logger.FINE{'Printing out currentTraceSpec map before supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
						currentTraceSpec[loggerArray[0]] =  mapToLevel(loggerArray[0], loggerArray[1]);
						logger.FINE{'Printing out currentTraceSpec map after supposed add of loggerName: ' + loggerArray[0] + ', is: ' + currentTraceSpec}
						logger.INFO{loggerArray[0] + ' is set to level ' + mapToLevel(loggerArray[0], loggerArray[1]) + ' and unrecognized state which will default to enabled'}
					}
				 break;			 				 
				 default:
				 	logger.WARNING{ 'There was an error setting the trace specification and level, please check your trace string syntax, and try again.'}
			     break;
			 }
		 }			 			 						 
		// sets the trace level into the Global Context	  
		zput(config.rainmaker.defaultPersistentZone[] + "/logging/dataCacheLevels", currentTraceSpec);
	 } finally {
	   // release the lock on the global context
       zpost(Appliance.unlock, tsk);
	 }
}

def onUpdate(){

    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

	def loggerName = request.params.traceSpecDataCacheId[];
	
	def json = Json.decode(request.input[]);
	def loggerLevel = json["level"];

	//def mappedLoggerName = (loggerName== '*')?'':loggerName;
	
	logger.INFO{ 'updating logger: ' + loggerName + ", to level: " + loggerLevel}
	
	String tsk=config.rainmaker.defaultPersistentZone[] + "/logging/dataCacheLevels";
	
    zpost(Appliance.lock, tsk);
    try{	
    def currentTraceSpec = zget(tsk, [:]);
	
	logger.FINE{'Printing out currentTraceSpec map before supposed add: ' + currentTraceSpec}
    //currentTraceSpec[loggerName] =  loggerLevel;
	// prevents updating default logger with improper level from drop-down menu
	// though this should not happen since default logging levels are limited from
	// the javascript
	currentTraceSpec[loggerName] =  mapToLevel(loggerName, loggerLevel);
	logger.FINE{'Printing out currentTraceSpec map after supposed add: ' + currentTraceSpec}
    
    zput(config.rainmaker.defaultPersistentZone[] + "/logging/dataCacheLevels", currentTraceSpec );
	}finally{
	  zpost(Appliance.unlock, tsk);
	}
}

def onDelete(){
	
    if (!user.session[].permissions.contains(Permissions.APPLIANCE_ADMIN))
        return com.ibm.websphere.rest.RenderUtils.renderError(403, 'RM08008');

	def loggerName = request.params.traceSpecDataCacheId[];

    String tsk=config.rainmaker.defaultPersistentZone[] + "/logging/dataCacheLevels";
    
    
    zpost(Appliance.lock, tsk);
    try{
      def currentTraceSpec = zget(tsk);

	  logger.FINE{'Printing out currentTraceSpec map before delete: ' + currentTraceSpec}
	  
      currentTraceSpec.remove(loggerName);
	  
	  logger.FINE{'Printing out currentTraceSpec map before delete: ' + currentTraceSpec}
	  
	  zput(config.rainmaker.defaultPersistentZone[] + "/logging/dataCacheLevels", currentTraceSpec );
	}finally{
	  zpost(Appliance.unlock, tsk);
	}
	
}

def mapToLevel(spec, level) {
	def foundLevel = false;
	def levelToReturn = null;
	def standardLoggerLevels = [
		'OFF',
		'SEVERE',
		'WARNING',
		'INFO',
		'FINE',
		'FINER',
		'FINEST',
		'ALL'];
	
	// the *=level=state logger
	def defaultLoggerLevels = [
		'OFF',
		'SEVERE',
		'WARNING',
		'INFO'];
	
	// if default logger, then only allow certain levels, all other levels will default to INFO
	if(spec.equalsIgnoreCase("*") && !defaultLoggerLevels.contains(level.toUpperCase())) {
		logger.FINER {'Default logger: ' + spec + ', contains invalid level: ' + level.toUpperCase() + ". Converting to level: INFO"};
		foundLevel = true;
		return "INFO";
	}
	
	for (levels in standardLoggerLevels) {
		if (levels.equalsIgnoreCase(level)) {
			foundLevel = true;
			levelToReturn = levels;
			break;
		} else {
			logger.FINE {'mapToLevel - level.toUpperCase(): ' + level.toUpperCase()};
			if (level.equalsIgnoreCase("event")) {
				foundLevel = true;
				levelToReturn = "FINE";
				logger.FINE {' mapToLevel - converting EVENT to FINE'};
				// break because we found what we were looking for
				break;
			} else if (level.equalsIgnoreCase("entryExit")) {
				foundLevel = true;
				levelToReturn = "FINER";
				logger.FINE {' mapToLevel - converting EVENT to FINER'};
				break;
			} else if (level.equalsIgnoreCase("debug")) {
				foundLevel = true;
				levelToReturn = "FINEST";
				logger.FINE {' mapToLevel - converting EVENT to FINEST'};
				break;
			}
		} // end else
	}  //end for loop
	
	logger.FINE {' mapToLevel - foundLevel flag: ' + foundLevel + ', with level: ' + levelToReturn};
		
	if (!foundLevel) {
		// return the default if the user specified an invalid one
		logger.INFO {' mapToLevel - could not find trace level ' + level + '. Defaulting to level INFO '};
		levelToReturn = "INFO";
	}
	
	return levelToReturn;
}

def renderView(){
    request.json.output =  zget(config.rainmaker.defaultPersistentZone[] + "/logging/dataCacheLevels")
    request.view = com.ibm.websphere.rest.RenderUtils.getRequestedView()
    request.view.resourceURL= '/resources/traceSpecDataCache';
    request.view.logLevelType= 'dataCacheLogging';
    request.view.logLevelHeader= 'BR10042';
    request.view.loggingLevels = request.json.output[];

    render()

}
