
import com.ibm.websphere.rainmaker.rest.doc.ScriptParser;

def serialize(obj)
{
    if (obj instanceof Collection)
        return obj.collect{ serialize(it) }.join('\n');
    
    else if (obj instanceof Map)
        return obj.collect{ "${it.key}: ${it.value}" }.join('\n');
    
    else if (obj instanceof Map.Entry)
        return "${obj.key}: ${obj.value}";
    
    return obj ?: '';
}

def generate(pattern, method)
{
    pattern = '/resources/' + pattern.replaceAll('/', '/{id}/');
    
    def text = [];
    text << '<row>';
    
    text << "<entry colname='name'>";
    text << serialize(method.name);
    text << '</entry>';
    
    text << "<entry colname='pattern'>";
    text << serialize(method.iscollection ? pattern : "${pattern}/{id}");
    text << '</entry>';
    
    text << "<entry colname='format'>";
    text << serialize(method.format);
    text << '</entry>';

    text << "<entry colname='success'><ul>";
    method.success.each {
        text << '<li>'
        text << serialize(it);
        text << '</li>'
    }
    text << '</ul></entry>';

    text << "<entry colname='error'><ul>";
    method.error.each {    
        text << '<li>'
        text << serialize(it);
        text << '</li>'
    }
    text << '</ul></entry>';
    
    text << "<entry colname='example'>";
    text << serialize(method.example);
    text << '</entry>';
    
    text << '</row>';
    return text;
}

def outputDir = new File(event.apphome[], 'restdoc');
outputDir.mkdir();

def indexText = [];
indexText << '<table>';
indexText << '<title>Index of WebSphere CloudBurst REST APIs</title>';

indexText << '<tgroup cols="1">';
indexText << '<colspec colname="api" colwidth="35%"/>';

indexText << '<thead>';
indexText << '<row valign="bottom">';
indexText << '<entry colname="api">api</entry>';
indexText << '</thead>';

indexText << '<tbody>';

def parser = new ScriptParser();

config.rainmaker.restdoc.resources[].each { name ->
    def groovyFile;

    def moduleDir = [
        event.apphome[],
        config.dependencies.'ibm:rainmaker.rest.pd'[]
    ].find {
        (groovyFile = new File(it, "app/resources/${name}.groovy")).exists()
    };

    if (! moduleDir)
        return;

    indexText << '<row>';
    indexText << '<entry colname=\'api\'>';
    indexText << capitalize(name);
    indexText << '</entry>';
    indexText << '</row>';
    
    def bndFile = new File(groovyFile.parentFile, "${name}.bnd");
    def uriPatterns = bndFile.exists() ? bndFile.text.split('\n') : [name];
    uriPatterns -= config.rainmaker.restdoc.ignorebindings[];

    def apiMethods = parser.parse(groovyFile);
    
    def resourceText = [];
    resourceText << '<table pgwide="1">';
    resourceText << "<title>REST API for ${capitalize(name)}</title>";

    resourceText << '<tgroup cols="6">';
    resourceText << '<colspec colname="name" colwidth="35*"/>';
    resourceText << '<colspec colname="pattern" colwidth="35*"/>';
    resourceText << '<colspec colname="format" colwidth="35*"/>';
    resourceText << '<colspec colname="success" colwidth="120*"/>';
    resourceText << '<colspec colname="error" colwidth="120*"/>'
    resourceText << '<colspec colname="example" colwidth="210*"/>';

    resourceText << '<thead>';
    resourceText << '<row valign="bottom">';
    resourceText << '<entry colname="name">name</entry>';
    resourceText << '<entry colname="pattern">pattern</entry>';
    resourceText << '<entry colname="format">format</entry>';
    resourceText << '<entry colname="success">success</entry>';
    resourceText << '<entry colname="error">error</entry>';
    resourceText << '<entry colname="example">example</entry>';
    resourceText << '</row>';
    resourceText << '</thead>';

    resourceText << '<tbody>';

    uriPatterns.each { pattern ->
        apiMethods.each { method -> 
            resourceText += generate(pattern, method);
        }
    }
    
    resourceText << '</tbody>';
    resourceText << '</tgroup>';
    resourceText << '</table>';
    
    def resourceFile = new File(outputDir, "${name}.dita");
    resourceFile.text = resourceText.join('\n') + '\n';
}

indexText << '</tbody>';
indexText << '</tgroup>';
indexText << '</table>';

def indexFile = new File(outputDir, 'index.dita');
indexFile.text = indexText.join('\n');
