/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.setup;

import java.sql.DriverManager; 
import zero.data.groovy.Manager;
import com.ibm.websphere.rainmaker.logging.Output;

def create(target, dependency, dbType, version) 
{ 
    def sql = new File(dependency, "sql/${dbType}/create-${version}.sql"); 
    executeSQL(target, sql); 
} 
 
def drop(target, dependency, dbType, version) 
{ 
    def sql = new File(dependency, 'sql/${dbType}/drop-${version}.sql');
    executeSQL(target, sql); 
}

def migrate(target, dependency, dbType, version)
{
    def sql = new File(dependency, 'sql/'+dbType+'/migrate-'+version+'.sql');
    executeSQL(target, sql);
}
 
def executeSQL(target, file) 
{
    if (!file.exists())
        return;
    
    def statements = parseSQL(file);
    def db = Manager.create('rainmaker-db'); 
    
    db.inTransaction { 
        statements.each {
            db.update(it.trim()); 
        } 
    }
     
    logger.FINE { "Finished executing file ${file.path}." };
}

def getDatabaseConfig()
{
    return zget('/config/db/rainmaker-db');
}

def getDatabaseType(jdbcClass)
{
    if (jdbcClass =~ /derby/)
        return 'derby';
    
    else if (jdbcClass =~ /db2/)
        return 'db2';
    
    throw new Exception("Unsupported database driver: ${jdbcClass}."); 
}

def getDatabaseVendor(dbType)
{
    if (dbType == 'derby')
        return 'Apache Derby';
    
    else if (dbType == 'db2')
        return 'IBM DB2';
    
    throw new Exception("Unsupported database type: ${dbType}."); 
}
 
def getResolvedProperties(dir) 
{ 
    def file = new File(dir, '.zero/private/resolved.properties'); 
    def properties = new Properties(); 
    properties.load(file.newInputStream());     
    return properties; 
}

def getVersions(dir, dbType)
{
    def versions = [] as Set;
    
    def sqlDir = new File(dir, "sql/${dbType}");
    
    if (!sqlDir.exists())
        return versions;
    
    def sqlFiles = sqlDir.eachFileMatch(~/create-\d+\.\d+\.\d+\.\d+\.sql/) {
        def start = it.name.indexOf('-') + 1;
        def end = it.name.lastIndexOf('.');
        versions << it.name[start..<end];
    }
    
    return versions;
}
 
def isLocal(name) 
{ 
    return name.startsWith(config.rainmaker.build.prefix[]); 
}

def onAppliance()
{
    new File('/firmware.xml').exists()
}

def parseSQL(file)
{
    if (!file.exists()) 
        return []; 
    
    logger.FINE { "Parsing file ${file.path}..." };
     
    def lines = file.readLines(); 
    def noComments = []; 
     
    lines.each { 
        def line = it.trim(); 
         
        if (!line.startsWith('--') && !line.startsWith('//')) 
            noComments << line; 
    } 
     
    return noComments.join().tokenize(';');
}

def sample(target, dependency, dbType, version) 
{ 
    def sql = new File(dependency, "sql/${dbType}/sample-${version}.sql"); 
    executeSQL(target, sql); 
}
 
def shutdown()
{
    try
    {
        def root = new File(config.root[]);
        DriverManager.getConnection("jdbc:derby:../${root.name}/rainmaker-db;shutdown=true");
    }
 
    catch (Throwable t)
    {
        //
        // do nothing - this always throws after shutdown
        // 
    }
    
    logger.FINE { "The database server has been stopped." }; 
}
 
