/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.ui;

import com.ibm.websphere.acl.PermissionsACL;
import groovy.rainmaker.constants.Status;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;

/**
 *
 * renderDashboard() creates an entire web console page, including the header, body, and footer.
 *
 * @param template
 *        The Groovy template to render as the page's body ('content').
 *
 * @param authenticated
 *        A flag that determines whether the screen is being shown post-authentication.
 *
 */
def renderDashboard(template, authenticated)
{
    logger.FINEST { "User-Agent is ${request.headers.in.'User-Agent'[]}." };
    
    def dashboard = [
        'core/header.gt', 
        'core/splash.gt', 
        'core/toolbar.gt', 
        template, 
        'core/footer.gt'
    ];
    
    if (!authenticated)
        dashboard -= ['core/splash.gt', 'core/toolbar.gt'];
    
    try
    {
        dashboard.each { section ->
            logger.FINEST { "Rendering template ${section} as part of page ${request.path[]}..." };
            request.view = section;
            render();
        }
    }
    
    catch (Throwable t)
    {
        logger.SEVERE { CleanStackTrace.cleanStackTrace(t); }
    }
}

def renderGroovyTemplate(template)
{
    logger.FINEST { "Rendering template located at ${template}." };
    
    def content = invokeTemplate(template);
    
    //
    // HACK: something about template file encoding is causing non-printable 
    //       characters to appear at the beginning of the content
    //
    def htmlStart = content.indexOf('<');
    return htmlStart >= 0 ? content.substring(htmlStart) : '';
}
