/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.ui;

import com.ibm.websphere.acl.PermissionsACL;
import groovy.rainmaker.constants.Status;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rainmaker.logging.MessageUtils;

class EnumerationCollection {

    static byType = [
        hypervisors: [
            type: [
                ESX: 'RM03000', 
                zVM: 'RM03002'
            ]
        ],
        subnets: [
            protocol: [
                static: 'RM03003'
                //dhcp: 'RM03004'
            ]
        ],
        storage: [
            shared: [
                T: 'RM10044', 
                F: 'RM10045'
            ]
        ]
    ];

}

def getEnumeration(type, name)
{
    return EnumerationCollection.byType[type][name];
}

def renderEnumeration(name, value, htmlId, mutable)
{
    def type = request.params.render.resourceType[];
    def id = request.params.render.resourceID[];
    def enumeration = getEnumeration(type, name);
    
    if (!mutable)
        return renderText(MessageUtils.message(user.lang[], enumeration[value], []), htmlId);
    
    def callback = "javascript:document.dashboard.details.updateResourceField('/resources/${type}', ${id}, '${htmlId}');";
    
    def html = new StringBuilder();
    html.append("<select class='dashboard-properties-drop-down-menu' id='${htmlId}' name='${name}' onChange=\"${callback}\">");
    
    enumeration.each {
        html.append("<option value='${it.key}' ${value == it.key ? 'selected' : ''}>${MessageUtils.message(user.lang[], it.value, [])}</option>");
    }
    
    html.append("</select>");
    return html;
}

def renderSubnetSelectBox(name, value, htmlId, mutable)
{
    def type = request.params.render.resourceType[];
    def hypervisorid = request.params.render.resourceID[];

    // Parse out the network ID in the list on the panel
    // 1. drop trailing text
    String netString = htmlId.substring(0, htmlId.indexOf("-subnetid"));
    // 2. drop leading text
    netString = netString.substring(netString.lastIndexOf("-") + 1);
    def netid = netString;

    def subnetList = getResources('subnets', [:], false);
    
    // Create an entry for None selected
    HashMap none = new HashMap();
    none.id = 0;
    none.name = MessageUtils.message(user.lang[], 'RM10359', []);

    // Build select box contents
    def enumeration = new ArrayList();
    enumeration.add(none);

    subnetList.each {
        enumeration.add(it);
    }

    def disabled = "";
    if (!mutable)
        disabled = "disabled";
    
    def callback = "javascript:document.dashboard.details.updateNetworkSubnetID('/resources/networks', ${netid}, '${htmlId}', ${hypervisorid});";
    
    def html = new StringBuilder();
    html.append("<label for='${htmlId}' style='display:none'>");
    html.append(MessageUtils.message(user.lang[], 'RM10684', []));
    html.append("</label>");
    html.append("<select class='dashboard-properties-drop-down-menu' id='${htmlId}' name='${name}' onChange=\"${callback}\" ${disabled}>");
    
    enumeration.each {
        html.append("<option value='${it.id}' ${value == it.id ? 'selected' : ''}>${it.name}</option>");
    }
    
    html.append("</select>");
    
    html.append("<img class='icon-image' style='margin-left: 15px; vertical-align: middle; display: none;' src='' alt='' title='' id='${htmlId}-success'/>");
    
    return html;
}
