/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

package groovy.rainmaker.ui;

import com.ibm.websphere.acl.PermissionsACL;
import groovy.rainmaker.constants.Status;
import com.ibm.websphere.users.Permissions;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import java.text.DecimalFormat;

def _convertToFahrenheit(value)
{
    return new DecimalFormat(".#").format(value * 1.8 + 32);
}

def _getAggregateAverage(averages)
{
    return (averages.sum() / averages.size()).toInteger();
}

def _getAggregateAverageColor(aggregateAvg)
{
    if (aggregateAvg < 0)
        return '#E1E1E1';
    
    def good = config.rainmaker.health.good[];
    def fair = config.rainmaker.health.fair[];
    def poor = config.rainmaker.health.poor[];
    return aggregateAvg >= poor ? '#E18080' : (aggregateAvg >= fair ? '#F4EA74' : '#77B359');
}

def _getAggregateAverageKey(aggregateAvg)
{
    def good = config.rainmaker.health.good[];
    def fair = config.rainmaker.health.fair[];
    def poor = config.rainmaker.health.poor[];
    return aggregateAvg >= poor ? 'RM10176' : (aggregateAvg >= fair ? 'RM10175' : 'RM10174');
}

def _getAggregateAverageText(aggregateAvg)
{
    def unknown = MessageUtils.message(user.lang[], 'RM10493', []);
    return aggregateAvg >= 0 ? MessageUtils.message(user.lang[], 'RM10494', [aggregateAvg.toString()]) : "<img src='/images/unknown-icon.gif' class='icon-image' style='vertical-align: middle;' alt=\"${unknown}\" title=\"${unknown}\"/>";
}

def _getTemperatureIcon(tempInfo)
{
    return (tempInfo.readingstatus) ? '/images/success-icon.gif' : '/images/warning-icon.gif';
}

def _getDiscreteTemperatureIcon(tempInfo)
{
	def icon = '/images/success-icon.gif';
	
	if( tempInfo.value == "High" ){
		icon = '/images/warning-icon.gif';
	} else if( tempInfo.value == "Over" ){
		icon = '/images/error-icon.gif';
	}
	
	return icon;
}

def _renderHealthIcon(nlsKey, aggregateAvg)
{
    def desc = MessageUtils.message(user.lang[], nlsKey, []);    
    return "<span class='default'><img src='${icon(nlsKey)}' alt=\"${desc}\" title=\"${desc}\" class='icon-image' style='vertical-align: bottom;'/>&nbsp;&nbsp;${desc}</span>";
}

def _renderHealthMeter(nlsKey, aggregateAvg)
{
    def desc = MessageUtils.message(user.lang[], nlsKey, []);
    
    def meterLength = 20;
    def colorLength = (meterLength * Math.max(aggregateAvg, 0) / 100).toInteger();
    
    colorLength = Math.min(colorLength, meterLength);
    
    def uncoloredLength = meterLength - colorLength;
    
    def color = _getAggregateAverageColor(aggregateAvg);
    def text = _getAggregateAverageText(aggregateAvg);
    
    return "<nobr><span class='default' style='vertical-align: middle; border: 1px solid #999999; background-color: #E6E6E6 '><span style='background-color: ${color};'>${'&nbsp;' * colorLength}</span>${'&nbsp;' * (uncoloredLength)}</span> &nbsp;<span class='default'>${text}</span></nobr>";
}

def renderOverallHealth(poweredOn, averages, includeDesc)
{
    def nlsKey = null;
    def aggregateAvg = -1;
    
    if (!poweredOn)
        nlsKey = 'RM10173';
    
    else if (averages.contains(-1))
        nlsKey = 'RM10173';
    
    else
    {
        aggregateAvg = _getAggregateAverage(averages);
        nlsKey = _getAggregateAverageKey(aggregateAvg);
    }
    
    def html = null;
    
    if (includeDesc)
        return _renderHealthIcon(nlsKey, aggregateAvg);
    
    return _renderHealthMeter(nlsKey, aggregateAvg);
}

def renderTemperature(tempInfo)
{
    def icon = _getTemperatureIcon(tempInfo);
    def scale = 'C';
    def value = null;

    try {
    	value = Float.parseFloat(tempInfo.value);
    } catch (NumberFormatException nfe) {
    	return renderDiscreteTemperature(tempInfo);
    }
    
    //
    // HACK: show fahrenheit degrees for USA users
    //
    if (user.lang[] == 'en' && request.locales[].find{ it.language == 'en' && it.country == 'us' }){
        value = _convertToFahrenheit(value);
        scale = 'F';
    }
    
    return "<span class='default'>${value}&#176;${scale}&nbsp;</span><img src='${icon}' class='icon-image' style='vertical-align: top;' alt='${value}'/>";
}

def renderDiscreteTemperature(tempInfo)
{
	def icon = _getDiscreteTemperatureIcon(tempInfo);
	def value = MessageUtils.message(user.lang[], "trouble.temperature.powersupply."+ tempInfo.value, []); 
	
	return "<span class='default'>${value}&nbsp;</span><img src='${icon}' class='icon-image' style='vertical-align: top;' alt='${value}'/>";

}
