/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
//
// ActionMenu represents the toolbar shown at the top of the details panel (right-hand side).
// It contains links to actions that are relevant to the resource being displayed below. If
// the menu is empty, it will not display anything on the screen.
//

dojo.provide('javascript.widgets.ActionMenu');
dojo.require('javascript.widgets.BaseWidget');

dojo.declare('javascript.widgets.ActionMenu', javascript.widgets.BaseWidget, {   

    _addActionWithIcon: function(uri, id, action, column, name) {
        var span = document.createElement('span');
        dojo.addClass(span, 'default-with-margin')
        column.appendChild(span);

        var link = this._createActionLink(uri, id, action, name);
        span.appendChild(link);

        if (action.icon)
        {
	        var img = document.createElement('img');
			dojo.attr(img, {
                 'class': 'icon-image' + (action.disabled ? ' action-menu-item-disabled' : ''), 
                 'src': action.icon, 
                 'alt': action.name, 
                 'title': action.name 
             });
	        dojo.style(img, 'verticalAlign', 'bottom');
            link.appendChild(img);
	        if ( action.hide ) {
	        	dojo.attr(img, {
	        		'id': action.id
	        	});
	        	Bedrock.hide(img);
	        }
        }

        else {
            link.innerHTML = action.name;
        }
    },

    _createActionLink: function(uri, id, action, name) {
        //
        // the action may be a function added to the DetailsView object or
        // a JSON representation to be passed to updateResource()
        //
        var functionCall = null;

        //
        // see if a function was added at runtime...
        //
        // TODO: what is going on here????
        if (dojo.isFunction(this.parent[action.state])) {
        	if(name){
        		//the name may have single quotes in it - need to escape those.
        		if(name.indexOf("\'")>-1)
        		{
        			functionCall = action.state + '(\'' + uri + '\', \'' + id +  '\', \'' + name.replace("\'","\\'") +  '\');';
        		}
        		else
        		{
        			functionCall = action.state + '(\'' + uri + '\', \'' + id +  '\', \'' + name +  '\');';
        		}
			}else{
				functionCall = action.state + '(\'' + uri + '\', \'' + id + '\');';
			}
            //functionCall = action.state + '(\'' + uri + '\', ' + id + ');';
		}
        //
        // otherwise, call updateResource() with the given JSON representation
        //
        else {
            functionCall = 'updateResource(\'' + uri + '\', ' + id + ', \'' + dojo.toJson(action.state) + '\');';
		}
	
		if ( action.disabled ) {
			return document.createElement('span');
		}
		else {
			var link = document.createElement('a');
			link.setAttribute('href', 'javascript:document.dashboard.details.' + functionCall);
			return link;
		}
    },

    initialize: function(parent, defaults) {
    	this.inherited(arguments);
    },
    
    constructor: function(parent, defaults) {
        this.initialize(parent, {
            id: 'action-menu',
            'class': 'action-menu'
        });

        var options = {
            toolbarDefault: '',
            keepDefault: false
        };
        Bedrock.addProperties(options, defaults || {});

        this.defaultMessage = options.toolbarDefault;
        this.keepDefault = options.keepDefault;
        
        this.parameters = {};
        this.showDefault();
    },

    selectResource: function(uri, id) {
    	console.log("action-menu, selectResource");
		dojo.xhrGet({
		    url: uri + '/' + id,
		    content: this.parameters,
		    handleAs: "json",
		    headers: {
				'Accept': 'application/rainmaker+action+menu'
		    },
		    load: dojo.hitch(this, this._updateActions, uri, id)
         });
    },

    showDefault: function(uri) {
        this.element.innerHTML = "<span class='heading action-menu-default-message'>" + this.defaultMessage + "</span>";
    },

    _updateActions: function(uri, id, response) {
    	console.log("action-menu _updateActions  response " ,  response);
    	console.log("ActionMenu._updateActions this", this);

		// parent node is the div passed in constructor
        var parentElement = this.element.parentNode;
        // set tab index to -1 for screen reader to acknowledge
        parentElement.setAttribute("tabIndex", "-1");
        
		// set aria live to polite to indicate to screen reader panel updates dynamically
		parentElement.setAttribute("aria-live", "polite");
        dijit.focus(parentElement);
    	

        var json = response;
        var name = json.label || json.name;
        var actions = json.actions;

        //
        // no actions for this resource - keep menu hidden
        //
        if (!actions || actions.length == 0)
            return;

        //
        // create a single table row/column, where each action icon is
        // separated by a margin (whitespace)
        //
        var table = document.createElement('table');
        
        // accessibility since table is purely for layout 
        table.setAttribute ("role", "presentation" );
        
        
        dojo.addClass(table, 'action-menu-table');
        //IE requires the 'tbody' to be created and appended...
        var tableBody = document.createElement('tbody');
	
        table.appendChild(tableBody);
        var row = document.createElement('tr');

        var nameColumn = document.createElement('td');
        dojo.addClass(nameColumn, 'action-menu-name-column');
        var nameElement = document.createElement('span');
        nameElement['id'] = 'action-menu-heading';
        dojo.addClass(nameElement, 'action-menu-heading');
        nameElement.setAttribute('aria-level', '2' );
        nameElement.setAttribute('role', 'heading');
        
        nameElement.innerHTML = this.keepDefault ? this.defaultMessage : name;
        nameColumn.appendChild(nameElement);

        //
        // for hypervisors, add a hyperlink to the admin console
        // next to the resource name
        //
        row.appendChild(nameColumn);

        var column = document.createElement('td');
        dojo.addClass(column, 'action-menu-icon-column');
        tableBody.appendChild(row);
        row.appendChild(column);

        for (var i = 0; i < actions.length; ++i)
            this._addActionWithIcon(uri, id, actions[i], column, name);

        //
        // delete action menu used by previous resource, if any
        //
        if (this.element.hasChildNodes())
            this.element.removeChild(this.element.firstChild);

        this.element.appendChild(table);

        this.show();
        
    }

});
