/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/
//
// FilterBox directs its output to the list sidebar on each page. It provides real-time search results, 
// which each result in the list having a status icon and one or more actions associated with it. FilterBox
// controls all of the AJAX requests made by the list sidebar, including what icons are
// rendered above the search results.
//

dojo.provide('javascript.widgets.FilterBox');
dojo.require('javascript.widgets.BaseWidget');

dojo.declare('javascript.widgets.FilterBox', javascript.widgets.BaseWidget, {
    _createProgressIndicator: function() {
        var progressID = this.element.id + '-progress';
        var progress = document.createElement('img');
        dojo.attr( progress, {
			'id' : progressID,
			'class' : 'dashboard-filter-progress',
			'style': 'display: none;',
			'src' : '/images/progress-indicator-small.gif',
			'alt' : messages['RM11036'] 
			}
		);
        this.element.appendChild(progress);
        return progress;
    },

    _createSortMenu: function(sortLink) {
        var sortMenu = document.createElement('div');
        dojo.attr( sortMenu, {
			'id' : 'list-sort-menu',
			'class' : 'dashboard-list-sort-menu',
			'style' : 'display: none;'
			}
		);
		
		Bedrock.hide(sortMenu);
		
        sortLink.parentNode.appendChild(sortMenu);

        var table = document.createElement('table');
        
		// accessibility since table is purely for layout 
        table.setAttribute ("role", "presentation" );
        
        
        //IE requires the 'tbody' to be created and appended...
        var tableBody = document.createElement('tbody');
        table.appendChild(tableBody);
        sortMenu.appendChild(table);

        for (var i = 0; i < this.menuOptions.length; ++i)
        {
	        var row = document.createElement('tr');
            //var row = new Element('tr');
            tableBody.appendChild(row);

	        var column = document.createElement('td');
	        dojo.addClass(column, 'sort-menu-row');
            row.appendChild(column);

	        var span = document.createElement('span');
	        dojo.addClass(span, 'default');
            column.appendChild(span);

	        var link = document.createElement('a');
			link.setAttribute('href', 'javascript:void(0);');
            link.onclick = dojo.hitch(this, this.sortResources, this.menuOptions[i].field);

            link.innerHTML = this.menuOptions[i].nls;
            span.appendChild(link);
        }
    },

    _createSortWidget: function(options) {
        var sortID = this.element.id + '-sort';
        var sortIcon = document.createElement('img');
        dojo.attr(sortIcon, {
			'id' : sortID,
			'class' : 'dashboard-filter-sort icon-image',
			'src' : '/images/sort-icon.gif'
			}
		);

        if (!options.sort || this.menuOptions == null || (this.menuOptions.length == 0 ) )
        {
            //sortIcon.addClassName('disabled-icon-image');
            dojo.addClass(sortIcon, 'disabled-icon-image');
            sortIcon.alt = sortIcon.title = messages['RM11037'];
            this.element.appendChild(sortIcon);
        }

        else
        {
            sortIcon.alt = sortIcon.title = messages['RM11036'];

	        var sortLink = document.createElement('a');
			sortLink.setAttribute('href', 'javascript:void(0);');
            sortLink.onclick = dojo.hitch( this, this.toggleSortMenu);

            sortLink.appendChild(sortIcon);
            this.element.appendChild(sortLink);

            this._createSortMenu(sortLink);
        }
    },

    _createTextBox: function() {
        var textID = this.element.id + '-text';
        var text = document.createElement('input');
        dojo.attr( text, {
			'id' : textID,
			'type' : 'text',
            'class': 'dashboard-filter-text form-prompt'
			}
		);
        text.defaultText = text.value = this.defaultText;
        this.element.appendChild(text);
        

        //
        // connect typing behavior to background AJAX requests
        // that update search results...
        //

        text.showAllChoices = dojo.hitch(this, this.showAllChoices);
        
        var getQueryString = dojo.hitch(this, this._getQueryStringAutocomplete, text);
        var showChoices = dojo.hitch(this, this.showChoices);
        text.onkeyup = function() {
            //
            // if search text is removed, show all
            //
            if (this.value.length == 0) {
                this.showAllChoices();
            }
            else {
            	console.log("update choices");
		        var fullQuery = getQueryString();
           		console.log("full query is " + fullQuery );
		    	showChoices(fullQuery);
            }
        }

        text.onfocus = function() {
            //
            // remove default instruction when user starts typing
            //
            if (this.value == this.defaultText)
            {
                this.value = '';
                //this.removeClassName('form-prompt');
                Bedrock.removeClassName(this.id, 'form-prompt');
            }
        }
        
        text.onblur = function() {
            //
            // re-insert default instruction when user leaves a blank field
            //
            if (this.value.length == 0)
            {
                this.value = this.defaultText;
                //text.addClassName('form-prompt');
                Bedrock.addClassName(text.id, 'form-prompt');
            }
        };

        return text;
    },

   /* _createToggleQueryString: function(resourceTypes) {
        var toggleKey = 'toggle=';

        if (resourceTypes.length < 2)
            return toggleKey;

        var typeStrings = [];

        //
        // encode resource type definitions as HTTP query params
        //
        for (var i = 0; i < resourceTypes.length; ++i)
            typeStrings.push(resourceTypes[i].name + ':' + encodeURIComponent(resourceTypes[i].uri));

        return toggleKey + typeStrings.join(';');
    },*/

    _getQueryString: function() {
        //var query = this.toggleQueryString;
        var query = "";

        // TODO: DOJO where does this selected id come from....
//        if (this.parent.currentSelectedID)
//            query += '&selected=' + this.parent.currentSelectedID;

        var ajaxParameters = this.ajaxParameters;
		dojo.forEach(Bedrock.keys(ajaxParameters),
		    function(k) {
				if(query != "") query += '&';
	           query += k + '=' + ajaxParameters[k];
            }
        );
        
        return query + '&order=' + this.orderBy + '&relevance=' + this.orderBy +
                       // item 2539: '&count=' + this.count + '&page=' + this.page +
                       '&uri=' + encodeURIComponent(this.uri) + '&baseId=' + this.display.id;
    },

    _getQueryStringAutocomplete: function(text) {
        var fullQuery = this._getQueryString();
        
        if ( text.value != text.defaultText ) {
	        fullQuery += '&' + this.propertyName + "=" + encodeURIComponent(text.value);
        }


        return fullQuery;
    },

    initialize: function(parent, details) {
    	this.inherited(arguments );
    },
    

    constructor: function(parent, display, resourceTypes, options) {
	
		this.propertyName = 'name';
		if(options.propertyName) this.propertyName = options.propertyName;
		
        // JMB, can only extend one level with the code like this...
        console.log("filter box ctr");
        this.initialize(parent, { 
            id: 'filter', 
            'class': 'dashboard-filter-container' 
        });
        
        this.display = display;

        this.uri = options.uri;
        //this.toggleQueryString = this._createToggleQueryString(resourceTypes);

        this.ajaxParameters = {};
        
        var keys = Bedrock.keys(options);
		dojo.forEach(keys,
		  dojo.hitch(this, 
		    function(k) {
             if ( Bedrock.indexOf([ 'draggableResources', 'selectableResources', 'typeIcons' ], k) >= 0)
                this.ajaxParameters[k] = options[k];
            }
          )
        );
        
        this.menuOptions = options.menuOptions;

        this.page = 1;
        this.count = 20;
        if(options.defaultSort){
        	this.orderBy = options.defaultSort;
        }else{
        	this.orderBy = this.propertyName;
		}
        this.defaultText = messages['RM11000'];

        this.text = this._createTextBox();
        this.progress = this._createProgressIndicator();

        this._createSortWidget(options);

		this.showAllChoices();
    },

    setAjaxParameter: function(key, value) {
    	console.log("setAjaxParameter(" + key + "," + value);
        var updated = false;

        if (value == null) {
            if (this.ajaxParameters[key] != undefined) {
                delete this.ajaxParameters[key];
                updated = true;
            }
        }
        else if (this.ajaxParameters[key] != value) {
            this.ajaxParameters[key] = value;
            updated = true;
        }

        if (updated) {
            this._getUpdatedChoices();
        }
    },

    //
    // showAllChoices() renders a full page of results by making an AJAX request
    // with no filters/search terms.
    //
    showAllChoices: function() {
    	this.showChoices(this._getQueryString());
    },

	_getUpdatedChoices: function() {
		console.log("_getUpdatedChoices");
		var query = this._getQueryStringAutocomplete(this.text);
    	this.showChoices(query);
	},


    showChoices: function(query) {
		console.log("showChoices, query is " + query);
    	Bedrock.show(this.progress);
        var updateChoices = dojo.hitch(this, this._updateChoices);
        var uri = this.uri + (this.uri.indexOf('?') < 0 ? '?' : '&') + query;
        
		dojo.xhrGet( 
			{ 
				url: uri, 
				content: this.ajaxParameters,
				sync: true,
				headers: {
	                 'Accept': 'application/rainmaker+list'
	            },
				load: updateChoices
			}
		);
        
		dojo.style(this.display, 'display', ''); 
        Bedrock.show(this.display);
    },

    //
    // showPage() renders a subset of the resource collection in the list sidebar.
    // Each page of results has N resources in it, where N = this.count.
    //
    showPage: function(page) {
        var prevPage = this.page;
        this.page = page;

        this._getUpdatedChoices();
		
        this.page = prevPage;
    },

    //
    // sortResources() re-renders the current list according to the field provided.
    //
    sortResources: function(field) {
        this.toggleSortMenu();

        if (this.orderBy == field)
            return;

        this.orderBy = field;
        
        this._getUpdatedChoices();
    },

    //
    // toggle() alters the REST API being used to collect data and render the list sidebar.
    //
    toggle: function(uri) {
        this.uri = uri;
        this.page = 1;
        this.showAllChoices();
    },

    toggleSortMenu: function(event) {
        var evt = event || window.event;  // IE uses window.event
        Bedrock.toggle('list-sort-menu');
        if (evt) {
//            Event.stop(evt);
			dojo.stopEvent(evt);
		}
    },

    _updateChoices: function(choices) {
    	//console.log("_updateChoices " + choices);
    	try {
			this.display.innerHTML = choices;
    	   	window.setTimeout(function() {Bedrock.evalScripts(choices)}, 1000);
        	Bedrock.cleanWhitespace(this.display);
	        Bedrock.cleanWhitespace(Bedrock.down(this.display));

    	    var hasEntries = this.display.firstChild && Bedrock.down(this.display).childNodes;
	        this.entryCount = hasEntries ? Bedrock.down(this.display).childNodes.length : 0;

    	    for (var i = 0; i < this.entryCount; ++i) {
        	    Bedrock.getEntry(this.display, i).autocompleteIndex = i;
			}

//	        this.stopIndicator();
			console.log("stop indication " + this.progress);
			Bedrock.fadeCommon(this.progress);
			Bedrock.fadeSplash();

//	        this.render();
    	}
    	catch ( e ) {
    		console.debug("error, " + e);
    	}
    }

});
