/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/


package register;

import zero.json.Json;
import groovy.rainmaker.users.NoMailException;
import com.ibm.websphere.rainmaker.logging.MessageUtils;
import com.ibm.websphere.rainmaker.logging.CleanStackTrace;

def _renderError(status, msg)
{
    request.status = status;
    request.writer[].write("<span>${msg ==~ /RM\d+/ ? MessageUtils.message(user.lang[], msg, []) : msg}</span>");
    logger.SEVERE { MessageUtils.message(user.lang[], msg, []) }
}

def onPOST()
{
    def json = null;
    try
    {
        json = Json.decode(request.input[]);
        if (!json.username)
            return _renderError(500, 'RM08024');
        else if (!json.name)
            return _renderError(500, 'RM08025');
        else if (getUserByName(json.username))
            return _renderError(500, 'RM08026');
        else if( config.connection.defaults.smtp.hostname[] && !json.email && !storage.rainmaker.security.enabled[])
        	return _renderError(500, 'RM08086');
    }
    catch (Throwable t){
        return _renderError(400, 'RM08002');
    }
    if (storage.rainmaker.security.enabled[])
        json.password = false;
    else
    {
        if (config.connection.defaults.smtp.hostname[]){
        	//if email server defined (meaning password can be emailed/generated/isn't required)
        	if (json.password.length() == 0){
        		// AND user didn't provide a password
        		// generate password
        		json.password = generatePassword(8);
        	}
        	else {
        		//AND the user still provided a password anyways
        		//check to see if password & verification match
            	json.password = json.password.trim();
            	json.verification = json.verification.trim();
            	if (!json.password.equals(json.verification))
            		return _renderError(400, 'RM08006');
        	}
        }
        else {
        	//email server isn't defined (meaning password can't be emailed or generated/is required)
        	json.password = json.password.trim();
        	json.verification = json.verification.trim();
        	if (!json.password.equals(json.verification))
        		return _renderError(400, 'RM08006');
        	if (json.password.length() == 0)
        		return _renderError(400, 'RM08017');
        }
    }
    def user = [
        username: json.username, 
        name: json.name, 
        email: json.email ? json.email : ''
    ];
    try
    {
        createUser(user, false, json.password);
    }
    catch (Throwable t)
    {
        if (!(t.cause instanceof NoMailException))
        {
            logger.SEVERE { CleanStackTrace.cleanStackTrace(t) };
            def text = t.message;
            if (text) {
                def colon = text.indexOf(':');
                if (colon != -1)
                    text = text.substring(colon + 1);
            }
            return _renderError(500, text);
        }
    }
}