dojo.provide("xsa.datamgmt.widgets.QueryDataGridDetail");

dojo.require("dojo.string");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.Tree");
dojo.require("dijit.Menu");
dojo.require("dijit.PopupMenuItem");
dojo.require("dijit.MenuSeparator");
dojo.require("dijit.Dialog");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.ValidationTextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.i18n");

dojo.require("dojox.widget.Standby");
dojo.require("dojox.grid.enhanced.plugins.Pagination");
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");

dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");
dojo.require("xsa.datamgmt.widgets.QueryDataGridDetailLayout");

dojo.declare("xsa.datamgmt.widgets.QueryDataGridDetail", [dijit._Widget, dijit._Templated],{
	// summary:
	//		This widget is a composite widget for containing a tree of 
	//		chartable statistics and a tab container for containing the generated
	//		charts.
	
	// templateString: String
	//		The string to use for the template of this widget.
	templateString: dojo.cache("xsa", "datamgmt/widgets/templates/QueryDataGridDetailTemplate.html"),

	// widgetsInTemplate: Boolean
	//		If true, the template as widgets in it.
	widgetsInTemplate: true,

	// _borderContainer: [private] Object
	//		The borderContainer widget used to house all the tree/chart stuff.
	_borderContainer: null,

	// _searchBoxNode: [private] DOMNode
	//		The DOM Node to house the search field, if any.
	_searchBoxNode: null,

    // _detailsNode: [private] DOMNode
    //		The DOM Node ot house the details widget.

	// _treeNode: [private] DOMNode
	//		The DOM Node to house the tree widget.
	_treeNode: null,

	// _treePane: [private] Object
	//		The reference to the content pane holding the tree and other data.
	_treePane: null,

	// _tree: [private] Object
	//		The tree widget to use to represent the statistics view.
	//		Created in startup.
	_tree: null,

	// _treeStore: [private] Object
	//		The backing store for the tree.
	//		Created in startup.
	_treeStore: null,

	// _treeStore: [private] Object
	//		The backing model for the tree.
	//		Created in startup.
	_treeModel: null,

	// statisticsTreeUrl: [public] String
	//		The URL to the service end point that provides the data that 
	//		inflates the tree.
	statisticsTreeUrl: "",
	
	// titleText: [public] String
	//		NLS message to display as the title of the panel.
	titleText: "",

	// _selectedItem: [private] object
	//		The datastore item selected by tree node click to add to the charts.
	_selectedItem: null,

	postMixInProperties: function(){
		// sunnary:
		//		Function to mixin translations, etc.
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
		dojo.mixin(this, this._messages);
	},

	startup: function(){
		// summary:
		//		Over-ridden startup function for the widget.
																					
		if(!this._started){
			this.inherited(arguments);
			// Build and init the tree.
			if(this.statisticsTreeUrl){
				this._createTree();
			}

			this.populateTitle();
		}
		this.inherited(arguments);
	},
	
    populateTitle: function() {
        this.selectedPanel.innerHTML = this.params.titleText;
    },

	resize: function(dim){
		// summary:
		//		Resize over-ride to allow calling of child widget resize so that
		//		the widget scales properly in a container.
		// dim:
		//		The dimensions object to process.
		if(dim){
			dojo.contentBox(this.domNode, dim);
		}

		var myDims = dojo.contentBox(this.domNode);
		
		// Internal function usage, may not port (but unlikely to change);
		// Remove any internal padding/margins and use that to set the size of the
		// contained border container.
		var pads = dojo._getPadBorderExtents(this.domNode);
		var margins = dojo._getMarginExtents(this.domNode);
		myDims.h -= (pads.h + margins.h);
		myDims.w -= (pads.w + margins.w);
		this._borderContainer.resize(myDims);

		this.inherited(arguments);
	},

	_createTree: function(){
		// summary:
		//		Function to create the tree and its clickable menu to 
		//		display when clicking on tree elements
		// tags:
		//		private.
		this._treeStore = new dojo.data.ItemFileReadStore({url: this.statisticsTreeUrl});
		this._treeModel = new dijit.tree.ForestStoreModel({
			store: this._treeStore,
			query: {},
			rootId: "grids",
			rootLabel: this.treeRootLabel,
            childrenAttrs: ["children"]
		});
		this._tree = new dijit.Tree({model: this._treeModel, 
			showRoot:false, 
			openOnClick: false,
			getLabelStyle: function(item, opened){
				if(item.seriesID && item.seriesID[0].indexOf("!") > -1){
					// Style the Maps to look clickable, a better way would be to add the class 'xsaStatisticsTree' to the
					// labels domNode.  not sure how to do that here.
					return {color: "#0088D8"};
				}
				return {};
			}
		});
		this._treeNode.appendChild(this._tree.domNode);

		// Turn off the icons.
		this._tree.getIconStyle = function(){
			return {display: "none"};
		};
		this.connect(this._tree, "onClick", "_onTreeNodeClick");
		this._tree.startup();
		
		var treeTop = dojo.query('div[role="tree"]')[0];
		if(treeTop) {
			dojo.attr(treeTop, "aria-label", this._messages.dataGridTreeNavigation);
		}
	},

	_onTreeNodeClick: function(item, treeNode, event){
		// summary:
		//		Function called when a tree node is clicked.
		//		This allows access to the item and to help know 
		//		when to open a menu at the click point.
		// item:
		//		The tree item clicked on.
		// tags:
		//		protected.

		var node = treeNode?treeNode.labelNode:"";
		var seriesID = this._treeStore.getValue(item, "seriesID");
		
		if (seriesID) {
			var p = {
			    seriesID: seriesID,
				type:'grids', 
				subtype:'maps', 
				init : 'numEntries'
			};
			
		    if (this.detailView) {
				this.detailView.destroy();
		    //this._detailsNode.removeChild(this.detailView.domNode);
		    }
		    
			if (seriesID.indexOf("!") > 0) {
				this.detailView = new xsa.datamgmt.widgets.QueryDataGridDetailLayout(p);
				dojo.place(this.detailView.domNode, this._detailsNode, "first");
				this.detailView.startup();
			}
		}	
		
	},

	_onAddToNewChart: function(){
		// summary:
		//		Hook point to add a new chart to the tab 
		//		container.
		// tags:
		//		private
		this._chartNameBox.attr("value", "");
		this._chartNameBox.validate();
		this._createChartButton.attr("disabled", true);
	},

	destroy: function(){
		// summary:
		//		Over-ridden destroy to handle internal cleanup.
		if(this._tree){
			this._tree.destroy();
			delete this._tree;
		}
		if(this._chartSubMenu){
			delete this._chartSubMenu;
		}
		delete this._selectedItem;
		this.inherited(arguments);
	}
});