/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.datamgmt.widgets.QueryDataGridDetailLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("xsa.datamgmt.widgets.QueryGrid");

dojo.requireLocalization("xsa.monitor.widgets.layout", "xsa_messages");

dojo.declare("xsa.datamgmt.widgets.QueryDataGridDetailLayout", [dijit._Widget, dijit._Templated], {   
    
    templatePath: dojo.moduleUrl("xsa","datamgmt/widgets/templates/QueryDataGridDetailLayoutTemplate.html"),
	widgetsInTemplate: true,
    xsa_messages : null,
	    
    constructor: function(kwArgs) {
		if(!kwArgs) { 
			kwArgs = {};
		}
		this.seriesID = kwArgs.seriesID ? kwArgs.seriesID : "grids";
        
        this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.layout", "xsa_messages");
        this.xsa_messages_overview = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
        this.xsa_messages_query = dojo.i18n.getLocalization("xsa.datamgmt.widgets", "xsa_messages");
    },
    
    postCreate: function() {
        if ( this.splashScreen ) {
	        Bedrock.hide(this.splashScreen);
        }
        if ( this.dashboardView ) {
	        Bedrock.hide(this.dashboardView);
        }
        var options = {
            contentDefault: messages['RM11006']
        };
		this.pageDescription.innerHTML = this.xsa_messages_query.queryGridContentsPageDescription;
    },

	startup: function() {
	   this.selectResource("", this.seriesID, true);
	},
	    
    destroy: function() {
    	if(this.grid) this.grid.destroy();
		this.inherited(arguments);
    },
    
    //
    // selectResource() modifies the data shown in the detail panels to be that 
    // of the resource with the given URI/ID. The action toolbar will also be 
    // updated to reflect the change.
    //
    selectResource: function(uri, id, dontShowAfter) {
        Bedrock.hide(this.defaultMessage);

        if (dojo.style(this.splashScreen, 'display') == 'none' && !dontShowAfter)
        {
            if (this.dashboardView) {
                Bedrock.hide(this.dashboardView);
            }    
            Bedrock.show(this.splashScreen);
        }
        
        var title = "";
        var ids = id.split("!");
        for (var i=0; i<ids.length; i++) {
            if (title == "") {
                title = ids[i];
            } else {
                title = title+": "+ids[i];
            }
        }
        
        //SMP: Bug 19879 & 19880
        //from ActionMenu.js: to make sure focus moves to detail panel after item is selected
        // parent node is the div passed in constructor
        var parentElement = this.dashboardView.parentNode;        
        // set tab index to -1 for screen reader to acknowledge
        parentElement.setAttribute("tabIndex", "-1");
		// set aria live to polite to indicate to screen reader panel updates dynamically
		parentElement.setAttribute("aria-live", "polite");
        dijit.focus(parentElement);
        
		this.selectedPanel.innerHTML = "&nbsp; "+this.xsa_messages_overview.XSA30115A+": "+ids[1];
		dojo.style( this.selectedPanel, 'display: block;');
		dojo.removeClass(this.selectedPanel, 'heading');
		dojo.removeClass(this.selectedPanel, 'action-menu-default-message');		
		dojo.addClass(this.selectedPanel, 'action-menu-heading');
		
		this.server = id;

        Bedrock.show(this.dashboardView);    	
		
		if ( this.currentSelectedId != id ) {
			var gridName = id.substring(0, id.indexOf("!"));
			var mapName =  id.substring(id.indexOf("!")+1);
			
			if(this.grid) this.grid.destroy();
			//Create the grid
			this.grid = new xsa.datamgmt.widgets.QueryGrid({gridName: gridName, mapName: mapName}, this.gridDiv);
		}

   		this.currentSelectedId = id;
    }
    
});
