/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.datamgmt.widgets.QueryGrid");

dojo.require("dojox.grid.EnhancedGrid");
dojo.require("dojox.data.QueryReadStore");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dijit.form.ValidationTextBox");
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");
dojo.require("dojox.grid.enhanced.plugins.Pagination");
dojo.require("dojox.grid.enhanced.plugins.Filter");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.form.FilteringSelect");
dojo.require("javascript.widgets.ModalDialog");
dojo.require("dijit.form.CheckBox");
dojo.require("javascript.widgets.MessageDialog");
dojo.require("javascript.widgets.XSFloatingPane");

dojo.requireLocalization("xsa.datamgmt.widgets", "xsa_messages");

dojo.declare("xsa.datamgmt.widgets.QueryGrid", [dijit._Widget, dijit._Templated],{

	/* holds a reference to localized message bundle */
    xsa_messages : null,

	_maxResultSize: 500,
	
	templatePath: dojo.moduleUrl("xsa.datamgmt.widgets","templates/QueryGridTemplate.html"),
	widgetsInTemplate: true,

	_enhancedGrid: null,
	_queryTextBox: null,
	_searchButton: null,
	_dialogQueryTextBox: null,
	_dialogApplyButton: null,
	_dialogCancelButton: null,
	_dialogOKButton: null,
	_filterDialog: null,
	_filterButton: null,
	_partitionFromSelect: null,
	_partitionToSelect: null,
	_to: null,
	_from: null,
	_dialog: null,
	_messageDialog: null,
	_invalidateDropDown: null,
	_clearButton: null,
	_showValues: null,
	_selectAllToggle: null,
	_error: null,
	
	_totalResults: null, //total number of keys that matched the query.  This may or may not be the same as the number of items in the table.
	_footerIconTD: null,
	
	_connections: [],
	_selectAllFocusConnection: null, //Handler for select all checkbox focus event
	
	_lastQuery: null,
	_queryId: null,
	
	//The URLs can be overridden for testing
	gridURL: "/resources/griddata", 
	popFiltersURL: "/resources/griddata", 
	invalidateURL: "/resources/griddata", 
	validateSearchURL: "/resources/griddata", 

	structure: null,
		
	plugins : {
					pagination: {
						pageSizes: [],// Array, custom the items per page button
						description: true,	// boolean, custom whether or not the description will be displayed
						sizeSwitch: true,	// boolean, custom whether or not the page size switch will be displayed
						pageStepper: false,	// boolean, custom whether or not the page step will be displayed
						gotoButton: false,	// boolean, custom whether or not the goto page button will be displayed
						maxPageStep: 0,	// Integer, custom how many page step will be displayed
						position: "bottom",	// String, custom the position of the pagination bar. There're two options: top, bottom
						defaultPageSize: 500 // Integer, what page size will be used by default
					},
					indirectSelection: true
	},
	
	constructor: function(kwArgs) {
		if(!kwArgs) kwArgs = {};
		this.gridName = kwArgs.gridName;
		this.mapName = kwArgs.mapName;
		this._lastQuery = "";
		this._totalResults = null;
		
		if(kwArgs.gridURL) this.gridURL = kwArgs.gridURL;
		if(kwArgs.popFiltersURL) this.popFiltersURL = kwArgs.popFiltersURL;
		if(kwArgs.invalidateURL) this.invalidateURL = kwArgs.invalidateURL;
		if(kwArgs.validateSearchURL) this.validateSearchURL = kwArgs.validateSearchURL;
		
		this.xsa_messages = dojo.i18n.getLocalization("xsa.datamgmt.widgets", "xsa_messages");
		this.xsa_mc_messages = dojo.i18n.getLocalization("xsa.messagecenter", "xsa_messages");
		
		this.structure = [{
			cells: [
				{ field: "id", name:"id", datatype:"string", width: "0px"},
				{ field: "key", name:this.xsa_messages.key, datatype:"string", width: "175px"},
				{ field: "value", name:this.xsa_messages.value, datatype:"string", width: "175px"},
				{ field: "templates", name:"TemplateIDs", datatype:"string", width: "250px"},
				{ field: "dependencies", name:"DependencyIDs", datatype:"string", width: "250px"},
				{ field: "partition", name:this.xsa_messages.partition, datatype:"string", width: "auto"}
			]
		}];
		
		//Setup page labels that are in the template
		this.fromLabel = this.xsa_messages.fromLabel;
		this.toLabel = this.xsa_messages.toLabel;
		this.showValuesLabel = this.xsa_messages.showValues;
		this.searchLabel = this.xsa_messages.searchRegEx;
		this.partitionRangeLabel = this.xsa_messages.partitionRange;
	},
	
	postCreate: function() {	
		this.inherited(arguments);
		this.createQueryTextBox();
		this.createToolbar();
		this.createEnhancedGrid();
		this.createFilterDialog();
    },
	
	createQueryTextBox: function(){
		this._queryTextBox = new dijit.form.ValidationTextBox({	
			name:"queryTextBox",
			title: this.xsa_messages.queryTextBoxTitle,
			value: this.xsa_messages.search, 
			validate: function(isFocused){
				//This method is called on every keypress which is way too much for serverside validation
				//override existing validation so that it doesn't interfere with setValidationError
			},
			setValidationError: function(isError, message){
				this._set("state", isError ? "Error" : "");
				this.set("message", isError ? message : "");
				dijit.setWaiState(this.focusNode, "invalid", isError);
			},
			onBlur: dojo.hitch(this, function(){
				this.toggleGraySearch();
				this._invalidateDropDown.set('disabled', this._lastQuery!=this._queryTextBox.textbox.value);
			}),
			onFocus: dojo.hitch(this, function(){
				this.toggleGraySearch();
			})
		}, this._queryTextBoxNode);
		dojo.style(this._queryTextBox.focusNode, "color", "#999999"); //Gray out the search value
		
		this._connections.push(dojo.connect(this._queryTextBox, "onKeyPress", this, function(e) {
			if(e.keyCode === dojo.keys.ENTER) {
				this._queryTextBox.value = this._queryTextBox.textbox.value;
				this.toggleGraySearch();
				this._executeQuery();
			}
			
			this._invalidateDropDown.set('disabled', true);
		}));
		
		this._searchButton = new dijit.form.Button({
			label: "Search",
			iconClass: "searchIcon",
			showLabel: false,
			type: "button",
			onClick: dojo.hitch(this, function(){
				this.toggleGraySearch();
				this._executeQuery();
			})
		}, this._searchButtonNode);
		
		var buttonPos = dojo.coords(this._searchButton.focusNode);
	//	dojo.style(this._queryTextBox.domNode, "height", buttonPos.h+"px");
		
		this._filterButton = new dijit.form.Button({
			label:  this.xsa_messages.filter,
			iconClass: "filterIcon",
			showLabel: false,
			type: "button",
			onClick: dojo.hitch(this, function(){
				this._dialogQueryTextBox.set("value", this._queryTextBox.value==this.xsa_messages.search ? "" : this._queryTextBox.value);
				this._filterDialog.show();
			})
		}, this._filterButtonNode);
		
		//hookup regular expression help dialog
		this._regexHelp.innerHTML = this.xsa_messages.regExHelp;
		this._connections.push(dojo.connect(this._regexHelp, "onclick", this, this.showRegularExpressionHelpModal));	
		this._regexHelp2.innerHTML = this.xsa_messages.regExHelp;
		this._connections.push(dojo.connect(this._regexHelp2, "onclick", this, this.showRegularExpressionHelpModal));			
	},
	
	createFilterDialog: function(){
		this._filterDialog = new javascript.widgets.GenericDialog({}, this._filterDialogNode);
		
		this._dialogQueryTextBox = new dijit.form.ValidationTextBox({name:"dialogQueryTextBox",
																title: this.xsa_messages.queryTextBoxTitle,
																validate: function(isFocused){
																	//This method is called on every keypress which is way too much for serverside validation
																	//override existing validation so that it doesn't interfere with setValidationError
																},
																setValidationError: function(isError, message){
																	this._set("state", isError ? "Error" : "");
																	this.set("message", isError ? message : "");
																	dijit.setWaiState(this.focusNode, "invalid", isError);
																},
																onBlur: dojo.hitch(this, function(){
																	if (this._queryTextBox.value.length == 0){
																		this._queryTextBox.setValidationError(false);
																	}
																})																
		}, this._dialogQueryTextBoxNode);
		
		//Create partition to and from range
		var validateFromPartitionSelection = function(){
			if(this._partitionFromSelect && this._partitionToSelect) {
				var fromValue = this._partitionFromSelect.value;
				var toValue = this._partitionToSelect.value;
				if(fromValue==="" || (toValue!=="" && fromValue>toValue)) {
					return false;
				}
			}
			return true;
		};
		var validateToPartitionSelection = function(){
			if(this._partitionFromSelect && this._partitionToSelect) {
				var fromValue = this._partitionFromSelect.value;
				var toValue = this._partitionToSelect.value;
				if(toValue==="" || (fromValue!=="" && fromValue>toValue)) {
					return false;
				}
			}
			return true;
		}
		
		this._partitionFromSelect = new dijit.form.FilteringSelect({
			id: "partitionFromSelect",
		    autoComplete: false,
		    searchAttr: "option",
		    value: "0",
		    style: "width:100px;",
			isValid: dojo.hitch(this, validateFromPartitionSelection),
			invalidMessage: this.xsa_messages.fromPartitionError
		}, "partitionFromSelect");
		
		this._partitionToSelect = new dijit.form.FilteringSelect({
		    id: "partitionToSelect",
		    autoComplete: false,
		    searchAttr: "option",
		    style: "width:100px;",
			isValid: dojo.hitch(this, validateToPartitionSelection),
			invalidMessage: this.xsa_messages.toPartitionError
		}, "partitionToSelect");
		
		var populateFilters = function(result){
			if(result.error) {
				this.createDisplayMessage(this._error, "error", null);
			} else {
				this._partitionFromSelect.store = new dojo.data.ItemFileReadStore({data:result.to});
				this._partitionToSelect.store = new dojo.data.ItemFileReadStore({data:result.from});
				this._partitionFromSelect.set('value', 0); 
				this._partitionToSelect.set('value', result.from.items.length-1); //select the last number in the list
				this._from=this._partitionFromSelect.value;
				this._to=this._partitionToSelect.value;
			}
		};
		
		var xhrArgs = {
			url: this.popFiltersURL,
			content: {action: "populatePartitionFilters", gridName:this.gridName, mapName:this.mapName},
			sync: true,
			handleAs: "json",
			headers: {"Content-Type":"text/json"},
			load: dojo.hitch(this, populateFilters),
			error: function(error){
				//TODO: handle error
			}
		};
		dojo.xhrGet(xhrArgs);
		
		//Create the dialog buttons
		this._dialogApplyButton = new dijit.form.Button({
			label:  this.xsa_messages.apply,
			type: "button",
			onClick: dojo.hitch(this, function(){
				if(this._partitionFromSelect.isValid() && this._partitionFromSelect.validate() && this._partitionToSelect.validate()) {
					this._from=this._partitionFromSelect.value;
					this._to=this._partitionToSelect.value;
					this._executeQuery(null, this._dialogQueryTextBox);
				}
			})
		}, this._dialogApplyButtonNode);
		
		this._dialogCancelButton = new dijit.form.Button({
			label:  this.xsa_messages.cancel,
			type: "button",
			onClick: dojo.hitch(this, function(){
				// remove all errors
				this._dialogQueryTextBox.setValidationError(false);
				this._partitionFromSelect.set("value",this._from);
				this._partitionToSelect.set("value",this._to);
				
				this._filterDialog.hide();
			})
		}, this._dialogCancelButtonNode);
	},
	
	createToolbar: function(){
		//Select all button
		this._selectAllToggle = new dijit.form.ToggleButton({
			iconClass: "dijitCheckBoxIcon",
			showLabel: false,
			checked: false,
			onChange: dojo.hitch(this, function(val){
				this._enhancedGrid.rowSelectCell.toggleAllSelection(val);
				this._selectAllToggle.set("label", this.xsa_messages.deselectAllTitle);
			}),
			label: this.xsa_messages.selectAllTitle
		}, this._selectAllToggleButton);
		
		//Clear button
		this._clearButton = new dijit.form.Button({
			label: this.xsa_messages.clear, 
			name: "clearButton",
			onClick: dojo.hitch(this, this.clear)
		},this._clearButton);
		
		//Invalidate menu button
		var menu = new dijit.Menu({ });
		menu.domNode.style.display="none";
		var menuItem1 = new dijit.MenuItem({
		    label: this.xsa_messages.selectedRowsOnlyMenuItem,
		    onClick: dojo.hitch(this, this.invalidateSelected)
		});
		menu.addChild(menuItem1);

		var menuItem2 = new dijit.MenuItem({
		    label: this.xsa_messages.entireQueryMenuItem,
		    onClick: dojo.hitch(this, this.invalidateQuery)
		});
		menu.addChild(menuItem2);
		
		this._invalidateDropDown = new dijit.form.DropDownButton({
			label: this.xsa_messages.invalidateButton, 
			name: "invalidateDropDown",
			dropDown: menu
		},this._invalidateDropDownButton);
			
		this._invalidateDropDown.set('disabled', true);
	
		//Show values checkbox
		var checked = dojo.cookie("XS_QueryGrid_showValues") ? (dojo.cookie("XS_QueryGrid_showValues")=="true" ? true : false) : false;
		this._showValues = new dijit.form.CheckBox({
			checked: checked,
			name: "showValues"
		}, this._showValuesCheckbox);
	},
	
	clear: function(){
		var performClear = function(){
			document.body.style.cursor = "wait";
			var xhrArgs = {
				url: "/resources/griddata",
				content: {action: "clear", gridName:this.gridName, mapName:this.mapName},
				handleAs: "json",
				headers: {"Content-Type":"text/json"},
				load: dojo.hitch(this, function(response){
					if(response.error){
						this.createDisplayMessage(response.error, "error", null, true);
					} else {
						if(this._queryTextBox.get("value") != this.xsa_messages.search){
							this._executeQuery();
						}
					}
					document.body.style.cursor = "";
				}),
				error: function(error){
					this.createDisplayMessage(this.xsa_messages.genericInvalidationError + " " + this.xsa_messages.checkServerLogsMsg, "error", null, true);
					document.body.style.cursor = "";
				}
			};
			dojo.xhrGet(xhrArgs);
		};
		var message = this.xsa_messages.clearDialogMessage;
			
		this._dialog = new javascript.widgets.ModalDialog('modal-dialog', message, dojo.hitch(this, performClear));
		this._dialog.show();
	},
	
	createEnhancedGrid: function(){
		var gridStore = new dojox.data.QueryReadStore({url: this.gridURL});
		
		gridStore._filterResponse = dojo.hitch(this, function(data){ //determine if the table is not displaying all of the matches
								this._totalResults=data.totalMatches;
								
								this._error="";
								if(data.error){
									this._error=data.error;
								} else if(data.warning) {
									this._warning=data.warning;
								}
								if(data.queryId) {
									this._queryId = data.queryId;
								}
								
								this._enhancedGrid.layout.setColumnVisibility(1, false); //always hide the id
								if(data.items && data.items[0]) { 
									//show/hide columns
									if(data.items[0].templates==undefined) {
										this._enhancedGrid.layout.setColumnVisibility(4, false);
									} else {
										this._enhancedGrid.layout.setColumnVisibility(4, true);
									}
									if(data.items[0].dependencies==undefined) {
										this._enhancedGrid.layout.setColumnVisibility(5, false);
									} else {
										this._enhancedGrid.layout.setColumnVisibility(5, true);
									}
								}
								if(this._showValues.checked) {
									this._enhancedGrid.layout.setColumnVisibility(3, true);
								} else {
									this._enhancedGrid.layout.setColumnVisibility(3, false);
								}
								return data;
		});
		
		this._enhancedGrid = new dojox.grid.EnhancedGrid({
			plugins:this.plugins, 
			structure: this.structure, 
			store: gridStore, 
			rowsPerPage: 500, //So if the user clicks to show 500 per page all rows are loaded. Select all doesnt work when not all the rows are loaded. 
			query: {action: "executeQuery", query: this._queryTextBox.value, fromPart:this._from, toPart:this._to, gridName:this.gridName, mapName:this.mapName, showValues:this._showValues.checked}, 
			selectionMode: "multiple", 
			selectable:true 
		}, this._enhancedGridNode);
		this._enhancedGrid.startup();
		this._enhancedGrid.query.action="gridAction";
		this._enhancedGrid.errorMessage=this._enhancedGrid.errorMessage+". "+this.xsa_messages.checkServerLogsMsg;
		
		//Create footer icon place holder
		//TODO: consider putting sytling in a class
		this._footerIconTD = dojo.create("td", {style:"width:1%;padding-left:5px;"});
		dojo.place(this._footerIconTD, this._enhancedGrid.pagination.plugin.paginators[0].descriptionTd, "before");
		
		var tableToEdit = dojo.query("table.dojoxGridPaginator")[0];
		if(tableToEdit) dojo.attr(tableToEdit, "role", "presentation");
		
		this._connections.push(dojo.connect(this._enhancedGrid, "_onFetchComplete", this, this.onGridReloadComplete));

		//Change the key/value to links???
		this._connections.push(dojo.connect(this._enhancedGrid.views.views[0], "onAfterRow", dojo.hitch(this, function(inRowIndex, cells, inRowNode){	
			var tableRow = dojo.query("tr", inRowNode)[0];
			if (tableRow.firstChild.nodeName === "TD") {
				dojo.forEach(tableRow.cells, dojo.hitch(this, function(tableRow, headerCells, cell, index){ 
					if (headerCells[index].field === "key" || headerCells[index].field === "value") {
						var cellText = cell.innerHTML;
                            dojo.attr(cell, "innerHTML", ""); //remove the current cell content
                            var link = dojo.create("a", {
                                innerHTML: cellText,
								href: "#",
								style: "color:black"
                            }, cell);
                            this._connections.push(dojo.connect(link, "onclick", dojo.hitch(this, "openRowDialog", tableRow, headerCells)));
					}
				}, tableRow, cells[0]), this);
			}
		})));
		
		//Needed for accessibiliy.  If a cell gets focus, the link in the cell should get focus.
        this.connect(this._enhancedGrid, "onCellFocus", dojo.hitch(this, function(cell, rowIndex){
            var node = cell.getNode(rowIndex);
            if (node.firstChild && node.firstChild.nodeName === "A") {
                //Give focus to the link for accessibility
                dojox.grid.util.fire(node.firstChild, "focus");
            }
        }));
		
	},
	
	openRowDialog: function(tableRow, headerCells, e){
		var rowId = "";
		dojo.forEach(headerCells, function(cell, index){
			if(cell.field==="id") {
				rowId = tableRow.cells[index].innerHTML;
			} 
		}, this);
		dojo.stopEvent(e);
		
		//get the row information from the server
		var xhrArgs = {
				url: this.invalidateURL,
				content: {action: "getRow", rowId:rowId, query: this._queryTextBox.value, fromPart:this._from, toPart:this._to, gridName:this.gridName, mapName:this.mapName, queryId:this._queryId},
				handleAs: "json",
				headers: {"Content-Type":"text/json"},
				load: dojo.hitch(this, function(response){
					if(response.error){
						if(response.error=="query_lost"){
							this.showQueryLostModal();
						} else {
							this.createDisplayMessage(response.error, "error", null);
						}
					} else {
						var node = document.createElement('div');
							var displayValue = "<div style='padding:10px;'><span style='font-weight:bold'>Key:</span><br><span style='word-wrap:break-word;'>"+response.key+"</span><br><br>";
							if(this._showValues.checked) displayValue = displayValue + "<span style='font-weight:bold'>Value:</span><br><span style='word-wrap:break-word;'>"+response.value+"</span>";
							displayValue = displayValue + "</div>";
							
							node.innerHTML = displayValue;
					
							dojo.place(node, this._resultsTable, "before");
							//dojo.body().appendChild(node);
							var tmp = new javascript.widgets.XSFloatingPane({
								title: this.xsa_messages.entryDetails,
								dockable: false,
								closable: true,
								resizable: true
							},node);
							tmp.startup();
							tmp.resize({ w:600, h:125 });
							tmp.bringToTop();
					}
				}),
				error: function(error){
					this.createDisplayMessage(xsa_messages.genericRetrieveError, "error", null);
				}
		};
		dojo.xhrGet(xhrArgs);
	},
	
	onGridReloadComplete: function(){
		//enable/disable invalidate drop down button
		this._invalidateDropDown.set('disabled', this._enhancedGrid.store._numRows==0);

		//remove all warning signs
		dojo.forEach(this._footerIconTD.children, dojo.destroy);

		//Check if result size is greater then the number of total items in the table
		var matches = this._enhancedGrid.store._numRows;
		var desc = this._enhancedGrid.pagination.plugin.paginators[0].descriptionDiv;
		if(this._totalResults > matches){
			//show warning message
			var warningMsg = this.xsa_messages.maxResultLimitReachedWarningMsg.replace("${_0_}",this._totalResults).replace("${_1_}",this._maxResultSize);
			
			if(this._warning) warningMsg = warningMsg +"  " + this._warning;
			this.createDisplayMessage(warningMsg, "warning", "maxResultLimitReachedCookie");
			
			//Update item description
			desc.innerHTML = this.xsa_messages.totalMatchesWarningDesc.replace("${_0_}",matches).replace("${_1_}",this._totalResults);	
		} else {
			//Update item description
			desc.innerHTML = this.xsa_messages.matches.replace("${_0_}",matches);	
			
			if(this._warning) {
				this.createDisplayMessage(this._warning, "warning", null);
			}
		}
						
		//Show error messages
		if(this._totalResults==undefined)this.createDisplayMessage(this._error, "error", null);
			
		//After a fetch, reset the action
		this._enhancedGrid.query.action="gridAction";
		this._enhancedGrid.query.queryId=this._queryId;
	},
	
	createDisplayMessage: function(message, type, cookieName, showMessage){
		if(this._enhancedGrid.query.action!="gridAction" || showMessage){ //do not redisplay message if user is just viewing results
			if(this._messageDialog==null){
				this._messageDialog = new javascript.widgets.MessageDialog({}, this._messageDialogNode);						
			}		
			this._messageDialog.setType(type);
			this._messageDialog.checkboxCookieName = cookieName;
			this._messageDialog.setShowCheckbox(cookieName ? true : false);
			this._messageDialog.setMessage(message);
			this._messageDialog.show();
		}
		
		//Add message to the table
		var imageLocation = "";
		var alt = "";
		if(type==="warning"){
			imageLocation = dojo.moduleUrl("dijit","themes/soria/images/warning.png");
			alt = this.xsa_mc_messages.warning;
		} else if(type==="error"){
			imageLocation = "/images/error-icon.gif";
			alt = this.xsa_mc_messages.severe;
		}
		if(dojo.byId("message_icon")) dojo.destroy(dojo.byId("message_icon"));
		dojo.place(dojo.create("img", {id:"message_icon", src:imageLocation, title:message, alt:alt}), this._footerIconTD, "first");
	},
	
	toggleGraySearch: function(){
		if (this._queryTextBox.value.length == 0){
			this._queryTextBox.set("value", this.xsa_messages.search);
			if(!dojo.isIE < 8) { 
				dojo.style(this._queryTextBox.focusNode, "color", "#999999"); //Gray out the search value
			}
			this._queryTextBox.setValidationError(false);
		} else if (this._queryTextBox.value == this.xsa_messages.search){
			this._queryTextBox.set("value", "");
			if(!dojo.isIE < 8) {
				dojo.style(this._queryTextBox.focusNode, "color", "inherit");
			}
		} else {
			if(!dojo.isIE < 8) {
				dojo.style(this._queryTextBox.focusNode, "color", "inherit");
			}
		}
	},
	
	invalidateSelected: function(){
		var selectedRows = this._enhancedGrid.selection.getSelected();
		
		if(selectedRows.length==0) {
			//TODO: determine how to tell the user that they need to select a row
		} else {
	
			//get the selected row ids
			var selection = new Array();
			
			dojo.forEach(selectedRows, dojo.hitch(this, function(row){if(row){selection.push(row.i.id)}}));
		
			var performInvaliateSelected = function(){
				document.body.style.cursor = "wait";
				var xhrArgs = {
					url: this.invalidateURL,
					content: {action: "invalidateSelected", selected: selection, query: this._queryTextBox.value, fromPart:this._from, toPart:this._to, count:25, gridName:this.gridName, mapName:this.mapName, queryId:this._queryId},
					handleAs: "json",
					headers: {"Content-Type":"text/json"},
					load: dojo.hitch(this, function(response){
						if(response.error){
							if(response.error=="query_lost"){
								this.showQueryLostModal();
							} else {
								this.createDisplayMessage(response.error, "error", null, true);
							}
						} else {
							//Remove invalidated rows from the grid
							this._executeQuery();
						}
						document.body.style.cursor = "";
					}),
					error: function(error){
						this.createDisplayMessage(this.xsa_messages.genericInvalidationError + " " + this.xsa_messages.checkServerLogsMsg, "error", null, true);
						document.body.style.cursor = "";
					}
				};
				dojo.xhrGet(xhrArgs);
			};
			var message = this.xsa_messages.invalidateSelectionDialog;
			if(selectedRows.length>1) {
				message = this.xsa_messages.invalidateSelectionsDialog.replace("${_NUMKEYS_}",selectedRows.length);
			}
			
			this._dialog = new javascript.widgets.ModalDialog('modal-dialog', message, dojo.hitch(this, performInvaliateSelected));
			this._dialog.show();	
		}
	},
	
	invalidateQuery: function(){
		var performInvaliateQuery = function(){
			document.body.style.cursor = "wait";
			var xhrArgs = {
				url: this.invalidateURL,
				content: {action: "invalidateQuery", query: this._queryTextBox.value, fromPart:this._from, toPart:this._to, gridName:this.gridName, mapName:this.mapName, queryId:this._queryId},
				handleAs: "json",
				headers: {"Content-Type":"text/json"},
				load: dojo.hitch(this, function(response){
					if(response.error){
						this.createDisplayMessage(response.error, "error", null, true);
					} else {
						this._executeQuery();
					}
					document.body.style.cursor = "";
				}),
				error: function(error){
					this.createDisplayMessage(this.xsa_messages.genericInvalidationError + " " + this.xsa_messages.checkServerLogsMsg, "error", null, true);
					document.body.style.cursor = "";
				}
			};
			dojo.xhrGet(xhrArgs);
		};
		
		var message = "";
		
		if(this._totalResults > 10000){
			message = this.xsa_messages.invalidateLargeWarning+"<br><br>";
		}
		
		var filter = this.showFilterInfo();
		if(filter){
			message = message + this.xsa_messages.invalidateQueryDialogWithFilterMsg.replace("${_0_}", this._queryTextBox.value).replace("${_1_}", filter);
		} else {
			message = message + this.xsa_messages.invalidateQueryDialogMsg.replace("${_0_}", this._queryTextBox.value);
		}
		
		this._dialog = new javascript.widgets.ModalDialog('modal-dialog', message, dojo.hitch(this, performInvaliateQuery));
		this._dialog.show();		
	},
	
	showRegularExpressionHelpModal: function(){
		if(this._messageDialog) dojo.destroy(this._messageDialog);

		this._messageDialog = new javascript.widgets.MessageDialog({
			type: "info",
			showCheckbox: false,
			message: "<div class='titleFont' style='text-align:center;background-color:#FFFFFF'>"+this.xsa_messages.regExHelpCap+"</div><br>"+
			         "<div style='padding:5px'>"+this.xsa_messages.regExHelpDesc+"</div>"+
					 "<table style='padding:5px' role='presentation'>"+
					 "<tr><td style='padding:7px'>.*</td><td>- "+this.xsa_messages.regExHelpFindAll+"</td></tr>"+
					 "<tr><td style='padding:7px'>A.*</td><td>- "+this.xsa_messages.regExHelpFindAllStarting.replace("${_0_}", "A")+"</td></tr>"+
					 "<tr><td style='padding:7px'>.*A</td><td>- "+this.xsa_messages.regExHelpFindAllEnding.replace("${_0_}", "A")+"</td></tr>"+
					 "<tr><td style='padding:7px'>.</td><td>- "+this.xsa_messages.regExHelpFindAny+"</td></tr>"+
					 "<tr><td style='padding:7px'>A?</td><td>- "+this.xsa_messages.regExHelpFindZeroOrOne.replace("${_0_}", "A")+"</td></tr>"+
					 "<tr><td style='padding:7px'>A*</td><td>- "+this.xsa_messages.regExHelpFindZeroOrMany.replace("${_0_}", "A")+"</td></tr>"+
					 "<tr><td style='padding:7px'>A+</td><td>- "+this.xsa_messages.regExHelpFindOnceOrMany.replace("${_0_}", "A")+"</td></tr>"+
					 "<tr><td style='padding:7px'>.A?B*C+</td><td>- "+this.xsa_messages.regExHelpFindExample.replace("${_0_}", "A").replace("${_1_}", "B").replace("${_2_}", "C")+"</td></tr>"+
					 "</table><br>"+
					 "<div style='padding:5px;'>"+this.xsa_messages.regExJavaDocMsg+"</div>"
					 
		}, this._messageDialogNode);						
							
		this._messageDialog.show();
	},
	
	showQueryLostModal: function(){
		var submitHandler = dojo.hitch(this, this._executeQuery);
		this._dialog = new javascript.widgets.ModalDialog('modal-dialog', this.xsa_messages.queryLostDialog, submitHandler);
		this._dialog.show();
	},
	
	showFilterInfo: function(){
		try{
			var store = this._partitionFromSelect.store;
			var firstPartitionIndex = store._arrayOfAllItems[0].option[0];
			var lastPartitionIndex = store._arrayOfAllItems[store._arrayOfAllItems.length-1].option[0];
			if(firstPartitionIndex!=this._partitionFromSelect.value || lastPartitionIndex!=this._partitionToSelect.value){
				this._filterInfo.innerHTML=this.xsa_messages.filterMsg + " " +this.xsa_messages.partitionFilterMsg.replace("${_0_}", this._partitionFromSelect.value).replace("${_1_}", this._partitionToSelect.value);
			} else {
				this._filterInfo.innerHTML="";
			}
		} catch(e) {
			this._filterInfo.innerHTML="";
		}
		return this._filterInfo.innerHTML;
	},
	
	_executeQuery: function(searchString, textBox){		
		this.showFilterInfo();
			
		this._enhancedGrid.rowSelectCell.toggleAllSelection(false); //Remove all row selections
		
		if(textBox==null) textBox=this._queryTextBox;
		if(searchString==null) searchString=textBox.value;
		
		//Validate the searchString is a valid regex
		var xhrArgs = {
			url: this.validateSearchURL,
			content: {action: "validateQuery", query: searchString},
			handleAs: "json",
			headers: {"Content-Type":"text/json"},
			load: dojo.hitch(this, function(response){
				if(response.error){
					textBox.setValidationError(true, this.xsa_messages.invalidRegex);
				} else {
					this._filterDialog.hide();
					textBox.setValidationError(false);
					this._queryTextBox.set("value", searchString); //in case the searchString came from a different textbox
					this.toggleGraySearch();
					
					this._enhancedGrid.setQuery({action: "executeQuery", query: searchString, fromPart:this._from, toPart:this._to, gridName:this.gridName, mapName:this.mapName, showValues:this._showValues.checked});

					this._selectAllToggle.set('checked', false);
					this._lastQuery=searchString;
					
					//set cookie to remember show values checkbox state
					dojo.cookie("XS_QueryGrid_showValues", this._showValues.checked, {expires: 30});
				}
			}),
			error: function(error){
				this.createDisplayMessage(this.xsa_messages.genericQueryError + " " + this.xsa_messages.checkServerLogsMsg, "error", null);
			}	
		};
		dojo.xhrGet(xhrArgs);
	},
	
	destroy: function(){
		//Cleanup connections
		dojo.forEach(this._connections, dojo.disconnect);
		if(this._selectAllFocusConnection) dojo.disconnect(this._selectAllFocusConnection);
		
		//Cleanup widgets
		if(this._enhancedGrid) this._enhancedGrid.destroyRecursive(true);
		if(this._queryTextBox) this._queryTextBox.destroyRecursive(true);
		if(this._searchButton) this._searchButton.destroyRecursive(true);
		if(this._filterDialog) this._filterDialog.destroyRecursive(true);
		if(this._dialogQueryTextBox) this._dialogQueryTextBox.destroyRecursive(true);
		if(this._dialogApplyButton) this._dialogApplyButton.destroyRecursive(true);
		if(this._dialogCancelButton) this._dialogCancelButton.destroyRecursive(true);
		if(this._dialogOKButton) this._dialogOKButton.destroyRecursive(true);
		if(this._filterButton) this._filterButton.destroyRecursive(true);
		if(this._partitionFromSelect) this._partitionFromSelect.destroyRecursive(true);
		if(this._partitionToSelect) this._partitionToSelect.destroyRecursive(true);
		if(this._invalidateDropDown) this._invalidateDropDown.destroyRecursive(true);
		if(this._selectAllToggle) this._selectAllToggle.destroyRecursive(true);
		if(this._messageDialog) this._messageDialog.destroyRecursive(true);
		if(this._clearButton) this._clearButton.destroyRecursive(true);
		if(this._showValues) this._showValues.destroyRecursive(true);
		
		delete this._enhancedGrid;
		delete this._queryTextBox;
		delete this._searchButton;
		delete this._filterDialog;
		delete this._dialogQueryTextBox;
		delete this._filterButton;
		delete this._dialogApplyButton;
		delete this._dialogCancelButton;
		delete this._partitionFromSelect;
		delete this._partitionToSelect;
		delete this._invalidateDropDown;
		delete this._selectAllToggle;
		delete this._dialog;
		delete this._messageDialog;
		delete this._clearButton;
		delete this._showValues;
				
		this.inherited(arguments);
	}
	 	
});