/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.layout.DataCacheDetailLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.charting.DataChart");
dojo.require("dojox.charting.widget.Legend");
dojo.require("dojox.grid.DataGrid");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Button");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("dojox.charting.action2d.Magnify");
dojo.require("dojox.charting.action2d.Tooltip");
dojo.require("dojox.charting.action2d.Highlight");

dojo.require("xsa.monitor.widgets.overview.CannedChart");
dojo.require("xsa.monitor.widgets.overview.MetricSnapshot");
dojo.require("xsa.monitor.widgets.overview.PoolDistribution");
dojo.requireLocalization("xsa.monitor.widgets.layout", "xsa_messages");

dojo.declare("xsa.monitor.widgets.layout.DataCacheDetailLayout", [dijit._Widget, dijit._Templated], {   
    
    templatePath: dojo.moduleUrl("xsa","monitor/widgets/layout/templates/DataCacheDetailLayout.html"),
	widgetsInTemplate: true,
    
    xaxis : null,
    yaxis : null,
    layout : null,
    started : false,
    interval : null,
    field : null,
    fieldMap : {},
    fieldMap2 : {},
    selectedItems : {},
    xsa_messages : null,
    
    constructor: function(kwArgs) {
        console.log("DataCacheDetailLayout constructor called....");
		if(!kwArgs) { 
			kwArgs = {};
		}
		this.seriesID = kwArgs.seriesID ? kwArgs.seriesID : "grids";
		this.widgetType = kwArgs.type ? kwArgs.type : "servers";
		this.subType = kwArgs.subtype ? kwArgs.subtype : "procs";
        this.initField = kwArgs.init ? kwArgs.init : "availProcs";
        this.field = this.initField;
        console.log("this.field is " + this.field);
        this.scroll = kwArgs.scroll ? kwArgs.scroll : false;
        this.selectedItem = null;
        this.toolTip = null;
        this.gridSortedFlag = false;
        
        this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.layout", "xsa_messages");
        this.xsa_messages_overview = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
        if ( this.xsa_messages ) {
        	console.log("found messages ", this.xsa_messages);
        }
    },
    
    postCreate: function() {
        if ( this.splashScreen ) {
	        Bedrock.hide(this.splashScreen);
        }
        if ( this.dashboardView ) {
	        Bedrock.hide(this.dashboardView);
        }
        var options = {
            contentDefault: messages['RM11006']
        };
    },

	startup: function() {
	   console.log("DataCacheDetailLayout:startup");
	   this.selectResource("", this.seriesID, true);
	},
	    
    destroy: function() {
    	console.log("DataCacheDetailLayout:destroy", [this.borderContainer1]);
    },
    
    //
    // selectResource() modifies the data shown in the detail panels to be that 
    // of the resource with the given URI/ID. The action toolbar will also be 
    // updated to reflect the change.
    //
    selectResource: function(uri, id, dontShowAfter) {
		console.log("select resource " + uri + " id = " + id);
        Bedrock.hide(this.defaultMessage);

        if (dojo.style(this.splashScreen, 'display') == 'none' && !dontShowAfter)
        {
        	console.log("show splash screen");
            if (this.dashboardView) {
                Bedrock.hide(this.dashboardView);
            }    
            Bedrock.show(this.splashScreen);
        }  
        
        var title = "";
        var ids = id.split("!");
        for (var i=0; i<ids.length; i++) {
            if (title == "") {
                title = ids[i];
            } else {
                title = title+": "+ids[i];
            }
        }
        
        //SMP: Bug 19879 & 19880
        //from ActionMenu.js: to make sure focus moves to detail panel after item is selected
        console.log("DataCacheDetailLayout.selectResource this", this);
        // parent node is the div passed in constructor
        var parentElement = this.dashboardView.parentNode;
        console.log("parent:", this.dashboardView.parentNode);
        // set tab index to -1 for screen reader to acknowledge
        parentElement.setAttribute("tabIndex", "-1");
		// set aria live to polite to indicate to screen reader panel updates dynamically
		parentElement.setAttribute("aria-live", "polite");
        dijit.focus(parentElement);
        
        
        console.debug("title", [title]);
		this.selectedPanel.innerHTML = "&nbsp; "+this.xsa_messages_overview.XSA30101A+" "+title;
		dojo.style( this.selectedPanel, 'display: block;');
		dojo.removeClass(this.selectedPanel, 'heading');
		dojo.removeClass(this.selectedPanel, 'action-menu-default-message');		
		dojo.addClass(this.selectedPanel, 'action-menu-heading');
		
		this.server = id;

		console.log("now show the dashboard view");
        Bedrock.show(this.dashboardView);    	
		
		console.log("this.currentSelectedId", this.currentSelectedId);
		if ( this.currentSelectedId != id ) {
			this.createMetricSnapshot1();
			this.createGridCapacityByMapPool();
			this.createGridCapacityByZonePool();
		    //this.createCannedChart1();
		}

   		this.currentSelectedId = id;
    },

	createCannedChart1: function() {
	    console.log("create Chart for this grid " + this.server);
	    
	    if (this.chart) {
	       var newDiv = dojo.create("div");
	       dojo.place(newDiv, this.chart.domNode, "before");
	       console.debug("Destory cannedChart1");
	       this.chart.destroy();
	       this.cannedChartDiv = newDiv;
	    }
	    
		var dataSeries = [
		  {metricType: 'usedBytes', timeRange: '1hr', type: 'datagrid', seriesId: this.server, scale: ''},
		  {metricType: 'numEntries', timeRange: '1hr', type: 'datagrid', seriesId: this.server, scale: ''}
		];
		
		var p_cannedChart = {
		  chartDescriptors: dataSeries,
		  scopeId:'all',
		  numContribs:"30",
		  type:'datagrid'
		};

		console.log("Attach p:", p_cannedChart);
		this.chart = new xsa.monitor.widgets.overview.CannedChart(p_cannedChart, this.cannedChartDiv1);
		console.log("createCannedChart1", this.chart);
		//this.chart.render();
		this.chart.startup();
	},

	createMetricSnapshot1: function() {
	    console.log("create metric snapshot 1 for this grid " + this.server);

	    if (this.metric) {
	       var newDiv = dojo.create("div");
	       dojo.place(newDiv, this.metric.domNode, "before");
	       this.metric.destroy();
	       this.metricSnapshotDiv1 = newDiv;
	    }

		var dataSeries = [
		  {metricType: 'usedBytes', type: 'datagrid', seriesId: this.server, scale: ''},
		  {metricType: 'zones', type: 'datagrid', seriesId: this.server, scale: ''}
		];
		
		var p_metric = {
		  titleKey: 'XSA30502',
		  chartDescriptors: dataSeries,
		  scopeId:this.server,
		  type:'datagrid',
		  ariaLevel:'3'
		};

		console.log("Attach p:", p_metric);
		this.metric = new xsa.monitor.widgets.overview.MetricSnapshot(p_metric, this.metricSnapshotDiv1);
		console.log("createMetricSnapshot1", this.metric);
		this.metric.startup();
	},
	
	createGridCapacityByMapPool: function() {
	    console.log("createGridCapacityByMapPool for this grid " + this.server);

	    if (this.poolChart1) {
	       var newDiv = dojo.create("div");
	       dojo.place(newDiv, this.poolChart1.domNode, "before");
	       this.poolChart1.destroy();
	       this.poolChartDiv1 = newDiv;
	    }

	    var metricTypes = [ "usedBytes" ];
		var p_pool = {
			breakdownType:'datacache',
			scopeId:this.server,
			numContribs:'20',
			metricType: metricTypes,
			timeRange:'1hr',
			seriesType:'map',
			ariaLevel:'3'
		};

		this.poolChart1 = new xsa.monitor.widgets.overview.PoolDistribution(p_pool, this.poolChartDiv1);
		this.poolChart1.startup();
	},
	
	createGridCapacityByZonePool: function() {
	    console.log("createGridCapacityByZonePool for this grid " + this.server);

	    if (this.poolChart2) {
	       var newDiv = dojo.create("div");
	       dojo.place(newDiv, this.poolChart2.domNode, "before");
	       this.poolChart2.destroy();
	       this.poolChartDiv2 = newDiv;
	    }

	    var metricTypes = [ "usedBytes" ];
		var p_pool = {
			breakdownType:'datacache',
			scopeId:this.server,
			numContribs:'20',
			metricType: metricTypes,
			timeRange:'1hr',
			seriesType:'zone',
			ariaLevel:'3'
		};

		this.poolChart2 = new xsa.monitor.widgets.overview.PoolDistribution(p_pool, this.poolChartDiv2);
		this.poolChart2.startup();
	}
    
});
