/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.layout.OverviewDataCacheLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");

dojo.require("xsa.monitor.widgets.overview.PoolDistribution");
dojo.require("xsa.monitor.widgets.overview.SparklineCollection");
dojo.require("xsa.monitor.widgets.overview.HiLowChart");

dojo.requireLocalization("xsa.monitor.widgets.layout", "xsa_messages");
dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.layout.OverviewDataCacheLayout", [dijit._Widget, dijit._Templated], {   
    
    templatePath: dojo.moduleUrl("xsa","monitor/widgets/layout/templates/OverviewDataCacheLayout.html"),
	widgetsInTemplate: true,

    xsa_messages : null,
    isAppliance : null,
    
    constructor: function(kwArgs) {
        console.log("OverviewDataCacheLayout constructor called....");
		if(!kwArgs) { 
			kwArgs = {};
		}

		this.titleText = kwArgs.titleText ? kwArgs.titleText : "";
		
		this.isAppliance = kwArgs.isAppliance;
		
        this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.layout", "xsa_messages");
        this.xsa_messages_overview = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
        if ( this.xsa_messages ) {
            console.log("found messages ", this.xsa_messages);
        }
        
        this.tabTitle1 = this.xsa_messages_overview.usedBytes;
        this.tabTitle2 = this.xsa_messages_overview.avgTP;
        this.tabTitle3 = this.xsa_messages_overview.avgTranTime;
    },
    
    postCreate: function() {
        console.log("OverviewDataCacheLayout:postCreate");

        if ( this.splashScreen ) {
	        Bedrock.hide(this.splashScreen);
        }
        var options = {
            contentDefault: messages.RM11006
        };
        
    },

	startup: function() {
        this.inherited(arguments);
	    console.log("OverviewDataCacheLayout:startup");
        this.populateTitle();
        this.createUsedBytesPoolDiv();
        //this.createUsedBytesPoolDiv();
        this.createUsedBytesHilowDiv();
        
        var tabs = dijit.byId("_tabDiv");
        tabs.resize();
        dojo.connect(tabs,"_transition", dojo.hitch(this, this.tabSelected));
    	//
	    //  listen for onClicks, so we can update aria-pressed attribute
    	//
	    var children = tabs.tablist.getChildren();
	    for( var i = 0; i < children.length; ++i )
	    {
	        dojo.connect(children[i],"onClick",dojo.hitch(this,this._removeWaiAttributes));
	    }	
	    this._removeWaiAttributes();
	},
	
	tabSelected: function(newTab, oldTab) {
		console.debug("Previous tab ", oldTab || "nothing");
		console.debug("New Tab ", newTab);
		if (newTab.id == "usedBytes") {
			if (!this.usedBytesPool) {
		        this.createUsedBytesPoolDiv();
		        this.createUsedBytesHilowDiv();
			}
		} else if (newTab.id == "avgTP") {
			if (!this.avgTPHilow) {
				this.createAvgTPSparkline();
				this.createAvgTPHilow();
			}
		} else if (newTab.id == "avgTranTime") {
			if (!this.avgTranTimeHilow) {
				this.createAvgTranTimeSparkline();
				this.createAvgTranTimeHilow();
			}
		}
	},
	    
    destroy: function() {
        console.log("OverviewDataCacheLayout:destroy");
		if(this.usedBytesPool) this.usedBytesPool.destroy();
		if(this.usedBytesHilow) this.usedBytesHilow.destroy();
		if(this.avgTranTimeHilow) this.avgTranTimeHilow.destroy();
		if(this.avgTPHilow) this.avgTPHilow.destroy();
		if(this.avgTranTimeSparkline) this.avgTranTimeSparkline.destroy();
		if(this.avgTPSparkline) this.avgTPSparkline.destroy();
		if(this._tabDiv) this._tabDiv.destroyRecursive();
		this.inherited(arguments);
    },
	   
    //
    //	this function removed invalid WAI-ARIA state.
    //  aria-pressed is invalid for a tab
    //
    _removeWaiAttributes: function(){
        var tabs = dijit.byId("_tabDiv");
	    var children = tabs.tablist.getChildren();
	    dojo.forEach(children,function(child){
	        dijit.removeWaiState(child.focusNode || child.domNode,"pressed");
	    });
    },
	 
    populateTitle: function() {
        this.overview_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
        //this.selectedPanel.innerHTML = "&nbsp; "+this.overview_messages.XSA30101A+" "+this.overview_messages.XSA30103;
        this.selectedPanel.innerHTML = "&nbsp; "+this.titleText;
    },

	createAvgTPSparkline: function() {
	    console.log("createAvgTPSparkline");
	    
	    //if (this.chart) {
	       //var newDiv = dojo.create("div");
	       //dojo.place(newDiv, this.chart.domNode, "before");
	       //this.chart.destroy();
	       //this.cannedChartDiv = newDiv;
	    //}
	    
		var p_spark = {
			type:'datacache',
			metricType:'avgTP', 
			top:'true',
			numContribs:'5',
			ariaLevel:'2'
		};
		this.avgTPSparkline = new xsa.monitor.widgets.overview.SparklineCollection(p_spark, this._avgTPSparklineDiv);
		this.avgTPSparkline.startup();
	},
	
	createAvgTranTimeSparkline: function() {
	    console.log("createAvgTranTimeSparkline");
	    
	    //if (this.chart) {
	       //var newDiv = dojo.create("div");
	       //dojo.place(newDiv, this.chart.domNode, "before");
	       //this.chart.destroy();
	       //this.cannedChartDiv = newDiv;
	    //}
		
		var p_spark = {
			type:'datacache',
			metricType:'avgTranTime',
			top:'true',
			numContribs:'5',
			ariaLevel:'2'
		};
		this.avgTranTimeSparkline = new xsa.monitor.widgets.overview.SparklineCollection(p_spark, this._avgTranTimeSparklineDiv);
		this.avgTranTimeSparkline.startup();
	},
	
	createAvgTPHilow: function() {
	    console.log("createAvgTPHilow");
	    
	    //if (this.chart) {
	       //var newDiv = dojo.create("div");
	       //dojo.place(newDiv, this.chart.domNode, "before");
	       //this.chart.destroy();
	       //this.cannedChartDiv = newDiv;
	    //}
	    
	    var p_hilow = {
	        type:'datacache',
	        metricType:'avgTP',
	        timeRange:'1hr',
	        scopeId:'all',
	        numContribs:'1',
	        ariaLevel:'2'
	    };
	    
		this.avgTPHilow = new xsa.monitor.widgets.overview.HiLowChart(p_hilow, this._avgTPHilowDiv);
		this.avgTPHilow.startup();
	},
	
	createAvgTranTimeHilow: function() {
	    console.log("createAvgTranTimeHilow");
	    
	    //if (this.chart) {
	       //var newDiv = dojo.create("div");
	       //dojo.place(newDiv, this.chart.domNode, "before");
	       //this.chart.destroy();
	       //this.cannedChartDiv = newDiv;
	    //}
	    
	    var p_hilow = {
	        type:'datacache',
	        metricType:'avgTranTime',
	        timeRange:'1hr',
	        scopeId:'all',
	        numContribs:'1',
	        ariaLevel:'2'
	    };
	    
		this.avgTranTimeHilow = new xsa.monitor.widgets.overview.HiLowChart(p_hilow, this._avgTranTimeHilowDiv);
		this.avgTranTimeHilow.startup();
	},
	
	createUsedBytesHilowDiv: function() {
	    console.log("createUsedBytesHilowDiv");

	    //if (this.usedBytesHilow) {
	       //var newDiv = dojo.create("div");
	       //dojo.place(newDiv, this.usedBytesHilow.domNode, "before");
	       //this.usedBytesHilow.destroy();
	       //this._usedBytesHilowDiv = newDiv;
	    //}

		var p_hilow = {
		    type:'datacache',
		    metricType:'usedBytes',
		    timeRange:'1hr',
		    scopeId:'all',
		    numContribs:'1',
		    ariaLevel: '2'
		};
		
		this.usedBytesHilow = new xsa.monitor.widgets.overview.HiLowChart(p_hilow, this._usedBytesHilowDiv);
		this.usedBytesHilow.startup();
	},
	
	createUsedBytesPoolDiv: function() {
	    console.log("createUsedBytesPoolDiv enter");

	    //if (this.usedBytesPool) {
	       //var newDiv = dojo.create("div");
	       //dojo.place(newDiv, this.usedBytesPool.domNode, "before");
	       //this.usedBytesPool.destroy();
	       //this._usedBytesPoolDiv = newDiv;
	    //}

    	var metricTypes = [ "primaryUsedBytes", "replicaUsedBytes", "unused_gridCap" ];
    	var enableComboBox = "true";
	    if (this.isAppliance == "false") {
		    metricTypes = [ "usedBytes"];
		    enableComboBox = "false";
	    }
	    
		var p_pool = {
			scopeId:'all',
			numContribs:'20',
			metricType: metricTypes,
			timeRange:'1hr',
			breakdownType: 'datacache',
			ariaLevel: '2',
			poolMetricName: 'usedBytes',
			enableComboBox: enableComboBox
		};
		this.usedBytesPool = new xsa.monitor.widgets.overview.PoolDistribution(p_pool, this._usedBytesPoolDiv);
		this.usedBytesPool.startup();
		//
		// set the 'for' attribute for combo box to comply with WAI-ARIA
		//
		//var label = document.getElementById('pool-row-label');
		//label.setAttribute('for', 'widget_' + this.usedBytesPool.combo.id);
	}
    
});
