/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.overview.CannedChart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.HorizontalRule");
dojo.require("dijit.form.HorizontalRuleLabels");
dojo.require("dijit.form.HorizontalSlider");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");

dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.date.locale");

dojo.require("dojox.charting.themes.PlotKit.blue");
//dojo.require("dojox.widget.DataPresentation");

dojo.require("dojox.charting.action2d.Magnify");
dojo.require("dojox.charting.action2d.Tooltip");
dojo.require("dojox.charting.action2d.Highlight");

dojo.require("xsa.monitor.widgets.ext.DataPresentation");
dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.overview.CannedChart", [dijit._Widget, dijit._Templated], {   

    templatePath: dojo.moduleUrl("xsa","monitor/widgets/overview/templates/CannedChart.html"),
    widgetsInTemplate: true,

     /* holds a reference to localized message bundle */
    xsa_messages : null,
    
    /* THESE ATTRIBUTES MUST BE SET WHEN INSTANTIATING THE WIDGET */
    /* the type of data artifact to aggregate the stats to (ie datacache, grid, map) */
    type : null,
    /* the time range to show */
    timeRange: null,
    /* the scope "all" or fully qualified name of artifact to scope the chart to , like "anngrid" or "anngrid!annMap" */
    scopeId : null,
    /* the number of contributors to the breakdown, the rest will be grouped into an "other" category */
    numContribs: null,
    titleKey: null,
    /* 
     * This is a delimited list of descriptors for every data series to be displayed on the chart.
     * Sample:
     * var dataSeries = [
     *   {metricType: '', numberDatapoints: '', timeRange: '', type: '', seriesId: '', scale: ''},
     *   {metricType: '', numberDatapoints: '', timeRange: '', type: '', seriesId: '', scale: ''}
     * ];
     *
     * var p_cannedChart = {
     *   chartDescriptors: dataSeries
     * };
     */
    chartDescriptors: null,
    

    /* set by the script */
    responseData : null,

     constructor: function(kwArgs) {
        //console.log("CannedChart dashboard constructor called....");
          if(!kwArgs) { 
               kwArgs = {};
          }
          /* initialize global vars for the widget */
          this.metricType1 = kwArgs.metricType1 ? kwArgs.metricType1 : "consumeCapacity";
          this.metricType2 = kwArgs.metricType2 ? kwArgs.metricType2 : "cacheEntriesNumber";
          this.type = kwArgs.type ? kwArgs.type : "datacache";
          this.timeRange = kwArgs.timeRange ? kwArgs.timeRange : "1hr";
          this.scopeId = kwArgs.scopeId ? kwArgs.scopeId : "all";
          this.numContribs = kwArgs.numContribs ? kwArgs.numContribs : "1";
          this.titleKey = kwArgs.titleKey ? kwArgs.titleKey : this.titleKey;
          this.chartDescriptors = kwArgs.chartDescriptors ? kwArgs.chartDescriptors : [];
          
          this.ariaLevel = kwArgs.ariaLevel ? kwArgs.ariaLevel: '1';

         /* set up the messages */
         this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
         if ( this.xsa_messages ) {
              //console.log("found messages ", this.xsa_messages);
              this.chartTabTitle = this.xsa_messages.XSA30120;
              this.tableTabTitle = this.xsa_messages.XSA30121;
         }
    },
    
    postCreate: function() {
        //console.log("CannedChart postCreate called....");
        if ( this.splashScreen ) {
             //console.log("hide splashScreen");
             Bedrock.hide(this.splashScreen);
        }
        if ( this.dashboardView ) {
             //console.log("hide dashboardView");
             Bedrock.hide(this.dashboardView);
        }
        
        /* load up the canned chart data */
        this.createTimeComboBox();
        this.loadCannedData();
        //this.buildCannedChart();
        //this.retroFitWidget();
        
        //Bug 19537 Hack: not sure why root template
        //title is being set to [object divElement]
		this.cannedChartRootDiv.title = "";
    },
     
    retroFitWidget: function() {
        //console.log("Adjusting widget to: "+screen.width+"x"+screen.height);
        //console.log("this.tabContainer",this.tabContainer);

        //dojo.removeClass(this.tabContainer, 'cannedchart-tab');
        //dojo.addClass(this.tabContainer, 'cannedchart-tab'+screen.width);

        //dojo.removeClass(this.cannedchart, 'cannedchart-tab');		
        //dojo.addClass(this.cannedchart, 'cannedchart-chart'+screen.width);

        //dojo.removeClass(this.cannedlegend, 'cannedchart-legend');		
        //dojo.addClass(this.cannedlegend, 'cannedchart-legend'+screen.width);

        //var newWidth = parseInt(screen.width) - parseInt(this.tabContainer.style.width);
        ////console.log("this.tabContainer.style.width", this.tabContainer.style.width);
        ////console.log("newWidth", newWidth);
        //dojo.style(this.tabContainer, "width",  newWidth+"px");
    },
	
	/**
	 * Creates data presentation widget.  If there are exactly 2 chart descriptors are specified
	 * then the data presentation widget creates the chart.  If there are 2 chart descriptors
	 * then the data presentation widget only creates the grid and defers the chart creation to 
	 * createCannedChart.
	 */
     //buildCannedChart: function(){
     buildCannedChart: function(response, ioArgs){
		this.fixHitRate(response);
        //console.log("CannedChart dashboard buildCannedChart called: "+[response]);

		try{
			var chartDescriptor = this.buildChartDescriptor();
	
			if(this._dp) {
				this._dp.setData(response, null);
			} else {
				this._dp = new xsa.monitor.widgets.ext.DataPresentation(this.cannedchart, {
						type: "chart",
						chartType: "Lines",
						data: response,
						/*url: "/resources/chartdata/"+this.type,
						urlContent: {
							   scopeId: this.scopeId,
							   chartType: "basic",
							   axisLabel: "range",
							   maxContributors: this.numContribs,
							   chartDescriptor: chartDescriptor 
						},*/
						series: dojo.hitch(this, this.makeseries),
						labelMod: 14,
						gridNode:this.griddiv,
						legendHorizontal: false,
						legendNode: this.cannedlegend,
						titleNode: this.title,
						footerNode: this.footer,
						animate: true,
						//refreshInterval: 30000, //30 seconds
						//theme: "xsa.monitor.widgets.themes.xsa"
						theme: "dojox.charting.themes.PlotKit.blue"//"dojox.charting.themes.Distinctive",
						//theme: new dojox.charting.Theme({ fillcolors: ["green", "darkblue"], colors: ["lightgreen","lightblue"], markers: ["m-1,0 c0,-1.3 2,-1.3 2,0 m-2,0 c0,1.3 2,1.3 2,0", "m-1,0 c0,-1.3 2,-1.3 2,0 m-2,0 c0,1.3 2,1.3 2,0"] })
				});
			}
			//console.debug("chart",[this._dp]);

		 }catch(err){
			  //console.debug("error",[err]);
		 }

		 //console.debug("this._dp.chartWidget",[this._dp.chartWidget]);
		 var anim1a = new dojox.charting.action2d.Magnify(this._dp.chartWidget, "primary-Lines");
		 var anim10b = new dojox.charting.action2d.Highlight(this._dp.chartWidget, "primary-Lines");
         var anim10c = new dojox.charting.action2d.Tooltip(this._dp.chartWidget, "primary-Lines", {text: dojo.hitch(this, this.getToolTip)});
            
		 var anim2a = new dojox.charting.action2d.Magnify(this._dp.chartWidget, "secondary-Lines");
		 var anim20b = new dojox.charting.action2d.Highlight(this._dp.chartWidget, "secondary-Lines");
         var anim20c = new dojox.charting.action2d.Tooltip(this._dp.chartWidget, "secondary-Lines", {text: dojo.hitch(this, this.getToolTip)});
            
            /*for (var i=0; i<this._dp.chartWidget.axes.independent.labels.length; i++) {
            	var text = this._dp.chartWidget.axes.independent.labels[i].text;
            	if (text != "") {
            		this._dp.chartWidget.axes.independent.labels[i].text = this.stackTimeStamp(text); 
            	}
            }*/
         //console.debug("this._dp.chartWidget",[this._dp.chartWidget]);
         //this._dp.chartWidget.render(); //12145: this renders the chart twice

         Bedrock.fadeCommon(this.splashScreen);
         if ( this.dashboardView ) {
             Bedrock.show(this.dashboardView);
         }     
     },
	 
	/** 
      * The hit rates are incorrect because if we add up 4 data points, the actual hit rate is different than the average
	  * of the hit rates for those 4 points.  Adding this code here because it is easier to fix in the javascript than on the java side.
	  * If anyone else needs to call the ChartDataCollector to get the hit rates, then we need to move this code up to the java code.
      */
	 fixHitRate: function(response) {
		var getCountValues;
		var hitCountValues;
		var hitRateValues;
		dojo.forEach(response.series, function(column){
			if(column.fullName==column.name+"!totalGetCount") getCountValues = column.scaledValues;
			if(column.fullName==column.name+"!totalHitCount") hitCountValues = column.scaledValues;
			if(column.fullName==column.name+"!hitRate") hitRateValues = column.scaledValues;
		});
		
		//Recalculate the new hit rate.
		dojo.forEach(getCountValues, function(getCountValue, i){
			if(getCountValue!=0) {
				var newHitRate = hitCountValues[i] / getCountValue + ''; //Add '' to convert it to a string
				if(newHitRate.length > 4) newHitRate = newHitRate.substring(0,4); //truncate it to look like a double
				hitRateValues[i] = newHitRate;
			}
		}, this);
	 },
     
     /** 
      * Required by the data presentation widget to manipulate the json data.
      */
	 makeseries: function(data) {
	   //console.debug("makeseries",[data]);
	   this.responseData = data;

	   //Set series legend labels
	   for (var i=0; i<this.responseData.series.length; i++) {
		    var units = "";
		    if (this.responseData.series[i].units) {
               var unitsLabel = this.xsa_messages[this.responseData.series[i].units+"_abv"];
               if (unitsLabel != undefined)
            	   units = " ("+unitsLabel+")";
		    }
	       var metricName = this.translate(this.chartDescriptors[i].metricType, [], this.responseData);
	       this.responseData.series[i].legend = metricName+units;
	   }
	   
	   var title = "";
	   if (this.titleKey != undefined) {
		   title = this.xsa_messages[this.titleKey];
	   } else {
	       title = this.responseData.series[0].legend+" "+this.xsa_messages.XSA30400+" "+this.responseData.series[1].legend;
	   }
	   //console.debug("title",[title]);
	   this.responseData.title=title;

	   var footer = this.translate(this.subtitleKey, this.subtitleKeyParms, this.responseData);
	   //console.debug("footer",[footer]);
	   this.responseData.footer=footer;

	   var name = this.xsa_messages.XSA30123;
	   //console.debug("name",[name]);
	   
	   //console.debug("this.responseData",[this.responseData]);
	   
		//Parse from string to int
		//var count = 1;
		for (i=0; i<this.responseData.series.length; i++) {
		    for (var j=0; j<this.responseData.series[i].scaledValues.length; j++) {
		        var intValue = parseFloat(this.responseData.series[i].scaledValues[j]);
		        //if (intValue == 0) { intValue = intValue+count; count=count+100; }
		        this.responseData.series[i].scaledValues[j] = intValue;		    
		    }
		}
		
	   //Convert long time into pretty locale time
       for (i=0; i<this.responseData.series[0].range.length; i++) {
          var seriesRange = this.convertTime(this.responseData.series[0].range[i]);
          //seriesRange = this.stackTimeStamp(seriesRange);
          this.responseData.series[0].range[i] = seriesRange;
       }

        
	   //Convert long time into pretty locale time
       for (i=0; i<this.responseData.range.length; i++) {
          var time = this.convertTime(this.responseData.range[i]);
          this.responseData.range[i] = time;
       }
	   
	   var txt = [];
       txt.push({ datapoints: "series[0].range", name: name, type: "range", formatter: this.getGridDateFormat, chart: false });
       for (i=0; i<this.responseData.series.length; i++) {
     	  if ((this.responseData.series[i].current == "-1" || this.responseData.series[i].current == -1)
    		  && this.chartDescriptors[i].hideOnNegative != undefined
    		  && this.chartDescriptors[i].hideOnNegative == "true")
    		  continue;
     	  
          txt.push({
        	  datapoints: "series["+i+"].scaledValues",
              namefield: "series["+i+"].legend",
              axis: this.chartDescriptors[i].axis,
              chartType: "bar"
          });
       }

	   //console.debug("txt", txt);
       return txt;
	 },

	 getGridDateFormat: function(date) {
		 //console.debug("****************************getGridDateFormat:", [date]);
		 return date;
	 },
	 
     /** 
      * Used by getToolTipText to figure how which cooresponding
      * index will retrieve that time value for the specified field item.
      */
     getXIndex: function(field) {
         for (var i=0; i<field.plot.series.length; i++) {
		     var data = field.plot.series[i].data;
			 for (var j=0; j<data.length; j++) {
			     if (data[j] == field.y) {
			         return i;
			     }
			 }
         }
         return 0;
     },

	 /** 
	  * Rest call to retrieve live data.  After live data is retrieved,
	  * buildCannedChart is called.
	  */
     loadCannedData: function(){
        //console.log("CannedChart loadCannedData called....");
        var chartDescriptor = this.buildChartDescriptor();
        var deferred = dojo.xhrGet({
           url: "/resources/chartdata/"+this.type,
		   //url: "../.../../../../xsa/monitor/widgets/tests/data/missingValues.json", //Use this url for testing
           handleAs: "json",
           preventCache: true,
           content: {
               scopeId: this.scopeId,
               chartType: "basic",
               axisLabel: "range",
               maxContributors: this.numContribs,
               chartDescriptor: chartDescriptor 
           },
		   headers: {
              'Accept': 'application/json'
           },
           load: dojo.hitch(this, this.buildCannedChart),
           error: function(error) {
               //console.log("An unexpected error occurred: " + error);
           }
        });
        //var data = this.tempChartData();
        //this.buildCannedChart(data, "");

        //console.log("done, waiting for callback....");

        if (dojo.style(this.splashScreen, 'display') == 'none')
        {
            //console.log("show splash screen");
            if (this.dashboardView) {
                Bedrock.hide(this.dashboardView);
            }    
            Bedrock.show(this.splashScreen);
        }
     },
     
     /** 
      * Builds a chart descriptor based on the array of chart descriptors passed in
      * at widget constructor time.
      *
      * HACK: Currently the seriesId is being replaced by "all" because stats infastructure
      * is not working.  Also "all" is not really retrieving all which will cause some more issues.
      */
     buildChartDescriptor: function() {
        //console.log("this.chartDescriptors", this.chartDescriptors);
        var chartDescriptor = "";

        var timeRange = "1hr";
        if (this.combo.item) {
            timeRange = this.combo.item.field[0];
        }

        if (this.chartDescriptors != null) {
           for (var i=0; i<this.chartDescriptors.length; i++) {
              if (this.chartDescriptors[i].numberDatapoints == undefined) {
                 this.chartDescriptors[i].numberDatapoints = "all";
              }
              
              if (chartDescriptor == "") {
                 chartDescriptor = this.chartDescriptors[i].metricType+":"+this.chartDescriptors[i].numberDatapoints+":"+timeRange+":"+this.chartDescriptors[i].type+":"+this.chartDescriptors[i].seriesId+":"+this.chartDescriptors[i].scale;
              } else {
                 chartDescriptor = chartDescriptor+";"+this.chartDescriptors[i].metricType+":"+this.chartDescriptors[i].numberDatapoints+":"+timeRange+":"+this.chartDescriptors[i].type+":"+this.chartDescriptors[i].seriesId+":"+this.chartDescriptors[i].scale;
              }
           }
        }
        //console.log("chartDescriptor", chartDescriptor);
        return chartDescriptor;
     },
     
     /** 
      * Creates the drop down box with the time ranges.  Time ranges
      * are retrieved from the timeranges rest resource.  "/grids" probably
      * can be removed from the url, just haven't had time to test it.
      * After a change has been detected we reload the chart.
      */
     createTimeComboBox: function() {
        this.timeRange.innerHTML = this.xsa_messages.XSA30401;
        var stateStore = new dojo.data.ItemFileReadStore({
            url: "/resources/timeranges/grids"
        });
        this.combo = new dijit.form.ComboBox({
            //id: "timeRangeComboBox",
            name: "state",
            value: this.xsa_messages.range_1hr,
            store: stateStore,
            searchAttr: "name",
            onChange: dojo.hitch(this, this.loadCannedData)
        },

        this.selectTimeRange);
		
		dojo.attr(this.timeRange, "for", this.combo.id);
     },
     
    /** 
     * Retrieves translated text. 
     */
	translate: function(key, parms, data){
	   //console.debug("translate enter",[key,parms,data]);
	   var message = this.xsa_messages[key];
	   //console.debug("message",[message]);
	   if (parms){
	       var parm;
	       for (parm in parms){
                if (this.xsa_messages[parms[parm].parmValue]){
                    //console.debug("parms[parm].parmValue",[parms[parm].parmValue]);
                    //look for the message key sub parm to be another message key
                    var name = parms[parm].parmName;
                    var value = this.xsa_messages[parms[parm].parmValue];
                    message = message.replace(name,value);
                }else{
                    //check if it is set in the data
                    if (this.responseData[parms[parm].parmValue]){
						//console.debug("parms[parm].parmValue",[parms[parm].parmValue]);
						var name2 = parms[parm].parmName;
                        //console.debug("name2",[name2]);
                        var value2 = this.xsa_messages[this.responseData[parms[parm].parmValue]];
						//console.debug("value2",[value2]);
                        message = message.replace(name2,value2);
                    }
              }
	       }
	   }
	   //console.debug("translate exit",[message]);
	   return message;
	},

     
     getToolTip: function(field) {
    	return this.getToolTipText(field, -1); 
     },
     
     getToolTipText: function(field, offset) {
         //console.debug("field",[field]);
         //console.debug("this.responseData",[this.responseData]);

         var dataSeriesIndex = -1;
         for (var i=0; i<this.responseData.series.length; i++) {
        	 var legend = this.responseData.series[i].legend;
        	 //console.debug("legend: ", [legend]);
        	 if (legend == field.run.name) {
        		 dataSeriesIndex = i;
        		 break;
        	 }
         }
         
         //console.debug("dataSeriesIndex: ", [dataSeriesIndex]);
         //Handle dynamic data size units.
         var unitsLabel = "";
         var units = this.responseData.series[dataSeriesIndex].units;
         if (units != undefined) {
        	 unitsLabel = this.xsa_messages[units+"_abv"];
        	 if (unitsLabel == undefined) {
        		 unitsLabel = this.xsa_messages[units];
        	 }
        	 
        	 if (unitsLabel == undefined) {
        		 unitsLabel = "";
        	 }
         }
         
         return "<b>"+this.responseData.range[parseInt(field.x)+offset].replace("<br>", " ")+"</b><br />"+field.run.name+": "+field.y+" "+unitsLabel;
      },
          
     /** 
      * Determines how the dates are displayed.
      */
     convertTime: function(time) {
    	var convertedTime = time;

        var timeRange = "1hr";
        if (this.combo.item) {
            timeRange = this.combo.item.field[0];
        }
    	
		var newDate = new Date( );
		newDate.setTime( convertedTime );
		convertedTime = dojo.date.locale.format(newDate, {formatLength: "short", locale: djConfig.locale});
        return convertedTime;
    },

    stackTimeStamp: function(time) {
       var times = time.split(" ");
       return times[0]+"<br>"+times[1]+" "+times[2];
    }

});
