/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.overview.MetricSnapshot");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.overview.MetricSnapshot", [dijit._Widget, dijit._Templated], {   

    templatePath: dojo.moduleUrl("xsa","monitor/widgets/overview/templates/MetricSnapshot.html"),
     widgetsInTemplate: true,

     /* holds a reference to localized message bundle */
    xsa_messages : null,
    
    /* THESE ATTRIBUTES MUST BE SET WHEN INSTANTIATING THE WIDGET */
    /* the scope "all" or fully qualified name of artifact to scope the chart to , like "anngrid" or "anngrid!annMap" */
    scopeId : null,
    /* List of metrics */
    metricList : null,
    /* "all" or <1hr|1day|1wk|1mon> ... use the values in the contants in StatsUtil.  This tells the collector which time period map to pull data from.  The "all" is only for high/low charts and will create a current datapoint per time range in a single data series  */
    timeRange : null,
    /*  this is the artifact type being charted .. (ie datacache, grid, map, zone, partition, etc).  The types here must align with what is found in StatsDescriptorUtil */
    type : null,
    /* the message key of title of the pool breakdown - default is set on rest call if not set */
    titleKey : null,
    titleKeyParms : null,
    /* 
     * This is a delimited list of descriptors for every data series to be displayed on the chart.
     * Sample:
     * var dataSeries = [
     *   {metricType: '', timeRange: '', type: '', seriesId: '', scale: ''},
     *   {metricType: '', timeRange: '', type: '', seriesId: '', scale: ''}
     * ];
     *
     * var p_cannedChart = {
     *   chartDescriptors: dataSeries
     * };
     */
    chartDescriptors: null,
	
	_domNodesToClean: null,

    /* CONSTANTS */
    PURPLE: "rgb(160,32,240)",
     
     constructor: function(kwArgs) {
        console.log("MetricSnapshot constructor called....");
          if(!kwArgs) { 
               kwArgs = {};
          }
          /* initialize global vars for the widget */
          this.titleKey = kwArgs.titleKey ? kwArgs.titleKey : "XSA30501";
          this.titleKeyParms = kwArgs.titleKeyParms ? kwArgs.titleKeyParms : [];
          this.chartDescriptors = kwArgs.chartDescriptors ? kwArgs.chartDescriptors : [];
          
          this.ariaLevel = kwArgs.ariaLevel ? kwArgs.ariaLevel: '1';
        
        /* set up the messages */
        this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
        if ( this.xsa_messages ) {
             console.log("found messages ", this.xsa_messages);
        }
             
		this._domNodesToClean = [];
    },
     
    postCreate: function() {
        console.log("MetricSnapshot postCreate called....");
        if ( this.splashScreen ) {
             console.log("hide splashScreen");
             Bedrock.hide(this.splashScreen);
        }
        if ( this.dashboardView ) {
             console.log("hide dashboardView");
             Bedrock.hide(this.dashboardView);
        }

        /* set up the caption for the table */
	    var title = this.translate(this.titleKey, this.titleKeyParms, "[]");
	    console.debug("title",[title]);
	    this.title.innerHTML=title;
        
        /* load up the sparkline data */
        this.loadMetricSnapshotData();
        
        //Bug 19537 Hack: not sure why root template
        //title is being set to [object divElement]
		this.metricRootDiv.title = "";
     },
     
	translate: function(key, parms, data){
	   console.debug("translate enter",[key,parms,data]);
	   var message = this.xsa_messages[key];
	   if (parms){
	       var parm;
	       for (parm in parms){
                if (this.xsa_messages[parms[parm].parmValue]){
                    //look for the message key sub parm to be another message key
                    var name = parms[parm].parmName;
                    var value = this.xsa_messages[parms[parm].parmValue];
                    message = message.replace(name,value);
                }else{
                    //check if it is set in the data
                    if (data[parms[parm].parmValue]){
						var name2 = parms[parm].parmName;
                        console.debug("name2",[name2]);
                        var value2 = this.xsa_messages[this.responseData[parms[parm].parmValue]];
						console.debug("value2",[value2]);
                        message = message.replace(name2,value2);
                    }
              }
	       }
	   }
	   console.debug("translate exit",[message]);
	   return message;
	},

     loadMetricSnapshotData: function(){
        console.log("MetricSnapshot loadMetricSnapshotData called....");
        
        var chartDescriptor = this.buildChartDescriptor();
        var deferred = dojo.xhrGet({
           url: "/resources/chartdata/"+this.type,
           handleAs: "json",
           preventCache: true,
           content: {
               scopeId: this.scopeId,
               chartType: "generic",
               axisLabel: "range",
               maxContributors: "1",
               chartDescriptor: chartDescriptor
           },
		   headers: {
              'Accept': 'application/json'
           },
           load: dojo.hitch(this, this.buildMetricSnapshot),
           error: function(error) {
               console.log("An unexpected error occurred: " + error);
           }
        });

        console.log("done, waiting for callback....");

        if (dojo.style(this.splashScreen, 'display') == 'none')
        {
            console.log("show splash screen");
            if (this.dashboardView) {
                Bedrock.hide(this.dashboardView);
            }    
            Bedrock.show(this.splashScreen);
        }
     },

     /** 
      * Builds a chart descriptor based on the array of chart descriptors passed in
      * at widget constructor time.
      *
      * HACK: Currently the seriesId is being replaced by "all" because stats infastructure
      * is not working.  Also "all" is not really retrieving all which will cause some more issues.
      */
     buildChartDescriptor: function() {
        console.log("this.chartDescriptors", this.chartDescriptors);
        var chartDescriptor = "";

		//<metricType>:<numDataPoints>:<timeRange>:<seriesType>:<seriesID>:<scaler>
        if (this.chartDescriptors != null) {
           for (var i=0; i<this.chartDescriptors.length; i++) {
              if (chartDescriptor == "") {
                 chartDescriptor = this.chartDescriptors[i].metricType+":current:"+"1hr"+":"+this.chartDescriptors[i].type+":"+this.chartDescriptors[i].seriesId+":"+this.chartDescriptors[i].scale;
              } else {
                 chartDescriptor = chartDescriptor+";"+this.chartDescriptors[i].metricType+":current:"+"1hr"+":"+this.chartDescriptors[i].type+":"+this.chartDescriptors[i].seriesId+":"+this.chartDescriptors[i].scale;
              }
              /* if (chartDescriptor == "") {
                 chartDescriptor = this.chartDescriptors[i].metricType+":all:"+"all"+":"+this.chartDescriptors[i].type+":"+"all"+":"+this.chartDescriptors[i].scale;
              } else {
                 chartDescriptor = chartDescriptor+";"+this.chartDescriptors[i].metricType+":all:"+"all"+":"+this.chartDescriptors[i].type+":"+"all"+":"+this.chartDescriptors[i].scale;
              }*/
           }
        }
        console.log("chartDescriptor", chartDescriptor);
        return chartDescriptor;
     },
     
     getUnits: function(response, i) {
         var units;
         if (response.series[i].units) {
           units = this.xsa_messages[response.series[i].units+"_abv"];
           if (!units) {
           	units = this.xsa_messages[response.series[i].units];
           }
         } else {
       	units = this.xsa_messages["u_"+this.chartDescriptors[i].metricType];
         }

         if (units) {
            units = " "+units;
         } else {
            units = "";
         }
         
         return units;
     },
     
     buildMetricSnapshot: function(response, ioArgs){
        console.debug('buildMetricSnapshot',[response, ioArgs]);
        /* add in content rows */
        if (this.chartDescriptors){
               var th_row = dojo.create("tr", {}, this.metricSnapshotTable);
			   var td_row = dojo.create("tr", {}, this.metricSnapshotTable);
			   this._domNodesToClean.push(th_row);
			   this._domNodesToClean.push(td_row);
	           for (var i=0; i<this.chartDescriptors.length; i++){
	              console.debug('chartDescriptors', [i, this.chartDescriptors[i]]);
		          //If value is negative and hideOnNegative is true skip the metric. 
	        	  if (response.series[i].current == "-1" 
	        		  && this.chartDescriptors[i].hideOnNegative != undefined
	        		  && this.chartDescriptors[i].hideOnNegative == "true")
	        		  continue;

					  this._domNodesToClean.push(dojo.create("th", {'class':"metric-label",bgcolor:"#D9E9FB",innerHTML:this.xsa_messages[this.chartDescriptors[i].metricType]}, th_row));
					  
					  var metric = "---";
	              if (response.series) {
	              	metric = response.series[i].current;
	              }
	              
	              var units = this.getUnits(response, i);
					  this._domNodesToClean.push(dojo.create("td", {'class':"metric-cell",innerHTML:metric+units}, td_row));					  
	           }
        }
     },
	 
	destroy: function() {
		//destroy the dom nodes that we created to prevent leaks
		dojo.forEach(this._domNodesToClean, dojo.destroy);
		this.inherited(arguments);
    }
     
});