/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.overview.PoolDistribution");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

//dojo.require("dojox.widget.DataPresentation");
dojo.require("xsa.monitor.widgets.ext.DataPresentation");
dojo.require("dojox.charting.themes.Distinctive");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.Tooltip");
dojo.require("dijit.form.ComboBox");

dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.overview.PoolDistribution", [dijit._Widget, dijit._Templated], {   

    templatePath: dojo.moduleUrl("xsa","monitor/widgets/overview/templates/PoolDistribution.html"),
    widgetsInTemplate: true,

     /* holds a reference to localized message bundle */
    xsa_messages : null,

    /* THESE ATTRIBUTES MUST BE SET WHEN INSTANTIATING THE WIDGET */
    /* the scope "all" or fully qualified name of artifact to scope the chart to , like "anngrid" or "anngrid!annMap" */
    scopeId : null,
    /* the number of contributors to the breakdown, the rest will be grouped into an "other" category */
    numContribs : null,
    /* the type of metric that will be represented in this pool distribution metric */
    metricType : null,
    /* "all" or <1hr|1day|1wk|1mon> ... use the values in the contants in StatsUtil.  This tells the collector which time period map to pull data from.  The "all" is only for high/low charts and will create a current datapoint per time range in a single data series  */
    timeRange : null,
    /*  this is the artifact type to use as the scope .. (ie datacache, grid, map, zone, partition, etc).  The types here must align with what is found in StatsDescriptorUtil */
    breakdownType : null,
    /*  this is the artifact type being charted .. (ie datacache, grid, map, zone, partition, etc).  The types here must align with what is found in StatsDescriptorUtil */
    seriesType : null,
    /* NOT USED: the scope of the used pool to evaluate, will reflect the "used" portion (ie ALL, datacache, grid, map) */
    //poolScope : null,
    /* NOT USED: the identifier of scope of the total pool to evaluate, will reflect the "used" portion (only applicable if not "ALL", ie data cache name, etc) */
    //poolScopeId : null,
    /* NOT USED: whether to show breakdown data for the 'top' or the 'bottom' x artifacts */
    //top: null,
    /* the message key of title of the pool breakdown - default is set on rest call if not set */
    titleKey : null,
    /* the message key of subtitle of the pool breakdown - default is set on rest call if not set */
    subtitleKey : null,
    subtitleKeyParms : null,
    /* the message key of chart & grid header name describing the data - default is set on rest call if not set */
    dataTitleKey : null,
    dataTitleKeyParms : null,
    /* the message key of grid header name describing the datapoints - default is set on rest call if not set */
    datapointTitleKey : null,
    datapointTitleKeyParms : null,
    /* specifies how the pool widget is sorted: by largest capacity consumers, or by largest percent of limited capacity used */
    sort : null,
    /* specifies which metric is displayed in a summed form at the top of the widget. */
    poolMetricName : null,
    /* specifies whether or not to replace chart title with drop down sort select.  */
    enableComboBox : null,
	
	_widgets : null,
    
    /* set by the script */
    data : null,
    responseData : null,
    
     constructor: function(kwArgs) {
        console.log("PoolDistribution dashboard constructor called....");
          if(!kwArgs) { 
               kwArgs = {};
          }
          /* initialize global vars for the widget */
          this.metricType = kwArgs.metricType ? kwArgs.metricType : [ "usedBytes" ];
          //this.poolScope = kwArgs.poolScope ? kwArgs.poolScope : "all";
          //this.poolScopeId = kwArgs.poolScopeId ? kwArgs.poolScopeId : "none";
          this.breakdownType = kwArgs.breakdownType ? kwArgs.breakdownType : "datacache";
          this.seriesType = kwArgs.seriesType ? kwArgs.seriesType : this.breakdownType;
          //this.top = kwArgs.top ? kwArgs.top : "true";
          this.numContribs = kwArgs.numContribs ? kwArgs.numContribs : "25";
          this.subtitleKey = kwArgs.subtitleKey ? kwArgs.subtitleKey : "XSA30201";
          this.subtitleKeyParms = kwArgs.subtitleKeyParms ? kwArgs.subtitleKeyParms : [{parmName: "${_UNITS_}", parmValue: "units"}];
          //this.dataTitleKey = kwArgs.dataTitleKey ? kwArgs.dataTitleKey : "XSA30205";
          //this.dataTitleKeyParms = kwArgs.dataTitleKeyParms ? kwArgs.dataTitleKeyParms : [{parmName: "${_METRICTYPE_}", parmValue: this.metricType[0]}];
          this.dataTitleKey = kwArgs.dataTitleKey ? kwArgs.dataTitleKey : "pool_subtitle_usedBytes";
          this.datapointTitleKeyParms = kwArgs.dataTitleKeyParms ? kwArgs.dataTitleKeyParms : null;
          this.datapointTitleKey = kwArgs.datapointTitleKey ? kwArgs.datapointTitleKey : "XSA30123";
          this.datapointTitleKeyParms = kwArgs.datapointTitleKeyParms ? kwArgs.datapointTitleKeyParms : null;
          
          this.sort = kwArgs.sort ? kwArgs.sort : "usedBytes"; //default: sort by capacity consumed
          this.poolMetricName = kwArgs.poolMetricName ? kwArgs.poolMetricName : this.metricType[0]; //default: use first metric specified.
          this.enableComboBox = kwArgs.enableComboBox ? kwArgs.enableComboBox : "false"; //default: display title and not combo box.
          
          this.ariaLevel = kwArgs.ariaLevel ? kwArgs.ariaLevel: '1';
          this.subAriaLevel = ((+this.ariaLevel) + 1).toString();

		  this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
		  /* set up the messages */
		  if ( this.xsa_messages ) {
			   console.log("found messages ", this.xsa_messages);
	           this.chartTabTitle = this.xsa_messages.XSA30120;
	           this.tableTabTitle = this.xsa_messages.XSA30121;
		  }
		  
		  this._widgets = [];

    },
     
	postMixinProperties: function(){
        try{
		   this.inherited(arguments);
		   console.log("postMixinProperties enter for: " + this.id);
		}catch(err){
		    console.debug("error",[err]);
		}
        console.log("postMixinProperties exit ...");
	},
	
	startup: function(){
        console.log("PoolDistribution startup enter for id: " + this.id);
		if(!this._started){
			this.inherited(arguments);
			this._started = true;
			if ( this.splashScreen ) {
				 console.log("hide splashScreen");
				 Bedrock.hide(this.splashScreen);
			}
			
			this.loadData();
			this.selectSortLabel.innerHTML = this.xsa_messages.XSA30210;
			dojo.attr(this.sortTable, "role", "presentation");
			console.debug(this.enableComboBox);
			if (this.enableComboBox == "true") {
				this.createSortComboBox();
			} else {
				this.combo = new dijit.form.ComboBox({
					//id: "sortComboBox",
					style: "visibility: hidden"}, 
					this.selectSort);
				dojo.style(this.selectSortLabel, 'display', 'none');
        	}
			this.selectSortLabel.setAttribute('for', this.combo.id);
			
			this.tabContainer.resize();
			//
			//  listen for onClicks, so we can update aria-pressed attribute
			//
			var children = this.tabContainer.tablist.getChildren();
			for( var i = 0; i < children.length; ++i )
			{
			    dojo.connect(children[i],"onClick",dojo.hitch(this,this._removeWaiAttributes));
			}	
			this._removeWaiAttributes();
		}	
        //Bug 19537 Hack: not sure why root template
        //title is being set to [object divElement]
		this.poolRootDiv.title = "";
        console.log("PoolDistribution startup exit");
	},

    //
    //	this function removed invalid WAI-ARIA state.
    //  aria-pressed is invalid for a tab
    //
    _removeWaiAttributes: function(){
	    var children = this.tabContainer.tablist.getChildren();
	    dojo.forEach(children,function(child){
	        dijit.removeWaiState(child.focusNode || child.domNode,"pressed");
	    });
    },
	 
	
    /** 
     * Builds a chart descriptor based on the array of metric types passed in
     * at widget constructor time.
     */
    buildChartDescriptor: function() {
       console.log("buildChartDescriptor this.metricType", this.metricType);
       var chartDescriptor = this.poolMetricName+":current:"+this.timeRange+":"+this.seriesType+":all:";
       for (var i=0; i<this.metricType.length; i++) {
          chartDescriptor = chartDescriptor+";"+this.metricType[i]+":current:"+this.timeRange+":"+this.seriesType+":high:";
       }

       console.log("chartDescriptor", chartDescriptor);
       return chartDescriptor;
    },
	
	loadData: function() {
	        var deferred = dojo.xhrGet({
	           url: "/resources/chartdata/"+this.breakdownType,
	           handleAs: "json",
	           preventCache: true,
	           content: this.getRequestParameters(),
			   headers: {
	              'Accept': 'application/json'
	           },
	           load: dojo.hitch(this, this.buildChart),
	           error: function(error) {
	                /* TODO: create a row that will span all cells with the error message */
	               console.log("An unexpected error occurred: " + error);
	           }
	        });
	
	},
	
	getRequestParameters: function() {
		var content ={
	        scopeId: this.scopeId,
            chartType: "pool",
            axisLabel: "name",
            maxContributors: this.numContribs,
            chartDescriptor: this.buildChartDescriptor()
	    }
		
		if (this.sort != null) {
			content.sort = this.sort;
		}
		
		console.debug("content", [content]);
		return content;
	},
	
	/** 
     * Creates the drop down box with sort options
     */
    createSortComboBox: function() {
        
        var ddOptions = {
        	"items": [
            	{
            		//"width": "auto",
            		"field": "usedBytes",
            		//"name": this.translate(this.dataTitleKey, this.dataTitleKeyParms,this.responseData)
            		"name" : this.xsa_messages.pool_primary_usedBytes
            	},
            	{
            		//"width": "auto",
            		"field": "gridCap",
            		"name": this.xsa_messages.gridCap
            	}
            ],
            "label": "name"
        };
        var stateStore = new dojo.data.ItemFileReadStore({ 
			data: ddOptions
        });
		
		this._widgets.push(stateStore);

        this.combo = new dijit.form.ComboBox({
            //id: "sortComboBox",
            name: "state",
            //value: this.translate(this.dataTitleKey, this.dataTitleKeyParms,this.responseData),
            value : this.xsa_messages.pool_primary_usedBytes,
            store: stateStore,
            searchAttr: "name",
            onChange: dojo.hitch(this, this.updateSort, ddOptions),
            style: "width: 360px"
        },

        this.selectSort);
        dojo.connect(this.combo,"openDropDown",this.onOpenDropDown);
        console.log("stateStore",[stateStore]);
    },
    onOpenDropDown: function(){
    	dijit.setWaiRole(this.dropDown.domNode,"listbox");
    },
    
    updateSort: function(items, value) {
    	console.debug("updateSortBy:Enter:",[items, value]);
    	var i=0;
    	for (var i=0; i<items.items.length; i++) {
    		if (items.items[i].name[0] == value) {
    			this.sort = items.items[i].field[0];
    			break;
    		}
    	}
    	this.loadData();
    },
	
	getToolTipText: function(field){
		console.debug("field",[field]);
		console.debug("responseData: ", [this.responseData]);
		
		var type = field.run.name;
		console.debug("type: ", [type]);
		
		var toolTip = "";
		
		var nameDesc = this.xsa_messages[this.seriesType+"_s"]+" ";
		var name = this.responseData.range[parseInt(field.y,10)-1];
		
		if (this.enableComboBox == "true"){
		
			if (type == (this.xsa_messages["usedBytes"]+" ("+this.xsa_messages[this.data.units+"_abv"]+")")) {	
				//Used Primary       
				var percentDesc = this.xsa_messages.XSA30207;
				var totalUsed = this.responseData.totalUsed;
				if (totalUsed == 0) totalUsed = 1;
				var percent = parseFloat((field.x/totalUsed)*100,10);
				percent = percent.toFixed(2);
				
				var valueDesc = this.xsa_messages.XSA30122+" ";
				var value = field.run.data[field.index];
					    
				toolTip = "<b>"+nameDesc+name+"</b><BR>"
					+percentDesc+percent+"%<BR>"
					+valueDesc+value+" "+this.xsa_messages[this.responseData.units+"_abv"];
			} else if (type == (this.xsa_messages["primaryUsedBytes"]+" ("+this.xsa_messages[this.data.units+"_abv"]+")")) {	
				//Used Primary
				var valueDesc = this.xsa_messages["primaryUsedBytes_c"]+" ";
				var value = field.plot.series[0].data[field.index];
				
				var totalCapacityDesc = this.xsa_messages["totalUsedBytes_c"]+" ";
				var totalValue = field.plot.series[1].data[field.index] + value;

				toolTip = "<b>"+nameDesc+name+"</b><BR><hr>"
					+"<b>"+valueDesc+value+" "+this.xsa_messages[this.responseData.units+"_abv"]+"</b><BR>"
					+totalCapacityDesc+totalValue+" "+this.xsa_messages[this.responseData.units+"_abv"];
			} else if (type == (this.xsa_messages["replicaUsedBytes"]+" ("+this.xsa_messages[this.data.units+"_abv"]+")")) {	
				//Used Replica
				var valueDesc = this.xsa_messages["replicaUsedBytes_c"]+" ";
				var value = field.plot.series[1].data[field.index];
				
				var totalCapacityDesc = this.xsa_messages["totalUsedBytes_c"]+" ";
				var totalValue = field.plot.series[0].data[field.index] + value;
				
				var replicaDesc = this.xsa_messages.pool_replicas+" ";
				// Adding .03 is a hack to account for rounding issues effect the heuristic.
				var replicas = parseInt((totalValue/field.plot.series[0].data[field.index]) + 0.03) - 1;
						    
				toolTip = "<b>"+nameDesc+name+"</b><BR><hr>"
					+"<b>"+valueDesc+value+" "+this.xsa_messages[this.responseData.units+"_abv"]+"</b><BR>"
					+replicaDesc+replicas+"<BR>"
					+totalCapacityDesc+totalValue+" "+this.xsa_messages[this.responseData.units+"_abv"];
			 } else if (type == (this.xsa_messages["pool_unused_gridCap"]+" ("+this.xsa_messages[this.data.units+"_abv"]+")")) {
				//gridcap
			 
				//Used bytes
				 console.debug("item0", [field.plot.series[0].data[field.index]]);
				 console.debug("item1", [field.plot.series[1].data[field.index]]);
				var usedBytes = field.plot.series[0].data[field.index] + field.plot.series[1].data[field.index];

				//unused limited capacity
				var unusedDesc = this.xsa_messages.pool_unused_gridCap+" ";
				var unusedCap = field.plot.series[2].data[field.index];
				console.debug("unusedCap", [unusedCap]);
				//capacity limit
				var limitDesc = this.xsa_messages.limit+" ";
				var capacityLimit = usedBytes + unusedCap;
				console.debug("capacityLimit", [capacityLimit]);
				
				//percent of limited capacity used
				var percentDesc = this.xsa_messages.gridCap+" ";
				var percent = parseFloat((usedBytes/capacityLimit)*100,10);
				percent = percent.toFixed(2);

				toolTip = "<b>"+nameDesc+name+"</b><BR><hr>"
					+"<b>"+unusedDesc+unusedCap+" "+this.xsa_messages[this.responseData.units+"_abv"]+"</b><BR>"
					+limitDesc+capacityLimit+" "+this.xsa_messages[this.responseData.units+"_abv"]+"<BR>"
					+percentDesc+percent+"%";
			}
		} else {
			var percentDesc = this.xsa_messages.XSA30207;
			var totalUsed = this.responseData.totalUsed;
			if (totalUsed == 0) totalUsed = 1;
			var percent = parseInt((field.x/totalUsed)*100,10);
				    
			var valueDesc = this.xsa_messages.XSA30122+" ";
			var value = field.x;
				    
			toolTip = "<b>"+nameDesc+name+"</b><BR><hr>"
				+percentDesc+percent+"%<BR>"
				+valueDesc+value+" "+this.xsa_messages[this.responseData.units+"_abv"];
		}
		
		 
		
		console.debug("tooltip",[toolTip]);
		return toolTip;
	},
	
	buildChart: function(request, ioArgs) {
		console.debug("Entering buildChart, request: ",[request]);
	
		//create the chart		
		try{
			this._dp = new xsa.monitor.widgets.ext.DataPresentation(this.chartDiv, {
					type: "chart",
					chartType: "StackedBars",
					data: request,
					series: dojo.hitch(this,this.makeseries),
					labelMod: 1,
					gridNode:this.griddiv,
					titleNode: this.title,
					footerNode: this.footer,
					animate: true,
					theme: "dojox.charting.themes.Distinctive"
			});   
			
			console.debug("chart",[this._dp]);
			console.debug("request", request);   	
        	
        	if (this.enableComboBox == "true") {
        		//On DG Overview pool distribution chart, if grid is using >= 90% of capacity limit, turn that bar red (as a visual warning)
	        	var percent, usedBytes, capacityLimit = 0;
	        	
	        	if (this._dp.chartWidget.series[2]) {
		        	for (var i_grids=0; i_grids< this._dp.chartWidget.series[2].data.length; i_grids++) {
		        		//ignore the grids with no capacity limit set
		        		if (this._dp.chartWidget.series[2].data[i_grids] != -1) { 
		        			//series[0]-primary, series[1]-replica, series[2]-unusedcap
		        			usedBytes = this._dp.chartWidget.series[0].data[i_grids] + this._dp.chartWidget.series[1].data[i_grids];
		        			capacityLimit = usedBytes + this._dp.chartWidget.series[2].data[i_grids];
							percent = parseFloat((usedBytes/capacityLimit)*100,10);
							
							console.debug("percent: "+percent);
							
		        			if (percent >= 90) {
		        				//turn all the bars shades of red
		        				if (this._dp.chartWidget.series[0].group.children[i_grids]) {
		        					this._dp.chartWidget.series[0].group.children[i_grids].setFill("#ff0000");
		        					this._dp.chartWidget.series[0].group.children[i_grids].setStroke("#ff0000");
		        				}
		        				if (this._dp.chartWidget.series[1].group.children[i_grids]) {
		        					this._dp.chartWidget.series[1].group.children[i_grids].setFill("#cc0000");
		        					this._dp.chartWidget.series[1].group.children[i_grids].setStroke("#cc0000");
		        				}
		        				if (this._dp.chartWidget.series[2].group.children[i_grids]) {
		        					this._dp.chartWidget.series[2].group.children[i_grids].setFill("#990000");
		        					this._dp.chartWidget.series[2].group.children[i_grids].setStroke("#990000");
		        				}
		        			}
		        		}
		        	}
	        	}
	        }

            var anim = new dojox.charting.action2d.Tooltip(this._dp.chartWidget, "primary-StackedBars",{text: dojo.hitch(this, this.getToolTipText)});
            this._widgets.push(anim);
			
            //Bug 20522 - truncated y-axis labels, need to be untruncated for grid.
            for (var i_names=0; i_names<request.series.length; i_names++) {
                this._dp.gridWidget._by_idty[i_names].item["data.0"][0] = request.range[i_names];
            }
            
            // Hardcoded.  If unused limited capacity has a value of -1, then set it to empty string.
            for (var i_names=0; i_names<request.series.length; i_names++) {
            	if (this._dp.gridWidget._by_idty[i_names].item["data.3"]) {
                if (this._dp.gridWidget._by_idty[i_names].item["data.3"][0] == -1)
                	this._dp.gridWidget._by_idty[i_names].item["data.3"][0] = this.xsa_messages.unlimited;
            	}
            }
            
		 }catch(err){
			  console.debug("error",[err]);
		 }
	},
	
	translate: function(key, parms, data){
	   //console.debug("translate enter",[key,parms,data]);
	   var message = this.xsa_messages[key];
	   if (parms){
	       var parm;
	       for (parm in parms){
                if (this.xsa_messages[parms[parm].parmValue]){
                    //look for the message key sub parm to be another message key
                    var name = parms[parm].parmName;
                    var value = this.xsa_messages[parms[parm].parmValue];
                    message = message.replace(name,value);
                }else{
                    //check if it is set in the data
                    if (this.responseData[parms[parm].parmValue]){
						var name2 = parms[parm].parmName;
                        var value2 = this.xsa_messages[this.responseData[parms[parm].parmValue]];
                        message = message.replace(name2,value2);
                    }
              }
	       }
	   }
	   //console.debug("translate exit",[message]);
	   return message;
	},
	
	makeseries: function(data) {
	   console.debug("makeseries",[data]);
	   this.responseData = data;

	   //this allows for some manipulation of the data before it really gets processed/used by the DP widget.
	   //override the title and footer
	   var title;
	   console.debug("title1",["pool_"+this.seriesType+"_"+this.poolMetricName]);
	   if (this.titleKey == null) {
           title = this.xsa_messages["pool_"+this.seriesType+"_"+this.poolMetricName];
	   } else {
		   title = this.xsa_messages[this.titleKey];
	   }
	   console.debug("title",[title]);
	   data.title=title;
	   
	   var footer = this.translate(this.subtitleKey, this.subtitleKeyParms, data);
	   console.debug("footer",[footer]);
	   data.footer=footer;
	   
	   var dataTitle = this.translate(this.dataTitleKey, this.dataTitleKeyParms,data);
	   console.debug("dataTitle",[dataTitle]);
	   
	   this.poolBreakdown_header.id = "pool-row-label-" + this.seriesType + "-" + this.poolMetricName;
	   if (this.enableComboBox == "true")
	   		this.poolBreakdown_header.innerHTML = "";
	   else
	   		this.poolBreakdown_header.innerHTML=dataTitle;

	   
	   //set up the pool HTML data table
	   var totalWidth = this.poolDistribution_poolTable.clientWidth;
	   
	   this.poolDistribution_poolHeader.innerHTML = this.xsa_messages.XSA30208;
	   this.poolDistribution_ruler.innerHTML = this.poolDistribution_poolHeader.innerHTML;
	   var headerWidth = this.poolDistribution_ruler.offsetWidth + 15; //pad by 15 px
	   this.poolDistribution_poolHeader_cell.style.width = headerWidth+"px";
	   dojo.style(this.poolDistribution_poolHeader_cell2, "width",  headerWidth+"px");
	   
	   this.poolDistribution_ruler.id = "ruler-" + this.seriesType + "-" + this.poolMetricName;
	   //this.poolDistribution_units.innerHTML = this.xsa_messages[data.units+"_abv"];
	   //this.poolDistribution_ruler.innerHTML = this.poolDistribution_units.innerHTML;
	   var unitsWidth = 0;//this.poolDistribution_ruler.offsetWidth + 15; //pad by 15 px
	   //this.poolDistribution_units_cell.style.width=unitsWidth+"px";
	   this.poolDistribution_ruler.innerHTML="";

	   var seriesValue = 0;
	   if (data.series.length > 0) {
	      seriesValue = data.series[0].values[0];
	   }
       data.totalUsed = seriesValue;
	   
	   if (data.totalpool == undefined) {
	      data.totalpool = seriesValue;
	   }

	   var chartWidth = totalWidth-headerWidth-unitsWidth;	   
	   this.poolDistribution_used.innerHTML = "&nbsp;";
	   //this.poolDistribution_used_cell.title = "";
	   
	   this._widgets.push(
		   new dijit.Tooltip({
			   connectId: [this.poolDistribution_used_cell],
			   label: "<b>"+this.xsa_messages.XSA30203+seriesValue+" "+this.xsa_messages[data.units+"_abv"]+"</b>",
			   position: "above"
		   }
	   ));
	   
	   var usedWidth;
	   if (data.totalpool == 0) {
	      usedWidth = parseInt(chartWidth*(seriesValue/1),10);
	   } else {
	      usedWidth = parseInt(chartWidth*(seriesValue/data.totalpool),10);
	   }

       dojo.style(this.poolDistribution_used_cell, "width",  usedWidth+"px");
       
       var freeValue = data.totalpool - seriesValue; 
       if (freeValue < 0) {
	       freeValue = 0;
       }

       this.poolDistribution_free.innerHTML = "&nbsp;";
	   this._widgets.push(
		   new dijit.Tooltip({
			   connectId: [this.poolDistribution_free_cell],
			   label: "<b>"+this.xsa_messages.XSA30204+freeValue+" "+this.xsa_messages[data.units+"_abv"]+"</b>",
			   position: "above"
		   })
	   )

	   if ((chartWidth-usedWidth) <0) {
           dojo.style(this.poolDistribution_free_cell, "width",  0+"px");
	   } else {
           dojo.style(this.poolDistribution_free_cell, "width",  (chartWidth-usedWidth)+"px");
	   }
	   //now set the color class
	   
	   var totalPoolWidth = parseInt(this.poolDistribution_poolHeader_cell.offsetWidth)+parseInt(this.poolDistribution_used_cell.offsetWidth)+parseInt(this.poolDistribution_free_cell.offsetWidth);
	   totalPoolWidth = totalPoolWidth - this.poolDistribution_poolHeader_cell2.offsetWidth - this.poolDistribution_used_cell2.offsetWidth;
	   dojo.style(this.poolDistribution_free_cell2, "width",  totalPoolWidth+"px");

	   var percentFree;
	   if (data.totalpool == 0) {
	      percentFree = parseInt(((data.totalpool-seriesValue)/1)*100,10);
	   } else {
	      percentFree = parseInt(((data.totalpool-seriesValue)/data.totalpool)*100,10);
	   }
	   
	   if (percentFree<=15){
          this.poolDistribution_free_cell.className = "pool-free-red";
	   }else if (percentFree<=30){
          this.poolDistribution_free_cell.className = "pool-free-yellow";
	   }else{
          this.poolDistribution_free_cell.className = "pool-free-green";
	   }
	   console.debug("pool free cell",[this.poolDistribution_free_cell]);
	   this.poolDistribution_min.innerHTML = "0";
	   this.poolDistribution_max.innerHTML = data.totalpool +" "+ this.xsa_messages[data.units+"_abv"];
	   var name = this.translate(this.datapointTitleKey, this.datapointTitleKeyParms, data);
	   console.debug("name",[name]);
	   
       this.data = data;
       var metricLabels = [];
       for (var i_metricNames=0; i_metricNames<this.metricType.length; i_metricNames++) {
    	   var tempLabel = this.xsa_messages[this.metricType[i_metricNames]];
    	   if (tempLabel == undefined) {
    		   tempLabel = this.xsa_messages["pool_"+this.metricType[i_metricNames]];
    	   }

    	   metricLabels.push(tempLabel+" ("+this.xsa_messages[this.data.units+"_abv"]+")");
       }
       //if (this.data.series.length > 0) {
       //   this.data.series[0].legend = this.xsa_messages.XSA30209+" "+metricLabel;
       //}
       
       dojo.style(this.chartTab, "height",  (this.data.series.length*30)+"px");

       // Remove "all" data set from range.
       if (this.data.range.length > 0) {
           this.data.range.shift();
       }
       
       // Remove "all" data set from data series.
       if (this.data.series.length > 0) {
           this.data.series.shift();
       }

       // Truncate long labels       
       for (var i_name=0; i_name<data.series.length; i_name++) {
         if (data.series[i_name].name.length > 11) {
            data.series[i_name].name = data.series[i_name].name.substring(0,12)+"...";
         }
       }
		
       console.debug("Hardcode removal of 3 metric if all values are -1");
       var removeData = "true";
       for (var ii=0; ii<this.data.series.length; ii++) {
    	   if (this.data.series[ii].values[2] != -1) {
    		   removeData = "false";
    	   }
       }
       
       console.debug("removeData:", [removeData]);
       
       if (removeData == "true") {
	       for (var iii=0; iii<this.data.series.length; iii++) {
	    		   this.data.series[iii].values.pop();
	       }
       }
       
	   console.debug("Pool data:", [this.data]);

	   var txt = []; 
	 		
	 	txt.push({ datapoints: "series", field: "name", name: name, type: "range", chart: false });
		if (this.data.series.length > 0) {
			for (var i_series=0; i_series<this.data.series[0].values.length; i_series++) {
				console.debug("metricLabels[i_series]", [metricLabels[i_series]]);
				txt.push({ datapoints: "series", field: "values."+i_series, name: metricLabels[i_series] });
			}
		} 
       
	   console.debug("txt", txt);
       return txt;
    },

	destroy: function(){
		// summary:
		//		Cleanup leftover widgets that we programmatically created.
		if(this._dp){
			this._dp.destroy();
			this._dp = null;
		}
		if(this.combo) {
			this.combo.destroyRecursive();
			this.combo = null;
		}
		if(this.tabContainer){
			this.tabContainer.destroyRecursive();
			this.tabContainer = null;
		}
		dojo.forEach(this._widgets, function(widget){
			if(widget.destroyRecursive) {
				widget.destroyRecursive();
			} else if(widget.destroy){
				widget.destroy();
			}
		});
		this.inherited(arguments);

	}
    
});
