/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.overview.SparklineCollection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.charting.widget.Chart2D");
dojo.require("dojox.charting.widget.Sparkline");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("dojox.charting.action2d.Magnify");
dojo.require("dojox.charting.action2d.Tooltip");
dojo.require("dojox.charting.action2d.Highlight");

dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.overview.SparklineCollection", [dijit._Widget, dijit._Templated], {   

    templatePath: dojo.moduleUrl("xsa","monitor/widgets/overview/templates/SparklineCollection.html"),
     widgetsInTemplate: true,

     /* holds a reference to localized message bundle */
    xsa_messages : null,
    
    /* THESE ATTRIBUTES MUST BE SET WHEN INSTANTIATING THE WIDGET */
    /* the type of metric that will be represented in this sparkline metric */
    metricType : null,
    /* the type of data artifact to aggregate the stats to (ie datacache, grid, map) */
    type : null,
    /* whether to show data for the 'top' or the 'bottom' x artifacts */
    top: null,
    /* the number of contributors to the sparkline table */
    numContribs : null,
    
    
    /* THESE ATTRIBUTES ARE SET FROM BACKEND QUERY OF THE DATA - see loadSparklineData*/
    timeInterval : 0,
    timeUnits : "",
    sparkItems : null,
    
    /* CONSTANTS */
    CIRCLE: "m-3,0 c0,-4 6,-4 6,0 m-6,0 c0,4 6,4 6,0",
    GREEN: "rgb(127,255,0)",
    PURPLE: "rgb(160,32,240)",
    ORANGE: "rgb(255,140,0)",
    BLACK: "rgb(0,0,0)",
	
	_domNodesToClean : null,
	_widgets : null,
     
     constructor: function(kwArgs) {
        console.log("SparklineCollection dashboard constructor called....");
          if(!kwArgs) { 
               kwArgs = {};
          }
          /* initialize global vars for the widget */
          this.metricType = kwArgs.metricType ? kwArgs.metricType : "avgTP";
          this.type = kwArgs.type ? kwArgs.type : "datacache";
          this.top = kwArgs.top ? kwArgs.top : "true";
          this.numContribs = kwArgs.numContribs ? kwArgs.numContribs : "5";
          
          this.ariaLevel = kwArgs.ariaLevel ? kwArgs.ariaLevel: '1';
        
        /* set up the messages */
        this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
        if ( this.xsa_messages ) {
             console.log("found messages ", this.xsa_messages);
        }
                
		this._domNodesToClean = [];
		this._widgets = [];
    },
     
    postCreate: function() {
        console.log("SparklineCollection dashboard postCreate called....");
        if ( this.splashScreen ) {
             console.log("hide splashScreen");
             Bedrock.hide(this.splashScreen);
        }

        var top;
        if (this.top == "true"){
        	top = "top";
        } else {
        	top = "bottom";
        }
        
        //console.log("sparkline title", ["sparkline_"+top+"_"+this.type]);
    	//var caption_p = this.xsa_messages["sparkline_"+top+"_"+this.type];
    	
    	console.log("sparkline title: ", ["sparkline_"+this.type+"_"+this.metricType]);
    	var caption_p = this.xsa_messages["sparkline_"+this.type+"_"+this.metricType];
    	
        var caption_s = this.xsa_messages.XSA30000B;
        caption_p = dojo.string.substitute(caption_p,{_NUMCONTRIB_:this.numContribs});
        caption_s = dojo.string.substitute(caption_s,{_METRICTYPE_:this.xsa_messages[this.metricType],_UNITS_:this.xsa_messages["u_"+this.metricType]});

        if ( this.xsa_messages ) {
             console.log("caption_p ", caption_p);
             console.log("caption_s ", caption_s);
        }
        this.sparklineCollectionCaption_primary.innerHTML = caption_p;
        this.sparklineCollectionCaption_secondary.innerHTML = caption_s;
        
        /* load up the sparkline data */
        this.loadSparklineData();
        
     },

     buildSparklineTable: function(response, ioArgs){
        console.log("SparklineCollection dashboard buildSparklineTable called: "+response);
     
        /* high level parse of the data */
        this.timeInterval = response.interval;
        this.timeUnits = response.timeUnits;
        this.sparkItems = response.sparkItems;
        if ( this.xsa_messages ) {
           console.log("this.sparkItems", this.sparkItems);
        }
        
        /* set up the sparkline table header */
        this.headerName.innerHTML = this.xsa_messages[this.type];
        this.headerCurrent.innerHTML = this.xsa_messages.XSA30002;
        this.headerMinMaxAvg.innerHTML = this.xsa_messages.XSA30003;
        /*      XSA30001 : "Data over last ${_TIMEINTERVAL_} ${_TIMEUNITS_}" */
        var chartHeader = this.xsa_messages.XSA30001;
        chartHeader = dojo.string.substitute(chartHeader,{_TIMEINTERVAL_:this.timeInterval,_TIMEUNITS_:this.xsa_messages[this.timeUnits]});
        if ( this.xsa_messages ) {
             console.log("chartHeader ", chartHeader);
        }
        this.headerChart.innerHTML = chartHeader;
        
        /* add in content rows */
        if (this.sparkItems){
           for (var i=0; i<this.sparkItems.length; i++){
              console.debug('sparkItem', [i, this.sparkItems[i]]);
              var sparkItem = this.sparkItems[i];
              var tr = dojo.doc.createElement("tr");
                       
              var td_name = dojo.doc.createElement("td");
              var gridName = sparkItem.name
              //Hack for bug 19977.  If grid name is too long add ... and title text.
              if (gridName.length > 11) {
                 gridName = gridName.substring(0,11)+"...";
                 td_name.title = sparkItem.name;
              }
              td_name.innerHTML = gridName;
              td_name.className = "sparkline-collection-cell";
              tr.appendChild(td_name);
                       
              var td_current = dojo.doc.createElement("td");
              td_current.innerHTML = "<span class=\"sparkline-current\">"+sparkItem.current+"</span>";
              td_current.className = "sparkline-collection-cell";
              tr.appendChild(td_current);
                       
              var td_sparkChart = dojo.doc.createElement("td");
              /* create spark chart */
              var chartDiv = dojo.doc.createElement("div");
              chartDiv.className="sparkline-chart";
              td_sparkChart.appendChild(chartDiv);

              tr.appendChild(td_sparkChart); 
                       
              var td_minmaxavg = dojo.doc.createElement("td");
              td_minmaxavg.innerHTML = "[<span class=\"sparkline-min\">"+sparkItem.min+"</span>|<span class=\"sparkline-max\">"+sparkItem.max+"</span>|"+sparkItem.avg+"]";
              td_minmaxavg.className = "sparkline-collection-cell";
              tr.appendChild(td_minmaxavg);
                       
              this.sparklineCollectionTable.appendChild(tr);
              console.debug('create sparkline chart',[sparkItem]);
              var sparkChart = this.createSparklineChart(chartDiv,sparkItem);
              console.debug('render chart',[sparkChart]);
              try{
                sparkChart.chart.render();
              }catch(err){
                console.debug("Error",[err]);
              }
			  
			  this._domNodesToClean.push(tr);
			  this._domNodesToClean.push(td_name);
			  this._domNodesToClean.push(td_current);
			  this._domNodesToClean.push(td_sparkChart);
			  this._domNodesToClean.push(chartDiv);
			  this._domNodesToClean.push(td_minmaxavg);

           }
        
        }
        
         console.log("hide splash screen, show sparklines");
         Bedrock.fadeCommon(this.splashScreen);
   /*      if ( this.sparklineView ) {
             Bedrock.show(this.sparklineView);
        }   
    */  
         console.log('all done');
     },

     loadSparklineData: function(){
        console.log("SparklineCollection dashboard loadSparklineData called....");
        var deferred = dojo.xhrGet({
           url: "/resources/sparkline/"+this.type,
           handleAs: "json",
           preventCache: true,
           content: {
               metricType: this.metricType,
               top: this.top,
               numContributors: this.numContribs
           },
		   headers: {
              'Accept': 'application/json'
           },
           load: dojo.hitch(this, this.buildSparklineTable),
           error: function(error) {
                /* TODO: create a row that will span all cells with the error message */
               console.log("An unexpected error occurred: " + error);
           }
        });
        console.log("done, waiting for callback....");

        if (dojo.style(this.splashScreen, 'display') == 'none')
        {
            console.log("show splash screen");
         /*   if (this.sparklineView) {
                Bedrock.hide(this.sparklineView);
            }   
          */ 
            Bedrock.show(this.splashScreen);
        }
     },
          
     createSparklineChart: function(parent,sparkItem){
     
        console.debug('createSparklineChart',[parent,sparkItem]);
          
        /* get the data to populate the dataseries. */
        /* TODO: may want to change the data returned from the server side to directly return data rather than fetch from data series */
        var datastore=new dojo.data.ItemFileReadStore({data:sparkItem.itemstore,clearOnClose:'true'});
        console.debug("datastore: ",[datastore]);
        var valueList = [];
        var buildList = function(items, request){
            dojo.forEach(items, function(i){
				 valueList.push({x: Number(i.timestamp[0]), y:Number(datastore.getValue(i, "value"))});
            });
            console.debug("All values are: ",[valueList]);
		};
		//Invoke the search
		datastore.fetch({
             onComplete: buildList
		});       
        
        var chartParms = {
             margins:{ l: 0, r: 5, t: 5, b: 5 }
        };
	
		var minIndex = -1;
		var maxIndex = -1;
		var curIndex = -1;
		dojo.forEach(sparkItem.itemstore.items, dojo.hitch(this, function(item){
			if(minIndex<0 &&sparkItem.min == item.value[0]){
				minIndex = Number(item.timestamp[0]);
			}
			if(sparkItem.max == item.value[0]){
				maxIndex = Number(item.timestamp[0]);
			}
			if(sparkItem.current == item.value[0]){
				curIndex = Number(item.timestamp[0]);
			}
		}));
		
        var chartWidget=new dojox.charting.widget.Sparkline( chartParms, parent);
        chartWidget.chart.addPlot("default",{type: "Lines", tension: 3, hAxis: "x", vAxis: "y"});

		chartWidget.chart.addAxis("x",{min:(Number(sparkItem.itemstore.items[0].timestamp[0])), max:(Number(sparkItem.itemstore.items[sparkItem.itemstore.items.length-1].timestamp[0])), majorLabels:false, minorTicks: false, stroke: "", minorLabels: false, microTicks: false, majorTick: {color: "white", length: 1}});
        //chartWidget.chart.addAxis("y",{min:(Number(sparkItem.min)), max:(Number(sparkItem.max)), fixLower: "none", fixUpper: "none", vertical:true, majorLabels:false, minorTicks: false, stroke: "white", minorLabels: false, microTicks: false, majorTick: {color: "white", length: 1}});
        chartWidget.chart.addSeries("defaultSeries",valueList);
        chartWidget.chart.addPlot("markers",{type: "Scatter"});
        chartWidget.chart.addSeries("markersAvg",[{x:Number(sparkItem.itemstore.items[0].timestamp[0]),y: Number(sparkItem.avg)}],{plot:"markers", stroke: {color: this.BLACK}, marker: this.CIRCLE});
        chartWidget.chart.addSeries("markersMin",[{x:minIndex,y: Number(sparkItem.min)}],{plot:"markers", stroke: {color: this.ORANGE}, marker: this.CIRCLE});
        chartWidget.chart.addSeries("markersMax",[{x:maxIndex,y: Number(sparkItem.max)}],{plot:"markers", stroke: {color: this.GREEN}, marker: this.CIRCLE});
        chartWidget.chart.addSeries("markersCur",[{x:curIndex,y: Number(sparkItem.current)}],{plot:"markers", stroke: {color: this.PURPLE}, marker: this.CIRCLE});
        
        var anim1b = new dojox.charting.action2d.Magnify(chartWidget.chart, "markers");
        var anim11b = new dojox.charting.action2d.Highlight(chartWidget.chart, "markers");
        var anim10d = new dojox.charting.action2d.Tooltip(chartWidget.chart, "markers",{text: dojo.hitch(this, this.getToolTipText)});
		
		this._widgets.push(datastore);
		this._widgets.push(chartWidget);
		this._widgets.push(anim1b);
		this._widgets.push(anim11b);
		this._widgets.push(anim10d);
        return chartWidget;
     },

     getToolTipText: function(field) {
        console.debug("field",[field]);
        var label = "XSA30122";
        var fieldColor = field.shape.strokeStyle.color;
        fieldColor ="rgb("+fieldColor.r+","+fieldColor.g+","+fieldColor.b+")";

        if (fieldColor==this.ORANGE) label = "sparkline_low0";
        else if (fieldColor==this.GREEN) label = "sparkline_high0";
        else if (fieldColor==this.PURPLE) label = "sparkline_current";
        else if (fieldColor==this.BLACK) label = "sparkline_all";

        return "<b>"+this.xsa_messages[label]+"</b> "+field.y;
     },
	 
	 destroy: function() {
		dojo.forEach(this._widgets, function(widget){
			if(widget.destroyRecursive) {
				widget.destroyRecursive();
			} else if(widget.destroy){
				widget.destroy();
			}
		});
		dojo.forEach(this._domNodesToClean, dojo.destroy);
		this.inherited(arguments);
    }


});
