/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2009. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.monitor.widgets.overview.TimeClusteredColumnChart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.widget.DataPresentation");
dojo.require("dojox.charting.Theme");
dojo.require("dojox.charting.themes.Distinctive");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.requireLocalization("xsa.monitor.widgets.overview", "xsa_messages");

dojo.declare("xsa.monitor.widgets.overview.TimeClusteredColumnChart", [dijit._Widget, dijit._Templated], {   

    templatePath: dojo.moduleUrl("xsa","monitor/widgets/overview/templates/TimeClusteredColumnChart.html"),
     widgetsInTemplate: true,

     /* holds a reference to localized message bundle */
    xsa_messages : null,
    
    /* THESE ATTRIBUTES MUST BE SET WHEN INSTANTIATING THE WIDGET */
    /* the type of metric that will be represented in this sparkline metric */
    metricType : null,
    /* the type of data artifact to aggregate the stats to (ie datacache, grid, map) */
    type : null,
    /* the time range to show */
    timeRange: null,
    /* the scope "all" or fully qualified name of artifact to scope the chart to , like "anngrid" or "anngrid!annMap" */
    scopeId : null,
    /* the number of contributors to the breakdown, the rest will be grouped into an "other" category */
    numContribs: null,
    /* the message key of title of the pool breakdown - default is set on rest call if not set */
    titleKey : null,
    titleKeyParms : null,
    /* the message key of subtitle of the pool breakdown - default is set on rest call if not set */
    subtitleKey : null,
    subtitleKeyParms : null,
    /* 
     * This is a delimited list of descriptors for every data series to be displayed on the chart.
     * Sample:
     * var dataSeries = [
     *   {metricType: '', timeRange: '', type: '', seriesId: '', scale: ''},
     *   {metricType: '', timeRange: '', type: '', seriesId: '', scale: ''}
     * ];
     *
     * var p_cannedChart = {
     *   chartDescriptors: dataSeries
     * };
     */
    chartDescriptors: null,
    
    /* set by the script */
    responseData : null,
     
     constructor: function(kwArgs) {
        console.log("TimeClusteredColumnChart dashboard constructor called....");
          if(!kwArgs) { 
               kwArgs = {};
          }
          /* initialize global vars for the widget */
          this.metricType = kwArgs.metricType ? kwArgs.metricType : "avgTP";
          this.type = kwArgs.type ? kwArgs.type : "datacache";
          this.timeRange = kwArgs.timeRange ? kwArgs.timeRange : "1hr";
          this.scopeId = kwArgs.scopeId ? kwArgs.scopeId : "all";
          this.numContribs = kwArgs.numContribs ? kwArgs.numContribs : "1";
          this.titleKey = kwArgs.titleKey ? kwArgs.titleKey : this.metricType;
          this.titleKeyParms = kwArgs.titleKeyParms ? kwArgs.titleKeyParms : [];
          this.subtitleKey = kwArgs.subtitleKey ? kwArgs.subtitleKey : "XSA30500";
          this.subtitleKeyParms = kwArgs.subtitleKeyParms ? kwArgs.subtitleKeyParms : [{parmName: "${_ARTIFACT_}", parmValue: this.type}];
          this.chartDescriptors = kwArgs.chartDescriptors ? kwArgs.chartDescriptors : [];
        
          /* set up the messages */
          this.xsa_messages = dojo.i18n.getLocalization("xsa.monitor.widgets.overview", "xsa_messages");
          if ( this.xsa_messages ) {
               console.log("found messages ", this.xsa_messages);
               this.chartTabTitle = this.xsa_messages.XSA30120;
               this.tableTabTitle = this.xsa_messages.XSA30121;
          }
                
    },
     
    postCreate: function() {
        console.log("TimeClusteredColumnChart dashboard postCreate called....");
        if ( this.splashScreen ) {
             console.log("hide splashScreen");
             Bedrock.hide(this.splashScreen);
        }
        if ( this.dashboardView ) {
             console.log("hide dashboardView");
             Bedrock.hide(this.dashboardView);
        }

        /* load up the sparkline data */
        this.loadHiLowData();
        
        //Bug 19537 Hack: not sure why root template
        //title is being set to [object divElement]
		this.timeClusteredRootDiv.title = "";
     },

     buildTimeClusteredColumnChart: function(response, ioArgs){
        console.log("TimeClusteredColumnChart dashboard buildTimeClusteredColumnChart called: "+response);
		//create the chart
		try{
			this._dp = new dojox.widget.DataPresentation(this.timeclusteredchart, {
					type: "chart",
					chartType: "ClusteredColumns",
					data: response,
					series: dojo.hitch(this, this.makeseries),
					labelMod: 1,
					gridNode:this.griddiv,
					titleNode: this.title,
					footerNode: this.footer,
					animate: true,
					//theme: "dojox.charting.themes.Distinctive",
                    theme: new dojox.charting.Theme({ fillcolors: ["blue", "green", "orange"], colors: ["lightblue", "lightgreen", "palegoldenrod"] })
			});

			console.debug("chart",[this._dp]);
            new dojox.charting.action2d.Highlight(this._dp.chartWidget, "default");

            var anim = new dojox.charting.action2d.Tooltip(this._dp.chartWidget, "primary-ClusteredColumns",{text: dojo.hitch(this, this.getToolTipText)});            
            var legend = new dojox.charting.widget.Legend({chart: this._dp.chartWidget, horizontal: false}, this.timeclusteredlegend);
		 }catch(err){
			  console.debug("error",[err]);
		 }
		
        /* var hiLowChart = this.createTimeClusteredColumnChart(response);
        console.debug('render chart',[hiLowChart]);
        hiLowChart.render(); */
        
        Bedrock.fadeCommon(this.splashScreen);
        if ( this.dashboardView ) {
            Bedrock.show(this.dashboardView);
        }
     },
	
     getToolTipText: function(field){
         console.debug("field",[field]);
         var typeLabel = this.xsa_messages[this.type+"_s"]+" ";
         var typeValue = this.responseData.series[this.getXIndex(field)].values[parseInt(field.x)].name;
         if (typeValue == "all") {
             typeValue = this.xsa_messages["legend_"+typeValue];
         }
         return "<b>"+typeLabel+typeValue+"</b><br />"+this.xsa_messages.XSA30122+" "+field.y+" "+this.xsa_messages["u_"+this.metricType];
     },
     
     getXIndex: function(field) {
         for (var i=0; i<field.plot.series.length; i++) {
		     var data = field.plot.series[i].data;
			 for (var j=0; j<data.length; j++) {
			     if (data[j] == field.y) {
			         return i;
			     }
			 }
         }
         return 0;
     },

     loadHiLowData: function(){
        console.log("TimeClusteredColumnChart dashboard loadHiLowData called....");
        var chartDescriptor = this.buildChartDescriptor();
        var deferred = dojo.xhrGet({
           url: "/resources/chartdata/"+this.type,
           handleAs: "json",
           preventCache: true,
           content: {
               scopeId: this.scopeId,
               chartType: "highlow",
               axisLabel: "range",
               maxContributors: this.numContribs,
               chartDescriptor: chartDescriptor
           },
		   headers: {
              'Accept': 'application/json'
           },
           load: dojo.hitch(this, this.buildTimeClusteredColumnChart),
           error: function(error) {
                /* TODO: create a row that will span all cells with the error message */
               console.log("An unexpected error occurred: " + error);
           }
        });
        
        console.log("done, waiting for callback....");

        if (dojo.style(this.splashScreen, 'display') == 'none')
        {
            console.log("show splash screen");
            if (this.dashboardView) {
                Bedrock.hide(this.dashboardView);
            }    
            Bedrock.show(this.splashScreen);
        }
     },

	 makeseries: function(data) {
	   console.debug("makeseries",[data]);
	   this.responseData = data;
	   
	   //this allows for some manipulation of the data before it really gets processed/used by the DP widget.
	   //override the title and footer
	   var title = this.translate(this.titleKey, this.titleKeyParms, data);
	   console.debug("title",[title]);
	   data.title=title;
	   
	   var footer = this.translate(this.subtitleKey, this.subtitleKeyParms, data);
	   console.debug("footer",[footer]);
	   data.footer=footer;

	   var name = this.xsa_messages.XSA30123;
	   console.debug("name",[name]);
	   
	   //Set hilow legend labels
       for (var i=0; i<data.series.length; i++) {
           var legend = this.translate("legend_"+data.series[i].legend, [], data);
           data.series[i].legend = legend;
       }
       
       console.debug("data",[data]);
       //Remove last week and last month stats
       for (var i=0; i<data.series.length; i++) {
           data.series[i].scaledValues.shift();
           data.series[i].scaledValues.shift();
           data.series[i].range.shift();
           data.series[i].range.shift();
           data.series[i].values.shift();
           data.series[i].values.shift();
       }
       data.range.shift();
       data.range.shift();
       
       //Inject current stats
       for (var i=0; i<data.series.length; i++) {
           data.series[i].scaledValues.unshift(data.series[i].current);
           data.series[i].range.unshift("current");
           data.series[i].values.unshift({ fullname: "", name: "", value: data.series[i].current });
       }
       data.range.unshift("current");

	   //Set hilow time range labels
       for (var i=0; i<data.range.length; i++) {
           var range = this.translate("range_"+data.range[i], [], data);
           data.range[i] = range;
       }
	   
	   console.debug("data",[data]);
	   
	   var txt = [];
	   txt.push({ datapoints: "range", name: name, type: "range", chart: false })
       for (var i=0; i<data.series.length; i++) {
             txt.push({
                 datapoints: "series["+i+"].scaledValues",
                 namefield: "series["+i+"].legend"
             });
       }

	   console.debug("txt", txt);
       return txt;
	 },
	 
     /** 
      * Builds a chart descriptor based on the array of chart descriptors passed in
      * at widget constructor time.
      *
      * HACK: Currently the seriesId is being replaced by "all" because stats infastructure
      * is not working.  Also "all" is not really retrieving all which will cause some more issues.
      */
     buildChartDescriptor: function() {
        console.log("this.chartDescriptors", this.chartDescriptors);
        var chartDescriptor = "";

		//<metricType>:<numDataPoints>:<timeRange>:<seriesType>:<seriesID>:<scaler>
        if (this.chartDescriptors != null) {
           for (var i=0; i<this.chartDescriptors.length; i++) {
              if (chartDescriptor == "") {
                 chartDescriptor = this.chartDescriptors[i].metricType+":current:"+"all"+":"+this.chartDescriptors[i].type+":"+this.chartDescriptors[i].seriesId+":"+this.chartDescriptors[i].scale;
              } else {
                 chartDescriptor = chartDescriptor+";"+this.chartDescriptors[i].metricType+":current:"+"all"+":"+this.chartDescriptors[i].type+":"+this.chartDescriptors[i].seriesId+":"+this.chartDescriptors[i].scale;
              }
              /* if (chartDescriptor == "") {
                 chartDescriptor = this.chartDescriptors[i].metricType+":all:"+"all"+":"+this.chartDescriptors[i].type+":"+"all"+":"+this.chartDescriptors[i].scale;
              } else {
                 chartDescriptor = chartDescriptor+";"+this.chartDescriptors[i].metricType+":all:"+"all"+":"+this.chartDescriptors[i].type+":"+"all"+":"+this.chartDescriptors[i].scale;
              }*/
           }
        }
        console.log("chartDescriptor", chartDescriptor);
        return chartDescriptor;
     },
	 
	translate: function(key, parms, data){
	   console.debug("translate enter",[key,parms,data]);
	   var message = this.xsa_messages[key];
	   if (parms){
	       var parm;
	       for (parm in parms){
                if (this.xsa_messages[parms[parm].parmValue]){
                    //look for the message key sub parm to be another message key
                    var name = parms[parm].parmName;
                    var value = this.xsa_messages[parms[parm].parmValue];
                    message = message.replace(name,value);
                }else{
                    //check if it is set in the data
                    if (this.responseData[parms[parm].parmValue]){
						var name2 = parms[parm].parmName;
                        console.debug("name2",[name2]);
                        var value2 = this.xsa_messages[this.responseData[parms[parm].parmValue]];
						console.debug("value2",[value2]);
                        message = message.replace(name2,value2);
                    }
              }
	       }
	   }
	   console.debug("translate exit",[message]);
	   return message;
	}
     
});
