/*
Licensed Materials - Property of IBM
IBM WebSphere CloudBurst Appliance (9235-72X)
Copyright IBM Corporation 2012. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("xsa.messagecenter.MessageCenter");

dojo.require("dojox.grid.EnhancedGrid");
dojo.require("dojox.data.QueryReadStore");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojox.grid.enhanced.plugins.Pagination");
dojo.require("dojox.grid.enhanced.plugins.Filter");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.DropDownButton");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.form.FilteringSelect");
dojo.require("javascript.widgets.ModalDialog");
dojo.require("dijit.form.CheckBox");
dojo.require("javascript.widgets.MessageDialog");
dojo.require("javascript.widgets.XSASingleMessage");
dojo.require("idx.oneui.messaging.SingleMessage");

dojo.requireLocalization("xsa.messagecenter", "xsa_messages");

dojo.declare("xsa.messagecenter.MessageCenter", [dijit._Widget, dijit._Templated],{

	/* holds a reference to localized message bundle */
    xsa_messages : null,

	templatePath: dojo.moduleUrl("xsa","messagecenter/templates/MessageCenterTemplate.html"),
	widgetsInTemplate: true,

	_enhancedGrid: null,
	_refreshButton: null,
	_lastShownTotal : null,
	_checkingForNewMessages : null,
	
	_connections: [],
	_subscribeHandle: null,
	
	_hasError: false,
	
	structure: null,
	dateFormat: {
		formatLength: "medium",
		locale: this.lang
	},
		
	plugins : {
					pagination: {
						pageSizes: ["25", "50", "100", "250"],// Array, custom the items per page button
						description: true,	// boolean, custom whether or not the description will be displayed
						sizeSwitch: true,	// boolean, custom whether or not the page size switch will be displayed
						pageStepper: true,	// boolean, custom whether or not the page step will be displayed
						gotoButton: true,	// boolean, custom whether or not the goto page button will be displayed
						position: "bottom"	// String, custom the position of the pagination bar. There're two options: top, bottom
					},
					filter: {
						isServerSide: true,
						disabledConditions: {string: ["isEmpty", "notEqualTo", "notContains", "notStartsWith", "notEndsWith"], date: ["isEmpty"], anycolumn: []},
						setupFilterQuery: function(commands, request){
							request.query.filterOp="";
							request.query.filterCondition1="";
							request.query.filterCondition2="";
							request.query.filterCondition3="";
							request.query.filterColumn1="";
							request.query.filterColumn2="";
							request.query.filterColumn3="";
							request.query.filterValue1="";
							request.query.filterValue2="";
							request.query.filterValue3="";
							var xsa_messages = dojo.i18n.getLocalization("xsa.messagecenter", "xsa_messages");
							request.query.INFO=xsa_messages.info;
							request.query.WARNING=xsa_messages.warning;
							request.query.SEVERE=xsa_messages.severe;
							
							var addRule = dojo.hitch(this, function(isAnyCol, data, index){
								if(index==1){
									request.query.filterCondition1=data.op;
								} else if(index==2){
									request.query.filterCondition2=data.op;
								} else if(index==3){
									request.query.filterCondition3=data.op;
								}
								dojo.forEach(data.data, function(data){
									if(data.isCol) {
										var columnVal = isAnyCol ? "any" : data.data;
										if(index==1){
											request.query.filterColumn1=columnVal;
										} else if(index==2){
											request.query.filterColumn2=columnVal;
										} else if(index==3){
											request.query.filterColumn3=columnVal;
										}
									} else {
										if(index==1){
											request.query.filterValue1=data.data;
										} else if(index==2){
											request.query.filterValue2=data.data;
										} else if(index==3){
											request.query.filterValue3=data.data;
										}
									}
								}, this);
							});
							
							var addMultipleRules = dojo.hitch(this, function(){
								var index=1;
								dojo.forEach(filter.data, function(data){
									data.op=="any" ? addRule(true, data.data[0], index)
												   : addRule(false, data, index);
									index++;
								}, this);
							});
							
							if(commands.filter){
								var filter = commands.filter;
								if(filter.op == "any"){
									request.query.filterOp = "any";
									if(filter.data.length>3){
										//Any column was selected
										addRule(true, filter.data[0], 1);
									} else {
										addMultipleRules();
									}
								} else if(filter.op == "all"){
									request.query.filterOp = "all";
									addMultipleRules();
								} else {
									request.query.filterOp = "any";
									addRule(false, filter, 1);
								}
							}							
						}
					}
	},
	
	constructor: function(kwArgs) {
		if(!kwArgs) kwArgs = {};
		
		this._lastShownTotal=0;
		this.xsa_messages = dojo.i18n.getLocalization("xsa.messagecenter", "xsa_messages");
		
		this.structure = [{
			cells: [
				{ field: "id", name:"ID", datatype:"string", width: "3%"},
				{ field: "severity", name:this.xsa_messages.type, datatype:"string", disabledConditions:["startsWith", "endsWith"], width: "6%"},
				{ field: "date", name:this.xsa_messages.date, datatype:"date", dataTypeArgs: this.dateFormat, disabledConditions:["isEmpty", "range"], width: "10%"},
				{ field: "source", name:this.xsa_messages.source, datatype:"string", width: "8%"},
				{ field: "message", name:this.xsa_messages.message, datatype:"string", width: "auto"},
				
			]
		}];
		this.pageDescription = this.xsa_messages.messageCenterPageDescription;
	},
				
	createNewMessagesSingleMessage: function(message){
		this.message = new javascript.widgets.XSASingleMessage({
			type: 'information',
			date: new Date(),
			title: message,
			dateFormat:{selector: "time", formatLength: "short"}, 
			showId: false, 
			showRefresh: false, 
			showAction:false,
			showLink:false
		});
		dojo.place(this.message.domNode, dojo.byId("msgNode"), "first");
	},
	
	createErrorMessage: function(data){
		this.message = new javascript.widgets.XSASingleMessage({
			type: 'error',
			date: new Date(),
			title: data.error,
			description: data.desc,
			dateFormat:{selector: "time", formatLength: "short"},
			showId: false, 
			showRefresh: false, 
			showAction:false
		});
		dojo.place(this.message.domNode, dojo.byId("msgNode"), "first");
	},
	
	postCreate: function() {	
		this.inherited(arguments);
		this.createEnhancedGrid();
		this.createButtons();
		
		//fix some a11y bugs 
		var filterTable = dojo.query("table.dojoxGridFBar");
		var footerTable = dojo.query("table.dojoxGridPaginator");
		var numEntriesDiv = dojo.query("div.dojoxGridDescription");
		var gridDiv = dojo.query("div.dojoxGrid");
		if(filterTable) dojo.attr(filterTable[0], "role", "presentation");
		if(footerTable) dojo.attr(footerTable[0], "role", "presentation");
		if(numEntriesDiv) dojo.attr(numEntriesDiv[0], "tabIndex", "0");
		if(gridDiv) dojo.attr(gridDiv[0], "aria-label", this.xsa_messages.messageCenter);
    },
	
	subscribe: function(){
		this._subscribeHandle = dojo.subscribe("TotalMessages", dojo.hitch(this, function(data){
			var numNew = data.totalNumberOfMessages - this._lastShownTotal;
			if(numNew > 0){
				this.destroyMessage();
				var displayMessage = this.xsa_messages.newMessages.replace("${_0_}", numNew);
				this.createNewMessagesSingleMessage(displayMessage);
				this.message.set("title", displayMessage);
				this.message.set("date", new Date());
			}
		}));
	},
		
	createButtons: function() {
		this._refreshButton = new dijit.form.Button({
			iconClass: "refreshIcon",
			showLabel: false,
			label:  this.xsa_messages.refresh,
			type: "button",
			onClick: dojo.hitch(this, function(){
				this._enhancedGrid.setQuery({action: "getMessages"});	
			})
		}, this._refreshButton);
	},
	
	createEnhancedGrid: function(){
		var gridStore = new dojox.data.QueryReadStore({
			url: "/resources/message_center"
		});
		
		gridStore._filterResponse = dojo.hitch(this, function(data){ //determine if the table is not displaying all of the matches
			this._hasError = false;
			if(data.lastShownTotal != undefined) {
				this._lastShownTotal = data.lastShownTotal;
			} 
			if(!this._subscribeHandle) this.subscribe();	
			dojo.forEach(data.items, function(item){
				var date = new Date(parseInt(item.time));
				item.date = dojo.date.locale.format(date, this.dateFormat);
			}, this);
			
			if(data.error) {
				this._hasError = true;
				this.destroyMessage();
				this.createErrorMessage(data);
				this._lastShownTotal=0;
			}
			return data;
		});
		
		this._enhancedGrid = new dojox.grid.EnhancedGrid({
			plugins:this.plugins, 
			structure: this.structure, 
			store: gridStore, 
			query: {query: ".*"},
			selectable:true
		}, this._enhancedGridNode);
		this._enhancedGrid.startup();

		this._connections.push(dojo.connect(this._enhancedGrid, "_onFetchComplete", this, this.onGridReloadComplete));	

		//Transform the type field into images
        this.connect(this._enhancedGrid.views.views[0], "onAfterRow", dojo.hitch(this, function(inRowIndex, cells, inRowNode){
            var tableRow = inRowNode.firstChild.firstChild.firstChild;

            dojo.forEach(tableRow.cells, dojo.hitch(this, dojo.hitch(this, function(tableRow, headerCells, cell, index){
                if (headerCells[index].field === "severity") {
                    var type = cell.innerHTML;
                    if (type === "INFO") {
						dojo.attr(cell, "innerHTML", ""); //remove the current cell content
                        dojo.create("div", {
                            'class': 'infoIcon dijitInline',
                            title: this.xsa_messages.info
                        }, cell);
						dojo.create("div", {
                            'class': 'dijitInline iconLabel',
                            title: this.xsa_messages.info,
							innerHTML: this.xsa_messages.info
                        }, cell);
                    } else if (type === "WARNING") {
						dojo.attr(cell, "innerHTML", ""); //remove the current cell content
                        dojo.create("div", {
                            'class': 'warningIcon dijitInline',
                            title: this.xsa_messages.warning
                        }, cell);
						dojo.create("div", {
                            'class': 'dijitInline iconLabel',
                            title: this.xsa_messages.warning,
							innerHTML: this.xsa_messages.warning
                        }, cell);
                    } else if (type === "SEVERE") {
						dojo.attr(cell, "innerHTML", ""); //remove the current cell content
                        dojo.create("div", {
                            'class': 'severeIcon dijitInline',
                            title: this.xsa_messages.severe
                        }, cell);
						dojo.create("div", {
                            'class': 'dijitInline iconLabel',
                            title: this.xsa_messages.severe,
							innerHTML: this.xsa_messages.severe
                        }, cell);
						
                    } 
				} else  if (headerCells[index].field === "message") {
					if(cell.nodeName != "TH") { //exclude the header cell
						//var s = new String(cell.innerHTML);
						//cell.innerHTML = s.replace("\;\g", "<br>");
					}
				}
            }, tableRow, cells[0])));
                
            
        }));
	},
	
	onGridReloadComplete: function(){
		if(!this._hasError) this.destroyMessage();
	},
	
	destroyMessage: function(){
		if(this.message) {
			this.message.destroy();
			this.message = null;
		}
	},
	
	destroy: function(){
		//Cleanup connections
		dojo.forEach(this._connections, dojo.disconnect);
		if(this._subscribeHandle) dojo.unsubscribe(this._subscribeHandle);
		
		//Cleanup widgets
		if(this._enhancedGrid) this._enhancedGrid.destroyRecursive(true);
		if(this._refreshButton) this._refreshButton.destroyRecursive(true);
		
		this.destroyMessage();
		
		delete this._enhancedGrid;
		delete this._refreshButton;
				
		this.inherited(arguments);
	}
	 	
});