import com.ibm.websphere.rainmaker.logging.CleanStackTrace;
import com.ibm.websphere.rainmaker.logging.Output;
import com.ibm.ws.xsa.datastores.AppliancePersistentStoreConstants;

def root = new File(config.rainmaker.disk.root[]);

if("".equals(config.rainmaker.appName[])){
	if (!root.exists() && onAppliance())
	    Output.fail("Hard drive is not mounted.");
}
else{		// only create the base drouter directory and not children.
	if (!root.getParentFile().exists() && onAppliance())
	    Output.fail("Hard drive is not mounted.");
}

//zdelete("/appliance/xsa/persistence", true);

config.rainmaker.directories[].each {
    def subDir = new File(root, it);
    
    if (subDir.exists())
        return;
    
    Output.info("Creating directory ${subDir.absolutePath}...");
    
    if (!subDir.mkdirs())
        Output.fail("Unable to create directory ${subDir.absolutePath}.");
    
    if (subDir.exists()) {
        Output.info("Created directory ${subDir.absolutePath}...");
    }
    else {
        Output.fail("Failed to create ${subDir.absolutePath}...");
    }
}

def target = new File(config.root[]);  
def properties = getResolvedProperties(target);  

// generate secret key per appliance for security
if(properties["ibm:rainmaker.rest.pd"]){
	
	def secretKeyConfigFile = new File (config.rainmaker.disk.shared[], "/config/security-secretkey.config")
	Output.info("secretKeyFile " + secretKeyConfigFile);
	if(!config.security.secretKey[]){
		if(!secretKeyConfigFile.exists()){
			new File (secretKeyConfigFile.getParent()).mkdirs();	//create directories if needed
			secretKeyConfigFile.createNewFile();	// create file
		}
		def secretKey = zero.core.security.encryption.SecretKeyGenerator.generateSecretKey()
		Output.info ("Creating secretKey config");
		secretKeyConfigFile.write ("/config/security/secretKey=\"${secretKey}\"")

	}
}	

if(config.rainmaker.setup.skipdbconfig[])
    return;

def targetVersion = null;

if (!event.args[].isEmpty())
{
    targetVersion = event.args[][0];
    
    if (!(targetVersion ==~ /^\d+\.\d+\.\d+\.\d+$/))
        Output.fail("The target version must be of the form w.x.y.z (all numeric). You provided ${targetVersion}.");
}

if (!targetVersion){
    targetVersion = config.rainmaker.version[].tokenize('-')[0];
    if (!(targetVersion ==~ /^\d+\.\d+\.\d+\.\d+$/)){
    	if ((targetVersion ==~ /^\d+\.\d+\.\d+$/)){
    		targetVersion = targetVersion + '.0';
    	}
    }
}

Output.info("The target version is ${targetVersion}.");

def currentVersion = null;
def dbDir = config.rainmaker.disk.databases[]
if(zcontains(AppliancePersistentStoreConstants.DEFAULT_ZONE_PREFIX + "/version")){
    currentVersion = zget(AppliancePersistentStoreConstants.DEFAULT_ZONE_PREFIX + "/version");
}

def noPreviousSetup = currentVersion == null;

if (currentVersion && !(currentVersion ==~ /^\d+\.\d+\.\d+\.\d+$/))
    Output.fail("The current version has an invalid format - the file at ${versionFile.absolutePath} may have been corrupted.");

else
    Output.info("The current version is ${currentVersion ?: '0.0.0.0 (setup has not been run)'}.");

if (noPreviousSetup){
    currentVersion = '0.0.0.0';
}

if (currentVersion == targetVersion)
    Output.exit('Target version is the same as the current version - no work will be done.');

def names = properties['dependencies'].tokenize(',').reverse();
names << "_self";

if (noPreviousSetup && properties['ibm:rainmaker.users'])
{
    try
    {
        config.rainmaker.alwaysJoinThreads = true;
        
        Output.info('Creating all-inclusive public group...');        
        createEveryoneGroup();

        Output.info('The public group was created successfully.');
        
        Output.info('Creating admin account...');        
        createAdmin();
        Output.info('The admin account was created successfully.');
        
        Output.info('Creating adminIdentityUser account...');        
        invokeMethod("groovy/xsa/users/AdminIdentityUser.groovy", "createAdminIdentityUser", null);
        Output.info('The adminIdentityUser account was created successfully.');

    }
    
    catch (Throwable t)
    {
        Output.error(CleanStackTrace.cleanStackTrace(t));
        Output.fail("Could not create default data: ${t.message}");
    }
}

zput(AppliancePersistentStoreConstants.DEFAULT_ZONE_PREFIX +"/version", targetVersion);

Output.info('Setup is complete.');

Output.info("Dumping the persistence zone for verification....");
Output.info(zdump("/appliance/xsa/persistence"));

